@@ -71,7 +71,7 @@ typedef int socklen_t;
 #endif   // !WIN32
 
 #ifndef INADDR_LOOPBACK
-#define INADDR_LOOPBACK (unsigned long)0x7f000001
+#define INADDR_LOOPBACK (in_addr_t)0x7f000001
 #endif
 
 #ifdef HAVE_NETINET_IN_H
@@ -240,7 +240,7 @@ Socket::Error Socket::error(Error err, const char *errs, long systemError) const
 }
 
 #ifdef _MSWINDOWS_
-Socket::Error Socket::connectError(void)
+Socket::Error Socket::connectError(void) const
 {
     const char* str = "Could not connect to remote host";
     switch(WSAGetLastError()) {
@@ -259,7 +259,7 @@ Socket::Error Socket::connectError(void)
     }
 }
 #else
-Socket::Error Socket::connectError(void)
+Socket::Error Socket::connectError(void) const
 {
     char* str = (char *)"Could not connect to remote host";
     switch(errno) {
@@ -433,7 +433,7 @@ Socket::Error Socket::setMulticastByFamily(bool enable, Family family)
     }
 }
 
-Socket::Error Socket::setTimeToLiveByFamily(unsigned char ttl, Family fam)
+Socket::Error Socket::setTimeToLiveByFamily(uint8_t ttl, Family fam)
 {
     if(!flags.multicast)
         return error(errMulticastDisabled,(char *)"Multicast not enabled on socket");
@@ -458,7 +458,7 @@ Socket::Error Socket::setTimeToLiveByFamily(unsigned char ttl, Family fam)
 
 Socket::Error Socket::setLoopbackByFamily(bool enable, Family family)
 {
-    unsigned char loop;
+    uint8_t loop;
 
     if(!flags.multicast)
         return error(errMulticastDisabled,(char *)"Multicast not enabled on socket");
@@ -864,7 +864,7 @@ ucommon::Socket::address Socket::getSender() const
     return addr;
 }
 
-IPV4Host Socket::getIPV4Sender(tpport_t *port) const
+IPV4Host Socket::getIPV4Sender(in_port_t *port) const
 {
     ucommon::Socket::address addr = getSender();
     sockaddr_in* from = addr;
@@ -883,7 +883,7 @@ IPV4Host Socket::getIPV4Sender(tpport_t *port) const
 }
 
 #ifdef  CCXX_IPV6
-IPV6Host Socket::getIPV6Sender(tpport_t *port) const
+IPV6Host Socket::getIPV6Sender(in_port_t *port) const
 {
     ucommon::Socket::address addr = getSender();
     sockaddr_in6* from = addr;
@@ -913,9 +913,9 @@ ucommon::Socket::address Socket::getLocal() const
     return saddr;
 }
 
-IPV4Host Socket::getIPV4Local(tpport_t *port) const
+IPV4Host Socket::getIPV4Local(in_port_t *port) const
 {
-    sockaddr_in* from = getLocal();
+    const sockaddr_in* from = getLocal();
     struct in_addr any;
     any.s_addr = INADDR_ANY;
 
@@ -931,9 +931,9 @@ IPV4Host Socket::getIPV4Local(tpport_t *port) const
 }
 
 #ifdef  CCXX_IPV6
-IPV6Host Socket::getIPV6Local(tpport_t *port) const
+IPV6Host Socket::getIPV6Local(in_port_t *port) const
 {
-    sockaddr_in6* from = getLocal();
+    const sockaddr_in6* from = getLocal();
     if (from == NULL) {
         if (port)
             *port = 0;
@@ -966,7 +966,7 @@ ucommon::Socket::address Socket::getPeer() const
     return saddr;
 }
 
-IPV4Host Socket::getIPV4Peer(tpport_t *port) const
+IPV4Host Socket::getIPV4Peer(in_port_t *port) const
 {
     sockaddr_in* from = getPeer();
     struct in_addr any;
@@ -984,7 +984,7 @@ IPV4Host Socket::getIPV4Peer(tpport_t *port) const
 }
 
 #ifdef  CCXX_IPV6
-IPV6Host Socket::getIPV6Peer(tpport_t *port) const
+IPV6Host Socket::getIPV6Peer(in_port_t *port) const
 {
     sockaddr_in6* from = getPeer();
     if (from == NULL) {
@@ -1057,7 +1057,7 @@ Socket::Error Socket::setLinger(bool linger)
 Socket::Error Socket::setTypeOfService(Tos service)
 {
 #ifdef  SOL_IP
-    unsigned char tos;
+    uint8_t tos;
     switch(service) {
 #ifdef  IPTOS_LOWDELAY
     case tosLowDelay:
