@@ -22,36 +22,32 @@
 
 namespace ucommon {
 
-UnlockAccess::~UnlockAccess()
+SharedProtocol::~SharedProtocol()
 {
 }
 
-SharedAccess::~SharedAccess()
+ExclusiveProtocol::~ExclusiveProtocol()
 {
 }
 
-ExclusiveAccess::~ExclusiveAccess()
+void SharedProtocol::exclusive(void)
 {
 }
 
-void SharedAccess::exclusive(void)
+void SharedProtocol::share(void)
 {
 }
 
-void SharedAccess::share(void)
-{
-}
-
-shared_access::shared_access(SharedAccess *obj)
+SharedProtocol::Locking::Locking(SharedProtocol *obj)
 {
     assert(obj != NULL);
     lock = obj;
     modify = false;
     state = 0;
-    lock->shared_lock();
+    lock->_share();
 }
 
-shared_access::shared_access(const shared_access& copy)
+SharedProtocol::Locking::Locking(const Locking& copy)
 {
     assert(copy.modify == false);
 
@@ -59,10 +55,10 @@ shared_access::shared_access(const shared_access& copy)
     modify = false;
     state = 0;
     if(lock)           
-        lock->shared_lock();
+        lock->_share();
 }
 
-shared_access& shared_access::operator=(const shared_access& copy)
+SharedProtocol::Locking& SharedProtocol::Locking::operator=(const Locking& copy)
 {
     assert(copy.modify == false);
 
@@ -70,57 +66,57 @@ shared_access& shared_access::operator=(const shared_access& copy)
     lock = copy.lock;
     state = 0;
     if(lock)
-        lock->shared_lock();
+        lock->_share();
 
     return *this;
 }
 
-exclusive_access::exclusive_access(ExclusiveAccess *obj)
+ExclusiveProtocol::Locking::Locking(ExclusiveProtocol *obj)
 {
     assert(obj != NULL);
     lock = obj;
-    lock->exclusive_lock();
+    lock->_lock();
 }
 
-shared_access::~shared_access()
+SharedProtocol::Locking::~Locking()
 {
     if(lock) {
         if(modify)
             lock->share();
-        lock->release_share();
+        lock->_unshare();
         lock = NULL;
         modify = false;
     }
 }
 
-exclusive_access::~exclusive_access()
+ExclusiveProtocol::Locking::~Locking()
 {
     if(lock) {
-        lock->release_exclusive();
+        lock->_unlock();
         lock = NULL;
     }
 }
 
-void shared_access::release()
+void SharedProtocol::Locking::release()
 {
     if(lock) {
         if(modify)
             lock->share();
-        lock->release_share();
+        lock->_unshare();
         lock = NULL;
         modify = false;
     }
 }
 
-void exclusive_access::release()
+void ExclusiveProtocol::Locking::release()
 {
     if(lock) {
-        lock->release_exclusive();
+        lock->_unlock();
         lock = NULL;
     }
 }
 
-void shared_access::exclusive(void)
+void SharedProtocol::Locking::exclusive(void)
 {
     if(lock && !modify) {
         lock->exclusive();
@@ -128,7 +124,7 @@ void shared_access::exclusive(void)
     }
 }
 
-void shared_access::share(void)
+void SharedProtocol::Locking::share(void)
 {
     if(lock && modify) {
         lock->share();
