@@ -221,7 +221,7 @@ ArrayRef::Array *ArrayRef::create(arraytype_t mode, size_t size)
         return NULL;
 
     size_t s = sizeof(Array) + (size * sizeof(Counted *));
-    caddr_t p = TypeRef::alloc(s);
+    caddr_t p = auto_release.allocate(s);
     return new(mem(p)) Array(mode, p, size);
 }
 
@@ -428,8 +428,9 @@ TypeRef::Counted *ArrayRef::get(size_t index)
     if(!array)
         return NULL;
 
-    if(index >= array->size || array->head == array->tail)
+    if(index >= array->size || array->head == array->tail) {
         return NULL;
+	}
 
     array->lock();
     index += array->head;
