@@ -38,7 +38,32 @@
 #include <limits.h>
 
 #ifdef  _MSWINDOWS_
-int cpr_setenv(const char *sym, const char *val, int flag)
+#ifdef  HAVE_SYS_TIMEB_H
+#include <sys/timeb.h>
+#endif
+
+int gettimeofday(struct timeval *tv_, void *tz_)
+{
+    assert(tv_ != NULL);
+
+#if defined(_MSC_VER) && _MSC_VER >= 1300
+    struct __timeb64 tb;
+    _ftime64(&tb);
+#else
+# ifndef __BORLANDC__
+    struct _timeb tb;
+    _ftime(&tb);
+# else
+    struct timeb tb;
+    ftime(&tb);
+# endif
+#endif
+    tv_->tv_sec = (long)tb.time;
+    tv_->tv_usec = tb.millitm * 1000;
+    return 0;
+}
+
+int setenv(const char *sym, const char *val, int flag)
 {
     char buf[128];
 
@@ -65,6 +90,28 @@ void cpr_runtime_error(const char *str)
     abort();
 }
 
+#if !defined(_MSWINDOWS_) && !defined(__QNX__)
+
+extern "C" int stricmp(const char *s1, const char *s2)
+{
+#ifdef  HAVE_STRICMP
+    return stricmp(s1, s2);
+#else
+    return strcasecmp(s1, s2);
+#endif
+}
+
+extern "C" int strnicmp(const char *s1, const char *s2, size_t size)
+{
+#ifdef  HAVE_STRICMP
+    return strnicmp(s1, s2, size);
+#else
+    return strncasecmp(s1, s2, size);
+#endif
+}
+
+#endif
+
 // just become we need to get binary types in a specific binary endian order.
 
 extern "C" uint16_t lsb_getshort(uint8_t *b)
@@ -167,14 +214,14 @@ extern "C" void *cpr_memalloc(size_t size)
         ++size;
 
     mem = malloc(size);
-    crit(mem != NULL, "memory alloc failed");
+    assert(mem != NULL);
     return mem;
 }
 
 extern "C" void *cpr_memassign(size_t size, caddr_t addr, size_t max)
 {
     assert(addr);
-    crit((size <= max), "memory assign failed");
+    assert(size <= max);
     return addr;
 }
 
@@ -231,6 +278,28 @@ void operator delete[](void *array) noexcept(true)
     free(array);
 }
 
+extern "C" {
+    long tzoffset(struct timezone *tz)
+    {
+        struct timeval now;
+        time_t t1, t2 = 0;
+        struct tm t;
+        
+        gettimeofday(&now, tz);
+        t1 = now.tv_sec;
+
+#ifdef  HAVE_GMTIME_R
+        gmtime_r(&t1, &t);
+#else
+        t = *gmtime(&t1);
+#endif
+
+        t.tm_isdst = 0;
+        t2 = mktime(&t);
+        return (long)difftime(t1, t2);
+    } 
+}
+
 #endif
 
 
