@@ -296,14 +296,12 @@ unsigned Date::dow(void) const
     return (unsigned)((julian + 1l) % 7l);
 }
 
-String Date::operator()() const
+stringref_t Date::operator()() const
 {
     char buf[11];
 
     put(buf);
-    String date(buf);
-
-    return date;
+    return stringref_t(buf);
 }
 
 long Date::get(void) const
@@ -313,7 +311,7 @@ long Date::get(void) const
     return atol(buf) * 10000 + atol(buf + 5) * 100 + atol(buf + 8);
 }
 
-Date& Date::operator++()
+Date& Date::operator++() 
 {
     ++julian;
     update();
@@ -327,14 +325,14 @@ Date& Date::operator--()
     return *this;
 }
 
-Date Date::operator+(long val)
+const Date Date::operator+(long val) const
 {
     Date result = *this;
     result += val;
     return result;
 }
 
-Date Date::operator-(long val)
+const Date Date::operator-(long val) const
 {
     Date result = *this;
     result -= val;
@@ -543,14 +541,12 @@ void Time::set(const char *str, size_t size)
     set(nhour(), nminute(), sec);
 }
 
-String Time::operator()() const
+stringref_t Time::operator()() const
 {
     char buf[9];
 
     put(buf);
-    String strTime(buf);
-
-    return strTime;
+    return stringref_t(buf);
 }
 
 long Time::get(void) const
@@ -586,14 +582,14 @@ Time& Time::operator-=(long val)
     return *this;
 }
 
-Time Time::operator+(long val)
+const Time Time::operator+(long val) const
 {
     Time result = *this;
     result += val;
     return result;
 }
 
-Time Time::operator-(long val)
+const Time Time::operator-(long val) const
 {
     Time result = *this;
     result -= val;
@@ -872,13 +868,12 @@ bool DateTime::operator!() const
 }
 
 
-String DateTime::format(const char *text) const
+stringref_t DateTime::format(const char *text) const
 {
     char buffer[64];
     size_t last;
     time_t t;
     tm_t *tbp;
-    String retval;
 
     t = get();
     tbp = local(&t);
@@ -886,8 +881,7 @@ String DateTime::format(const char *text) const
     release(tbp);
 
     buffer[last] = '\0';
-    retval = buffer;
-    return retval;
+    return stringref_t(buffer);
 }
 
 long DateTime::operator-(const DateTime &dt)
@@ -897,14 +891,14 @@ long DateTime::operator-(const DateTime &dt)
     return secs;
 }
 
-DateTime DateTime::operator+(long value)
+const DateTime DateTime::operator+(long value) const
 {
     DateTime result = *this;
     result += value;
     return result;
 }
 
-DateTime DateTime::operator-(long value)
+const DateTime DateTime::operator-(long value) const
 {
     DateTime result = *this;
     result -= value;
@@ -1093,26 +1087,4 @@ final:
     return code;
 }
 
-extern "C" {
-    long tzoffset(struct timezone *tz)
-    {
-        struct timeval now;
-        time_t t1, t2 = 0;
-        struct tm t;
-        
-        gettimeofday(&now, tz);
-        t1 = now.tv_sec;
-
-#ifdef  HAVE_GMTIME_R
-        gmtime_r(&t1, &t);
-#else
-        t = *gmtime(&t1);
-#endif
-
-        t.tm_isdst = 0;
-        t2 = mktime(&t);
-        return (long)difftime(t1, t2);
-    } 
-}
-
 } // namespace ucommon
