@@ -671,14 +671,14 @@ int fsys::inherit(fd_t& from, bool enable)
     return remapError();
 }
 
-void fsys::operator=(fd_t from)
+fsys& fsys::operator=(fd_t from)
 {
     HANDLE pHandle = GetCurrentProcess();
 
     if(fd != INVALID_HANDLE_VALUE) {
         if(!CloseHandle(fd)) {
             error = remapError();
-            return;
+            return *this;
         }
     }
     if(DuplicateHandle(pHandle, from, pHandle, &fd, 0, FALSE, DUPLICATE_SAME_ACCESS))
@@ -687,16 +687,17 @@ void fsys::operator=(fd_t from)
         fd = INVALID_HANDLE_VALUE;
         error = remapError();
     }
+	return *this;
 }
 
-void fsys::operator=(const fsys& from)
+fsys& fsys::operator=(const fsys& from)
 {
     HANDLE pHandle = GetCurrentProcess();
 
     if(fd != INVALID_HANDLE_VALUE) {
         if(!CloseHandle(fd)) {
             error = remapError();
-            return;
+            return *this;
         }
     }
     if(DuplicateHandle(pHandle, from.fd, pHandle, &fd, 0, FALSE, DUPLICATE_SAME_ACCESS))
@@ -705,6 +706,7 @@ void fsys::operator=(const fsys& from)
         fd = INVALID_HANDLE_VALUE;
         error = remapError();
     }
+	return *this;
 }
 
 int fsys::drop(offset_t size)
@@ -747,11 +749,7 @@ ssize_t dir::read(char *buf, size_t len)
 
 ssize_t fsys::read(void *buf, size_t len)
 {
-#ifdef  __PTH__
-    int rtn = ::pth_read(fd, buf, len);
-#else
     int rtn = ::read(fd, buf, len);
-#endif
 
     if(rtn < 0)
         error = remapError();
@@ -770,11 +768,7 @@ int fsys::sync(void)
 
 ssize_t fsys::write(const void *buf, size_t len)
 {
-#ifdef  __PTH__
-    int rtn = pth_write(fd, buf, len);
-#else
     int rtn = ::write(fd, buf, len);
-#endif
 
     if(rtn < 0)
         error = remapError();
@@ -1091,7 +1085,7 @@ int fsys::inherit(fd_t& fd, bool enable)
     return 0;
 }
 
-void fsys::operator=(fd_t from)
+fsys& fsys::operator=(fd_t from)
 {
     close();
     if(fd == INVALID_HANDLE_VALUE && from != INVALID_HANDLE_VALUE) {
@@ -1099,9 +1093,10 @@ void fsys::operator=(fd_t from)
         if(fd == INVALID_HANDLE_VALUE)
             error = remapError();
     }
+	return *this;
 }
 
-void fsys::operator=(const fsys& from)
+fsys& fsys::operator=(const fsys& from)
 {
     close();
     if(fd == INVALID_HANDLE_VALUE && from.fd != INVALID_HANDLE_VALUE) {
@@ -1109,6 +1104,7 @@ void fsys::operator=(const fsys& from)
         if(fd == INVALID_HANDLE_VALUE)
             error = remapError();
     }
+	return *this;
 }
 
 int fsys::drop(offset_t size)
@@ -1202,12 +1198,13 @@ fsys::~fsys()
     close();
 }
 
-void fsys::operator*=(fd_t& from)
+fsys& fsys::operator*=(fd_t& from)
 {
     if(fd != INVALID_HANDLE_VALUE)
         close();
     fd = from;
     from = INVALID_HANDLE_VALUE;
+	return *this;
 }
 
 int fsys::linkinfo(const char *path, char *buffer, size_t size)
@@ -1792,10 +1789,10 @@ int fsys::exec(const char *path, char **argv, char **envp)
 
 #ifdef _MSWINDOWS_
 
-static stringref_t sysprefix(void)
+stringref_t fsys::prefix(void)
 {
     char *cp = _getcwd(NULL, 0);
-    stringref result(cp);
+    stringref_t result(cp);
     if(cp)
         ::free(cp);
     return result;
@@ -1803,10 +1800,10 @@ static stringref_t sysprefix(void)
 
 #else
 
-static stringref_t sysprefix(void)
+stringref_t fsys::prefix(void)
 {
     size_t size = 40;
-    charvalues_t buf = stringref::create(40);
+    charvalues_t buf = stringref<auto_release>::create(40);
     stringref_t out;
     for(;;) {
         if(NULL != (getcwd(buf->get(), buf->max())))
@@ -1815,7 +1812,7 @@ static stringref_t sysprefix(void)
             *(buf->get()) = 0;
             break;
         }
-        stringref::expand(&buf, size);
+        stringref<auto_release>::expand(&buf, size);
         size += 40;
     }
     out.assign(buf);
@@ -1823,12 +1820,5 @@ static stringref_t sysprefix(void)
 }
 
 #endif
-
-string_t fsys::prefix(void)
-{
-    stringref_t sys = sysprefix();
-    string_t out = *sys;
-    return out;
-}
-        
+ 
 } // namespace ucommon
