@@ -62,7 +62,7 @@ const char *keydata::get(const char *key) const
 {
     assert(key != NULL);
 
-    iterator keys = begin();
+    keydata::pointer keys = begin();
 
     while(is(keys)) {
         if(eq_case(key, keys->id))
@@ -76,7 +76,7 @@ void keydata::clear(const char *key)
 {
     assert(key != NULL);
 
-    iterator keys = begin();
+    keydata::pointer keys = begin();
 
     while(is(keys)) {
         if(eq_case(key, keys->id)) {
@@ -92,7 +92,7 @@ void keydata::set(const char *key, const char *value)
     assert(key != NULL);
 
     void *mem = root->alloc(sizeof(keydata::keyvalue));
-    keydata::iterator keys = begin();
+    keydata::pointer keys = begin();
 
     while(is(keys)) {
         if(eq_case(key, keys->id)) {
@@ -151,7 +151,7 @@ keydata *keyfile::get(const char *key) const
 {
     assert(key != NULL);
 
-    iterator keys = begin();
+    keyfile::pointer keys = begin();
 
     while(is(keys)) {
         if(eq_case(key, keys->name))
@@ -241,8 +241,8 @@ void keyfile::load(HKEY keys, keydata *section, const char *path)
     }
     index = 0;
     vsize = sizeof(keyvalue);
-    if(vsize > size() - 64)
-        vsize = size() - 64;
+    if(vsize > (DWORD)(size() - 64))
+        vsize = (DWORD)(size() - 64);
     while((RegEnumValue(keys, index++, keyname, &ksize, NULL, &vtype, (BYTE *)keyvalue, &vsize) == ERROR_SUCCESS) && (vtype == REG_SZ) && (keyname[0] != 0)) {
         if(section)
             section->set(keyname, keyvalue);
@@ -250,8 +250,8 @@ void keyfile::load(HKEY keys, keydata *section, const char *path)
             defaults->set(keyname, keyvalue);
         ksize = sizeof(keyname);
         vsize = sizeof(keyvalue);
-        if(vsize > size() - 64)
-            vsize = size() - 64;
+        if(vsize > (DWORD)(size() - 64))
+            vsize = (DWORD)(size() - 64);
     }
 }
 
