@@ -24,158 +24,9 @@
 
 namespace ucommon {
 
-// I am not sure if we ever use these...seemed like a good idea at the time
-const LinkedObject *LinkedObject::nil = (LinkedObject *)NULL;
-const LinkedObject *LinkedObject::inv = (LinkedObject *)-1;
-
-MultiMap::MultiMap(unsigned count) : ReusableObject()
-{
-    assert(count > 0);
-
-    paths = count;
-    links = new link_t[count];
-    // just a cheap way to initially set the links to all NULL...
-    memset(links, 0, sizeof(link_t) * count);
-}
-
-MultiMap::~MultiMap()
-{
-    unsigned path = 0;
-
-    while(path < paths)
-        delist(path++);
-
-    delete[] links;
-}
-
-MultiMap *MultiMap::next(unsigned path) const
-{
-    assert(path < paths);
-
-    return links[path].next;
-}
-
-void MultiMap::delist(unsigned path)
-{
-    assert(path < paths);
-
-    if(!links[path].root)
-        return;
-
-    while(links[path].root) {
-        if(*links[path].root == this) {
-            *links[path].root = next(path);
-            break;
-        }
-		if (links[path].root && *links[path].root)
-			links[path].root = &((*links[path].root)->links[path].next);
-    }
-    links[path].root = NULL;
-    links[path].next = NULL;
-}
-
-void MultiMap::enlist(unsigned path, MultiMap **root)
-{
-    assert(path < paths);
-    assert(root != NULL);
-
-    delist(path);
-    links[path].next = *root;
-    links[path].root = root;
-    links[path].key = NULL;
-    links[path].keysize = 0;
-    *root = this;
-}
-
-void MultiMap::enlist(unsigned path, MultiMap **root, caddr_t key, unsigned max, size_t keysize)
-{
-    assert(path < paths);
-    assert(root != NULL);
-    assert(key != NULL);
-    assert(max > 0);
-
-    unsigned value = 0;
-
-    delist(path);
-    while(keysize && !key[0]) {
-        ++key;
-        --keysize;
-    }
-
-    while(keysize--)
-        value = (value << 1) ^ (*key++);
-
-    enlist(path, &root[keyindex(key, max, keysize)]);
-
-    if(!keysize)
-        keysize = strlen(key);
-
-    links[path].keysize = keysize;
-    links[path].key = key;
-}
-
-bool MultiMap::equal(unsigned path, caddr_t key, size_t keysize) const
-{
-    assert(path < paths);
-    assert(key != NULL);
-
-    if(!keysize)
-        keysize = strlen(key);
-
-    if(links[path].keysize != keysize)
-        return false;
-
-    if(memcmp(key, links[path].key, links[path].keysize) == 0)
-        return true;
-
-    return false;
-}
-
-// this transforms either strings or binary fields (such as addresses)
-// into a hash index.
-unsigned MultiMap::keyindex(caddr_t key, unsigned max, size_t keysize)
-{
-    assert(key != NULL);
-    assert(max > 0);
-
-    unsigned value = 0;
-
-    // if we are a string, we can just used our generic text hasher
-    if(!keysize)
-        return NamedObject::keyindex(key, max);
-
-    // strip off lead 0's...just saves time for data that is not changing,
-    // especially when little endian 64 bit wide...
-    while(keysize && !key[0]) {
-        ++key;
-        --keysize;
-    }
-
-    while(keysize--)
-        value = (value << 1) ^ (*key++);
-
-    return value % max;
-}
-
-MultiMap *MultiMap::find(unsigned path, MultiMap **root, caddr_t key, unsigned max, size_t keysize)
-{
-    assert(key != NULL);
-    assert(max > 0);
-
-    MultiMap *node = root[keyindex(key, max, keysize)];
-
-    while(node) {
-        if(node->equal(path, key, keysize))
-            break;
-        node = node->next(path);
-    }
-
-    return node;
-}
-
 LinkedObject::LinkedObject(LinkedObject **root)
 {
-    assert(root != NULL);
+    assert(root != nullptr);
     enlist(root);
 }
 
@@ -187,7 +38,7 @@ void LinkedObject::purge(LinkedObject *root)
 {
     LinkedObject *after;
 
-    assert(root != NULL);
+    assert(root != nullptr);
 
     while(root) {
         after = root->Next;
@@ -198,7 +49,7 @@ void LinkedObject::purge(LinkedObject *root)
 
 bool LinkedObject::is_member(LinkedObject *list) const
 {
-    assert(list != NULL);
+    assert(list != nullptr);
 
     while(list) {
         if(list == this)
@@ -210,7 +61,7 @@ bool LinkedObject::is_member(LinkedObject *list) const
 
 void LinkedObject::enlist(LinkedObject **root)
 {
-    assert(root != NULL);
+    assert(root != nullptr);
 
     Next = *root;
     *root = this;
@@ -218,9 +69,9 @@ void LinkedObject::enlist(LinkedObject **root)
 
 void LinkedObject::delist(LinkedObject **root)
 {
-    assert(root != NULL);
+    assert(root != nullptr);
 
-    LinkedObject *prior = NULL, *node = *root;
+    LinkedObject *prior = nullptr, *node = *root;
 
     while(node && node != this) {
         prior = node;
@@ -238,22 +89,22 @@ void LinkedObject::delist(LinkedObject **root)
 
 void ReusableObject::release(void)
 {
-    Next = (LinkedObject *)nil;
+    Next = nullptr;
 }
 
 NamedObject::NamedObject() :
 OrderedObject()
 {
-    Id = NULL;
+    Id = nullptr;
 }
 
 NamedObject::NamedObject(OrderedIndex *root, char *nid) :
 OrderedObject()
 {
-    assert(root != NULL);
-    assert(nid != NULL && *nid != 0);
+    assert(root != nullptr);
+    assert(nid != nullptr && *nid != 0);
 
-    NamedObject *node = static_cast<NamedObject*>(root->head), *prior = NULL;
+    NamedObject *node = static_cast<NamedObject*>(root->head), *prior = nullptr;
 
     while(node) {
         if(node->equal(nid)) {
@@ -267,7 +118,7 @@ OrderedObject()
         prior = node;
         node = node->getNext();
     }
-    Next = NULL;
+    Next = nullptr;
     Id = nid;
     if(!root->head)
         root->head = this;
@@ -284,23 +135,23 @@ OrderedObject()
 NamedObject::NamedObject(NamedObject **root, char *nid, unsigned max) :
 OrderedObject()
 {
-    assert(root != NULL);
-    assert(nid != NULL && *nid != 0);
+    assert(root != nullptr);
+    assert(nid != nullptr && *nid != 0);
     assert(max > 0);
 
-    Id = NULL;
+    Id = nullptr;
     add(root, nid, max);
 }
 
 void NamedObject::add(NamedObject **root, char *nid, unsigned max)
 {
-    assert(root != NULL);
-    assert(nid != NULL && *nid != 0);
+    assert(root != nullptr);
+    assert(nid != nullptr && *nid != 0);
     assert(max > 0);
 
     clearId();
 
-    NamedObject *node, *prior = NULL;
+    NamedObject *node, *prior = nullptr;
 
     if(max < 2)
         max = 0;
@@ -334,7 +185,7 @@ void NamedObject::clearId(void)
 {
     if(Id) {
         free(Id);
-        Id = NULL;
+        Id = nullptr;
     }
 }
 
@@ -366,7 +217,7 @@ void LinkedObject::release(void)
 
 LinkedObject *LinkedObject::getIndexed(LinkedObject *root, unsigned index)
 {
-    while(index-- && root != NULL)
+    while(index-- && root != nullptr)
         root = root->Next;
 
     return root;
@@ -374,7 +225,7 @@ LinkedObject *LinkedObject::getIndexed(LinkedObject *root, unsigned index)
 
 unsigned LinkedObject::count(const LinkedObject *root)
 {
-    assert(root != NULL);
+    assert(root != nullptr);
 
     unsigned c = 0;
     while(root) {
@@ -386,7 +237,7 @@ unsigned LinkedObject::count(const LinkedObject *root)
 
 unsigned NamedObject::keyindex(const char *id, unsigned max)
 {
-    assert(id != NULL && *id != 0);
+    assert(id != nullptr && *id != 0);
     assert(max > 1);
 
     unsigned val = 0;
@@ -399,7 +250,7 @@ unsigned NamedObject::keyindex(const char *id, unsigned max)
 
 int NamedObject::compare(const char *cid) const
 {
-    assert(cid != NULL && *cid != 0);
+    assert(cid != nullptr && *cid != 0);
 
 #ifdef  HAVE_STRCOLL
     return strcoll(Id, cid);
@@ -412,8 +263,8 @@ extern "C" {
 
     static int ncompare(const void *o1, const void *o2)
     {
-        assert(o1 != NULL);
-        assert(o2 != NULL);
+        assert(o1 != nullptr);
+        assert(o2 != nullptr);
         const NamedObject * const *n1 = static_cast<const NamedObject * const*>(o1);
         const NamedObject * const*n2 = static_cast<const NamedObject * const*>(o2);
         return ((*n1)->compare((*n2)->getId()));
@@ -422,7 +273,7 @@ extern "C" {
 
 NamedObject **NamedObject::sort(NamedObject **list, size_t size)
 {
-    assert(list != NULL);
+    assert(list != nullptr);
 
     if(!size) {
         while(list[size])
@@ -435,23 +286,23 @@ NamedObject **NamedObject::sort(NamedObject **list, size_t size)
 
 NamedObject **NamedObject::index(NamedObject **idx, unsigned max)
 {
-    assert(idx != NULL);
+    assert(idx != nullptr);
     assert(max > 0);
     NamedObject **op = new NamedObject *[count(idx, max) + 1];
     unsigned pos = 0;
-    NamedObject *node = skip(idx, NULL, max);
+    NamedObject *node = skip(idx, nullptr, max);
 
     while(node) {
         op[pos++] = node;
         node = skip(idx, node, max);
     }
-    op[pos] = NULL;
+    op[pos] = nullptr;
     return op;
 }
 
 NamedObject *NamedObject::skip(NamedObject **idx, NamedObject *rec, unsigned max)
 {
-    assert(idx != NULL);
+    assert(idx != nullptr);
     assert(max > 0);
 
     unsigned key = 0;
@@ -462,7 +313,7 @@ NamedObject *NamedObject::skip(NamedObject **idx, NamedObject *rec, unsigned max
         while(key < max && !idx[key])
             ++key;
         if(key >= max)
-            return NULL;
+            return nullptr;
         return idx[key];
     }
 
@@ -471,7 +322,7 @@ NamedObject *NamedObject::skip(NamedObject **idx, NamedObject *rec, unsigned max
 
 void NamedObject::purge(NamedObject **idx, unsigned max)
 {
-    assert(idx != NULL);
+    assert(idx != nullptr);
     assert(max > 0);
 
     LinkedObject *root;
@@ -487,7 +338,7 @@ void NamedObject::purge(NamedObject **idx, unsigned max)
 
 unsigned NamedObject::count(NamedObject **idx, unsigned max)
 {
-    assert(idx != NULL);
+    assert(idx != nullptr);
     assert(max > 0);
 
     unsigned count = 0;
@@ -508,8 +359,8 @@ unsigned NamedObject::count(NamedObject **idx, unsigned max)
 
 NamedObject *NamedObject::remove(NamedObject **idx, const char *id, unsigned max)
 {
-    assert(idx != NULL);
-    assert(id != NULL && *id != 0);
+    assert(idx != nullptr);
+    assert(id != nullptr && *id != 0);
     assert(max > 0);
 
     if(max < 2)
@@ -520,8 +371,8 @@ NamedObject *NamedObject::remove(NamedObject **idx, const char *id, unsigned max
 
 NamedObject *NamedObject::map(NamedObject **idx, const char *id, unsigned max)
 {
-    assert(idx != NULL);
-    assert(id != NULL && *id != 0);
+    assert(idx != nullptr);
+    assert(id != nullptr && *id != 0);
     assert(max > 0);
 
     if(max < 2)
@@ -532,7 +383,7 @@ NamedObject *NamedObject::map(NamedObject **idx, const char *id, unsigned max)
 
 NamedObject *NamedObject::find(NamedObject *root, const char *id)
 {
-    assert(id != NULL && *id != 0);
+    assert(id != nullptr && *id != 0);
 
     while(root) {
         if(root->equal(id))
@@ -544,10 +395,10 @@ NamedObject *NamedObject::find(NamedObject *root, const char *id)
 
 NamedObject *NamedObject::remove(NamedObject **root, const char *id)
 {
-    assert(id != NULL && *id != 0);
-    assert(root != NULL);
+    assert(id != nullptr && *id != 0);
+    assert(root != nullptr);
 
-    NamedObject *prior = NULL;
+    NamedObject *prior = nullptr;
     NamedObject *node = *root;
 
     while(node) {
@@ -558,9 +409,9 @@ NamedObject *NamedObject::remove(NamedObject **root, const char *id)
     }
 
     if(!node)
-        return NULL;
+        return nullptr;
 
-    if(prior == NULL)
+    if(prior == nullptr)
         *root = node->getNext();
     else
         prior->Next = node->getNext();
@@ -576,21 +427,21 @@ NamedTree::NamedTree(char *nid) :
 NamedObject(), Child()
 {
     Id = nid;
-    Parent = NULL;
+    Parent = nullptr;
 }
 
 NamedTree::NamedTree(const NamedTree& source)
 {
     Id = source.Id;
-    Parent = NULL;
+    Parent = nullptr;
     Child = source.Child;
 }
 
 NamedTree::NamedTree(NamedTree *p, char *nid) :
 NamedObject(), Child()
 {
-    assert(p != NULL);
-    assert(nid != NULL && *nid != 0);
+    assert(p != nullptr);
+    assert(nid != nullptr && *nid != 0);
 
     enlistTail(&p->Child);
     Id = nid;
@@ -599,13 +450,13 @@ NamedObject(), Child()
 
 NamedTree::~NamedTree()
 {
-    Id = NULL;
+    Id = nullptr;
     purge();
 }
 
 NamedTree *NamedTree::getChild(const char *tid) const
 {
-    assert(tid != NULL && *tid != 0);
+    assert(tid != nullptr && *tid != 0);
 
     linked_pointer<NamedTree> node = Child.begin();
 
@@ -614,7 +465,7 @@ NamedTree *NamedTree::getChild(const char *tid) const
             return *node;
         node.next();
     }
-    return NULL;
+    return nullptr;
 }
 
 void NamedTree::relistTail(NamedTree *trunk)
@@ -644,7 +495,7 @@ void NamedTree::relistHead(NamedTree *trunk)
 
 NamedTree *NamedTree::path(const char *tid) const
 {
-    assert(tid != NULL && *tid != 0);
+    assert(tid != nullptr && *tid != 0);
 
     const char *np;
     char buf[65];
@@ -656,7 +507,7 @@ NamedTree *NamedTree::path(const char *tid) const
 
     while(*tid == '.') {
         if(!node->Parent)
-            return NULL;
+            return nullptr;
         node = node->Parent;
 
         ++tid;
@@ -671,7 +522,7 @@ NamedTree *NamedTree::path(const char *tid) const
         if(np)
             tid = ++np;
         else
-            tid = NULL;
+            tid = nullptr;
         node = node->getChild(buf);
     }
     return node;
@@ -679,7 +530,7 @@ NamedTree *NamedTree::path(const char *tid) const
 
 NamedTree *NamedTree::getLeaf(const char *tid) const
 {
-    assert(tid != NULL && *tid != 0);
+    assert(tid != nullptr && *tid != 0);
 
     linked_pointer<NamedTree> node = Child.begin();
 
@@ -688,12 +539,12 @@ NamedTree *NamedTree::getLeaf(const char *tid) const
             return *node;
         node.next();
     }
-    return NULL;
+    return nullptr;
 }
 
 NamedTree *NamedTree::leaf(const char *tid) const
 {
-    assert(tid != NULL && *tid != 0);
+    assert(tid != nullptr && *tid != 0);
 
     linked_pointer<NamedTree> node = Child.begin();
     NamedTree *obj;
@@ -701,19 +552,19 @@ NamedTree *NamedTree::leaf(const char *tid) const
     while(node) {
         if(node->is_leaf() && eq(node->Id, tid))
             return *node;
-        obj = NULL;
+        obj = nullptr;
         if(!node->is_leaf())
             obj = node->leaf(tid);
         if(obj)
             return obj;
         node.next();
     }
-    return NULL;
+    return nullptr;
 }
 
 NamedTree *NamedTree::find(const char *tid) const
 {
-    assert(tid != NULL && *tid != 0);
+    assert(tid != nullptr && *tid != 0);
 
     linked_pointer<NamedTree> node = Child.begin();
     NamedTree *obj;
@@ -728,12 +579,12 @@ NamedTree *NamedTree::find(const char *tid) const
         }
         node.next();
     }
-    return NULL;
+    return nullptr;
 }
 
 void NamedTree::setId(char *nid)
 {
-    assert(nid != NULL && *nid != 0);
+    assert(nid != nullptr && *nid != 0);
 
     Id = nid;
 }
@@ -746,7 +597,7 @@ void NamedTree::remove(void)
     if(Parent)
         delist(&Parent->Child);
 
-    Id = NULL;
+    Id = nullptr;
 }
 
 void NamedTree::purge(void)
@@ -761,7 +612,7 @@ void NamedTree::purge(void)
         obj = *node;
 		if (!obj)
 			break;
-        obj->Parent = NULL; // save processing
+        obj->Parent = nullptr; // save processing
         node = obj->getNext();
         delete obj;
     }
@@ -773,26 +624,35 @@ void NamedTree::purge(void)
 
 LinkedObject::LinkedObject()
 {
-    Next = 0;
+    Next = nullptr;
+}
+
+LinkedObject::LinkedObject(const LinkedObject& from)
+{
+    Next = nullptr;
 }
 
 OrderedObject::OrderedObject() : LinkedObject()
 {
 }
 
+OrderedObject::OrderedObject(const OrderedObject& from) : LinkedObject()
+{
+}
+
 OrderedObject::OrderedObject(OrderedIndex *root) :
 LinkedObject()
 {
-    assert(root != NULL);
-    Next = NULL;
+    assert(root != nullptr);
+    Next = nullptr;
     enlistTail(root);
 }
 
 void OrderedObject::delist(OrderedIndex *root)
 {
-    assert(root != NULL);
+    assert(root != nullptr);
 
-    OrderedObject *prior = NULL, *node;
+    OrderedObject *prior = nullptr, *node;
 
     node = root->head;
 
@@ -815,17 +675,17 @@ void OrderedObject::delist(OrderedIndex *root)
 
 void OrderedObject::enlist(OrderedIndex *root)
 {
-    assert(root != NULL);
+    assert(root != nullptr);
 
-    Next = NULL;
+    Next = nullptr;
     enlistTail(root);
 }
 
 void OrderedObject::enlistTail(OrderedIndex *root)
 {
-    assert(root != NULL);
+    assert(root != nullptr);
 
-    if(root->head == NULL)
+    if(root->head == nullptr)
         root->head = this;
     else if(root->tail)
         root->tail->Next = this;
@@ -835,10 +695,10 @@ void OrderedObject::enlistTail(OrderedIndex *root)
 
 void OrderedObject::enlistHead(OrderedIndex *root)
 {
-    assert(root != NULL);
+    assert(root != nullptr);
 
-    Next = NULL;
-    if(root->tail == NULL)
+    Next = nullptr;
+    if(root->tail == nullptr)
         root->tail = this;
     else if(root->head)
         Next = root->head;
@@ -846,38 +706,45 @@ void OrderedObject::enlistHead(OrderedIndex *root)
     root->head = this;
 }
 
-LinkedList::LinkedList()
+DLinkedObject::DLinkedObject()
 {
-    Root = 0;
-    Prev = 0;
-    Next = 0;
+    Root = nullptr;
+    Prev = nullptr;
+    Next = nullptr;
 }
 
-LinkedList::LinkedList(OrderedIndex *r)
+DLinkedObject::DLinkedObject(const DLinkedObject& from)
 {
-    Root = NULL;
-    Next = Prev = 0;
+    Root = nullptr;
+    Prev = nullptr;
+    Next = nullptr;
+}
+
+DLinkedObject::DLinkedObject(OrderedIndex *r)
+{
+    Root = nullptr;
+    Next = Prev = nullptr;
     if(r)
         enlist(r);
 }
 
-void LinkedList::enlist(OrderedIndex *r)
+void DLinkedObject::enlist(OrderedIndex *r)
 {
-    assert(r != NULL);
+    assert(r != nullptr);
 
     enlistTail(r);
 }
 
-void LinkedList::insert(LinkedList *o)
+void DLinkedObject::insert(DLinkedObject *o)
 {
-    assert(o != NULL);
+    assert(o != nullptr);
 
     insertTail(o);
 }
 
-void LinkedList::insertHead(LinkedList *o)
+void DLinkedObject::insertHead(DLinkedObject *o)
 {
-    assert(o != NULL);
+    assert(o != nullptr);
 
     if(o->Root)
         o->delist();
@@ -888,16 +755,16 @@ void LinkedList::insertHead(LinkedList *o)
     }
     else {
         Root->head = o;
-        o->Prev = NULL;
+        o->Prev = nullptr;
     }
     o->Root = Root;
     o->Next = this;
     Prev = o;
 }
 
-void LinkedList::insertTail(LinkedList *o)
+void DLinkedObject::insertTail(DLinkedObject *o)
 {
-    assert(o != NULL);
+    assert(o != nullptr);
 
     if(o->Root)
         o->delist();
@@ -908,57 +775,56 @@ void LinkedList::insertTail(LinkedList *o)
     }
     else {
         Root->tail = o;
-        o->Next = NULL;
+        o->Next = nullptr;
     }
     o->Root = Root;
     o->Prev = this;
     Next = o;
 }
 
-void LinkedList::enlistHead(OrderedIndex *r)
+void DLinkedObject::enlistHead(OrderedIndex *r)
 {
-    assert(r != NULL);
+    assert(r != nullptr);
 
     if(Root)
         delist();
 
     Root = r;
-    Prev = 0;
-    Next = 0;
+    Prev = nullptr; 
+    Next = nullptr;
 
     if(!Root->tail) {
         Root->tail = Root->head = static_cast<OrderedObject *>(this);
         return;
     }
 
-    Next = static_cast<LinkedList *>(Root->head);
-    ((LinkedList*)Next)->Prev = this;
+    Next = static_cast<DLinkedObject *>(Root->head);
+    ((DLinkedObject*)Next)->Prev = this;
     Root->head = static_cast<OrderedObject *>(this);
 }
 
 
-void LinkedList::enlistTail(OrderedIndex *r)
+void DLinkedObject::enlistTail(OrderedIndex *r)
 {
-    assert(r != NULL);
+    assert(r != nullptr);
 
     if(Root)
         delist();
 
     Root = r;
-    Prev = 0;
-    Next = 0;
+    Next = Prev = nullptr;
 
     if(!Root->head) {
         Root->head = Root->tail = static_cast<OrderedObject *>(this);
         return;
     }
 
-    Prev = static_cast<LinkedList *>(Root->tail);
+    Prev = static_cast<DLinkedObject *>(Root->tail);
     Prev->Next = this;
     Root->tail = static_cast<OrderedObject *>(this);
 }
 
-void LinkedList::delist(void)
+void DLinkedObject::delist(void)
 {
     if(!Root)
         return;
@@ -969,27 +835,27 @@ void LinkedList::delist(void)
         Root->head = static_cast<OrderedObject *>(Next);
 
     if(Next)
-        (static_cast<LinkedList *>(Next))->Prev = Prev;
+        (static_cast<DLinkedObject *>(Next))->Prev = Prev;
     else if(Root->tail == static_cast<OrderedObject *>(this))
         Root->tail = static_cast<OrderedObject *>(Prev);
 
-    Root = 0;
-    Next = Prev = 0;
+    Root = nullptr;
+    Next = Prev = nullptr;
 }
 
-LinkedList::~LinkedList()
+DLinkedObject::~DLinkedObject()
 {
     delist();
 }
 
 OrderedIndex::OrderedIndex()
 {
-    head = tail = NULL;
+    head = tail = nullptr;
 }
 
 OrderedIndex::~OrderedIndex()
 {
-    head = tail = NULL;
+    head = tail = nullptr;
 }
 
 void OrderedIndex::copy(const OrderedIndex& source)
@@ -1000,14 +866,14 @@ void OrderedIndex::copy(const OrderedIndex& source)
 
 void OrderedIndex::operator*=(OrderedObject *object)
 {
-    assert(object != NULL);
+    assert(object != nullptr);
 
     object->enlist(this);
 }
 
 void OrderedIndex::add(OrderedObject *object)
 {
-    assert(object != NULL);
+    assert(object != nullptr);
 
     object->enlist(this);
 }
@@ -1017,12 +883,12 @@ LinkedObject *OrderedIndex::get(void)
     LinkedObject *node;
 
     if(!head)
-        return NULL;
+        return nullptr;
 
     node = head;
     head = static_cast<OrderedObject *>(node->getNext());
     if(!head)
-        tail = NULL;
+        tail = nullptr;
 
     return static_cast<LinkedObject *>(node);
 }
@@ -1031,13 +897,13 @@ void OrderedIndex::purge(void)
 {
     if(head) {
         LinkedObject::purge((LinkedObject *)head);
-        head = tail = NULL;
+        head = tail = nullptr;
     }
 }
 
 void OrderedIndex::reset(void)
 {
-    head = tail = NULL;
+    head = tail = nullptr;
 }
 
 void OrderedIndex::lock_index(void)
@@ -1048,82 +914,6 @@ void OrderedIndex::unlock_index(void)
 {
 }
 
-DLinkedObject::DLinkedObject() : OrderedObject()
-{
-    Prev = NULL;
-}
-
-void DLinkedObject::delist(void)
-{
-    if(Prev)
-        Prev->Next = Next;
-
-    if(Next)
-        ((DLinkedObject *)Next)->Prev = Prev;
-
-    Next = Prev = NULL;
-}
-
-ObjectQueue::ObjectQueue() :
-OrderedIndex() {}
-
-void ObjectQueue::add(DLinkedObject *object)
-{
-    assert(object);
-
-    if(tail) {
-        ((DLinkedObject *)tail)->Next = object;
-        object->Prev = (DLinkedObject *)tail;
-    }
-
-    object->Next = NULL;
-    tail = object;
-    if(!head)
-        head = tail;
-}
-
-void ObjectQueue::push(DLinkedObject *object)
-{
-    assert(object);
-
-    if(head) {
-        ((DLinkedObject *)head)->Prev = object;
-        object->Next = (DLinkedObject *)head;
-    }
-    object->Prev = NULL;
-    head = object;
-    if(!tail)
-        tail = head;
-}
-
-DLinkedObject *ObjectQueue::pull(void)
-{
-    DLinkedObject *obj = (DLinkedObject *)head;
-
-    if(!obj)
-        return NULL;
-
-    head = (OrderedObject *)(obj->Next);
-    if(!head)
-        tail = NULL;
-    obj->delist();
-    return obj;
-}
-
-DLinkedObject *ObjectQueue::pop(void)
-{
-    DLinkedObject *obj = (DLinkedObject *)tail;
-
-    if(!obj)
-        return NULL;
-
-    tail = (OrderedObject *)(obj->Prev);
-    if(!tail)
-        head = NULL;
-    obj->delist();
-    return obj;
-}
-
 LinkedObject **OrderedIndex::index(void) const
 {
     LinkedObject **op = new LinkedObject *[count() + 1];
@@ -1135,7 +925,7 @@ LinkedObject **OrderedIndex::index(void) const
         op[idx++] = node;
         node = node->Next;
     }
-    op[idx] = NULL;
+    op[idx] = nullptr;
     return op;
 }
 
@@ -1166,36 +956,4 @@ unsigned OrderedIndex::count(void) const
     return count;
 }
 
-ObjectStack::ObjectStack()
-{
-    root = NULL;
-}
-
-ObjectStack::ObjectStack(LinkedObject *list)
-{
-    root = list;
-}
-
-void ObjectStack::push(LinkedObject *list)
-{
-    assert(list != NULL);
-
-    list->Next = root;
-    root = list;
-}
-
-LinkedObject *ObjectStack::pull(void)
-{
-    LinkedObject *obj;
-
-    obj = root;
-
-    if(obj) {
-        root = obj->Next;
-        obj->Next = NULL;
-    }
-
-    return obj;
-}
-
 } // namespace ucommon
