@@ -236,7 +236,7 @@ void MappedMemory::create(const char *fn, size_t len)
         assert(len > 0);    // cannot use dummy for r/o...
         map = (caddr_t)malloc(len);
         if(!map)
-            fault();
+            __THROW_ALLOC();
         size = len;
         return;
     }
@@ -262,7 +262,7 @@ void MappedMemory::create(const char *fn, size_t len)
         VirtualLock(map, size);
     }
     else
-        fault();
+        __THROW_ALLOC();
 }
 
 MappedMemory::~MappedMemory()
@@ -315,7 +315,7 @@ void MappedMemory::create(const char *fn, size_t len)
         if(len)
             map = (caddr_t)malloc(len);
         if(!map)
-            fault();
+            __THROW_ALLOC();
         size = mapsize = len;
         return;
     }
@@ -350,7 +350,7 @@ void MappedMemory::create(const char *fn, size_t len)
 
     map = (caddr_t)mmap(NULL, len, prot, MAP_SHARED, fd, 0);
     if(!map)
-        fault();
+        __THROW_ALLOC();
     ::close(fd);
     if(map != (caddr_t)MAP_FAILED) {
         size = mapsize = len;
@@ -440,7 +440,7 @@ void MappedMemory::create(const char *name, size_t len)
         assert(len > 0);
         map = (caddr_t)malloc(len);
         if(!map)
-            fault();
+            __THROW_ALLOC();
         size = len;
         return;
     }
@@ -472,7 +472,7 @@ remake:
     }
     map = (caddr_t)shmat(fd, NULL, 0);
     if(!map)
-        fault();
+        __THROW_ALLOC();
 #ifdef  SHM_LOCK
     if(fd > -1)
         shmctl(fd, SHM_LOCK, NULL);
@@ -506,23 +506,12 @@ void MappedMemory::release(void)
 
 #endif
 
-void *MappedMemory::invalid(void) const
-{
-    abort();
-    return NULL;
-}
-
-void MappedMemory::fault(void) const
-{
-    abort();
-}
-
 void *MappedMemory::sbrk(size_t len)
 {
     assert(len > 0);
     void *mp = (void *)(map + used);
     if(used + len > size)
-        fault();
+        __THROW_RANGE("Outside mapped memory");
     used += len;
     return mp;
 }
@@ -530,7 +519,7 @@ void *MappedMemory::sbrk(size_t len)
 bool MappedMemory::copy(size_t offset, void *buffer, size_t bufsize) const
 {
     if(!map || (offset + bufsize > size)) {
-        fault();
+        __THROW_RANGE("Outside mapped memory");
         return false;
     }
 
@@ -546,7 +535,7 @@ bool MappedMemory::copy(size_t offset, void *buffer, size_t bufsize) const
 void *MappedMemory::offset(size_t offset) const
 {
     if(offset >= size)
-        return invalid();
+        __THROW_RANGE("outside mapped memory"); 
     return (void *)(map + offset);
 }
 
@@ -573,7 +562,7 @@ bool MappedReuse::avail(void) const
 {
     bool rtn = false;
 
-    __AUTOLOCK__
+    __AUTOLOCK(this);
 
     if(freelist || used < size)
         rtn = true;
@@ -584,7 +573,7 @@ ReusableObject *MappedReuse::request(void)
 {
     ReusableObject *obj = NULL;
 
-    __AUTOLOCK__
+    __AUTOLOCK(this);
 
     if(freelist) {
         obj = freelist;
@@ -631,7 +620,7 @@ ReusableObject *MappedReuse::getTimed(timeout_t timeout)
     if(timeout && timeout != Timer::inf)
         set(&ts, timeout);
 
-    __AUTOLOCK__
+    __AUTOLOCK(this);
 
     while(rtn && (!freelist || (freelist && reading)) && used >= size) {
         ++waiting;
