@@ -24,6 +24,11 @@
 #include <ucommon/thread.h>
 #include <ucommon/fsys.h>
 #include <ucommon/shell.h>
+
+#ifndef  UCOMMON_SYSRUNTIME
+#include <ucommon/stream.h>
+#endif
+
 #include <stdlib.h>
 #include <stdarg.h>
 #ifdef  HAVE_UNISTD_H
@@ -779,47 +784,6 @@ skip:
     return _argv;
 }
 
-void shell::error(const char *format, ...)
-{
-    va_list args;
-    char buf[256];
-
-    String::set(buf, sizeof(buf) - 1, format);
-    size_t len = strlen(buf);
-
-    if(buf[len - 1] != '\n') {
-        buf[len] = '\n';
-        buf[len + 1] = 0;
-    }
-    else
-        --len;
-
-    format = buf;
-
-    va_start(args, format);
-    if(!eq("*** ", format, 4))
-        fputs("*** ", stderr);
-    vfprintf(stderr, format, args);
-    fflush(stderr);
-
-    buf[len] = 0;
-
-#ifdef  HAVE_SYSLOG_H
-    if(errname && errmode != NONE && (ERR <= errlevel)) {
-        if(eq("*** ", format, 4)) {
-            format += 4;
-            const char *cp = format;
-            while(isalnum(*cp) || *cp == '-' || *cp == '.')
-                ++cp;
-            if(*cp == ':' && cp[1] == ' ')
-                format = cp + 2;
-        }
-        vsyslog(LOG_ERR, format, args);
-    }
-#endif
-    va_end(args);
-}
-
 void shell::errlog(const char *format, ...)
 {
     va_list args;
@@ -861,8 +825,6 @@ void shell::errlog(const char *format, ...)
     va_end(args);
 }
 
-
-
 void shell::errexit(int exitcode, const char *format, ...)
 {
     if(!exitcode)
@@ -921,39 +883,6 @@ size_t shell::printf(const char *format, ...)
     return result;
 }
 
-size_t shell::readln(char *address, size_t size)
-{
-    address[0] = 0;
-
-    if(!fgets(address, (socksize_t)size, stdin))
-        return 0;
-
-    if(address[size - 1] == '\n') {
-        --size;
-        if(address[size - 1] == '\r')
-            --size;
-    }
-    address[size] = 0;
-    return size;
-}
-
-size_t shell::read(String& string)
-{
-    char *cp = string.c_mem();
-    size_t size = string.size();
-    size = readln(cp, size);
-    String::fix(string);
-    return size;
-}
-
-size_t shell::writes(const char *string)
-{
-    size_t result = fputs(string, stdout);
-    if(result == ((size_t)(EOF)))
-        return 0;
-    return result;
-}
-
 #ifdef _MSWINDOWS_
 
 static void pathfinder(const char *name, char *buf, size_t size)
@@ -1086,7 +1015,7 @@ String shell::path(path_t id)
     return result;
 }
 
-const char *shell::env(const char *id, const char *value)
+const char *shell::getenv(const char *id, const char *value)
 {
     char buf[512];
     char path[255];
@@ -1733,7 +1662,7 @@ String shell::path(path_t id)
     return result;
 }
 
-const char *shell::env(const char *id, const char *value)
+const char *shell::getenv(const char *id, const char *value)
 {
     const char *v = ::getenv(id);
     if(v)
@@ -2518,7 +2447,7 @@ bool shell::is_sym(const char *name) const
     return false;
 }
 
-const char *shell::get(const char *name, const char *value)
+const char *shell::getsym(const char *name, const char *value)
 {
     symlock.acquire();
     linked_pointer<syms> sp = _syms;
@@ -2533,10 +2462,10 @@ const char *shell::get(const char *name, const char *value)
     }
 
     symlock.release();
-    return env(name, value);
+    return shell::getenv(name, value);
 }
 
-void shell::set(const char *name, const char *value)
+void shell::setsym(const char *name, const char *value)
 {
     symlock.acquire();
     linked_pointer<syms> sp;
