@@ -20,6 +20,7 @@
 #include <ucommon/export.h>
 #include <ucommon/socket.h>
 #include <ucommon/string.h>
+#include <ucommon/typeref.h>
 #include <ucommon/thread.h>
 #include <ucommon/fsys.h>
 #ifndef _MSWINDOWS_
@@ -35,12 +36,6 @@
 #endif
 #include <errno.h>
 
-#if defined(HAVE_SOCKS)
-#include <socks.h>
-#undef  HAVE_POLL_H
-#undef  HAVE_SYS_POLL_H
-#endif
-
 #if defined(HAVE_POLL_H)
 #include <poll.h>
 #elif defined(HAVE_SYS_POLL_H)
@@ -109,54 +104,9 @@ struct addrinfo {
 
 #endif
 
-#if defined(HAVE_SOCKS)
-#define _getsockname_(so, addr, alen) ::Rgetsockname(so, addr, alen)
-#define _getpeername_(so, addr, alen) ::Rgetpeername(so, addr, alen)
-#define _accept_(so, addr, alen) ::Raccept(so, addr, alen)
-#define _connect_(so, addr, alen) ::Rconnect(so, addr, alen)
-#define _bind_(so, addr, alen) ::Rbind(so, addr, alen)
-#define _listen_(so, backlog) ::Rlisten(so, backlog)
-#define _select_(cnt, rfd, wfd, efd, timeout) ::Rselect(cnt, rfd, wfd, efd, timeout)
-#define _send_(so, buf, bytes, flag) ::Rsend(so, buf, bytes, flag)
-#define _recv_(so, buf, bytes, flag) ::Rrecv(so, buf, bytes, flag)
-#define _sendto_(so, buf, bytes, flag, to, tolen) ::Rsendto(so, buf, bytes, flag, to, tolen)
-#define _recvfrom_(so, buf, bytes, flag, from, fromlen) ::Rrecvfrom(so, buf, bytes, flag, from, fromlen)
-#undef  USE_POLL
-#undef  accept
-#undef  sendto
-#undef  recvfrom
-#undef  select
-#elif defined(__PTH__)
-#define _send_(so, buf, bytes, flag) pth_send(so, buf, bytes, flag)
-#define _recv_(so, buf, bytes, flag) pth_recv(so, buf, bytes, flag)
-#define _sendto_(so, buf, bytes, flag, to, tolen) pth_sendto(so, buf, bytes, flag, to, tolen)
-#define _recvfrom_(so, buf, bytes, flag, from, fromlen) pth_recvfrom(so, buf, bytes, flag, from, fromlen)
-#define _connect_(so, addr, addrlen) pth_connect(so, addr, addrlen)
-#define _accept_(so, addr, addrlen) pth_accept(so, addr, addrlen)
-#define _select_(cnt, rfd, wfd, efd, timeout) pth_select(cnt, rfd, wfd, efd, timeout)
-#define _poll_(fds, cnt, timeout) pth_poll(fds, cnt, timeout)
-#define _getsockname_(so, addr, alen) ::getsockname(so, addr, alen)
-#define _getpeername_(so, addr, alen) ::getpeername(so, addr, alen)
-#define _bind_(so, addr, alen) ::bind(so, addr, alen)
-#define _listen_(so, count) ::listen(so, count)
-#else
-#define _send_(so, buf, bytes, flag) ::send(so, buf, bytes, flag)
-#define _recv_(so, buf, bytes, flag) ::recv(so, buf, bytes, flag)
-#define _sendto_(so, buf, bytes, flag, to, tolen) ::sendto(so, buf, bytes, flag, to, tolen)
-#define _recvfrom_(so, buf, bytes, flag, from, fromlen) ::recvfrom(so, buf, bytes, flag, from, fromlen)
-#define _connect_(so, addr, addrlen) ::connect(so, addr, addrlen)
-#define _accept_(so, addr, addrlen) ::accept(so, addr, addrlen)
-#define _select_(cnt, rfd, wfd, efd, timeout) ::select(cnt, rfd, wfd, efd, timeout)
-#define _poll_(fds, cnt, timeout) ::poll(fds, cnt, timeout)
-#define _getsockname_(so, addr, alen) ::getsockname(so, addr, alen)
-#define _getpeername_(so, addr, alen) ::getpeername(so, addr, alen)
-#define _bind_(so, addr, alen) ::bind(so, addr, alen)
-#define _listen_(so, count) ::listen(so, count)
-#endif
-
 namespace ucommon {
 
-typedef unsigned char   bit_t;
+typedef uint8_t   bit_t;
 
 static int query_family = 0;
 static int v6only = 0;
@@ -572,8 +522,8 @@ void Socket::init(void)
     initialized = true;
     version = MAKEWORD(2,2);
     status.wVersion = 0;
-    WSAStartup(version, &status);
-    crit(status.wVersion == version, "socket init failure");
+    if(WSAStartup(version, &status))
+		__THROW_RUNTIME("socket init failure");
     atexit(_socketcleanup);
     _started = true;
 }
@@ -583,40 +533,6 @@ void Socket::init(void)
 }
 #endif
 
-void Socket::init(const char *progname)
-{
-    assert(progname != NULL);
-
-    Socket::init();
-#ifdef  HAVE_SOCKS
-    const char *cp;
-
-#ifdef  _MSWINDOWS_
-    char path[65];
-
-    cp = strrchr(progname, '/');
-    if(!cp)
-        cp = strrchr(progname, '\\');
-    if(!cp)
-        cp = strrchr(progname, ':');
-    if(cp)
-        progname = ++cp;
-
-    String::set(path, sizeof(path), progname);
-    const char *ext = strrchr(path, '.');
-    if(ext && (case_eq(ext, ".exe") || case_eq(ext, ".com")))
-        *ext = 0;
-    progname = path;
-#else
-    cp = strrchr(progname, '/');
-    if(cp)
-        progname = ++cp;
-#endif
-
-    SOCKSinit((char *)progname);
-#endif
-}
-
 void Socket::v4mapping(bool enable)
 {
     if(enable)
@@ -740,7 +656,7 @@ bool cidr::is_member(const struct sockaddr *s) const
 {
     assert(s != NULL);
 
-    inethostaddr_t host;
+    struct hostaddr_internet host;
     struct sockaddr_internet *addr = (struct sockaddr_internet *)s;
 
     if(addr->address.sa_family != Family)
@@ -766,9 +682,9 @@ bool cidr::is_member(const struct sockaddr *s) const
     }
 }
 
-inethostaddr_t cidr::broadcast(void) const
+struct hostaddr_internet cidr::broadcast(void) const
 {
-    inethostaddr_t bcast;
+    struct hostaddr_internet bcast;
 
     switch(Family) {
     case AF_INET:
@@ -795,7 +711,7 @@ unsigned cidr::mask(const char *cp) const
     const char *sp = strchr(cp, '/');
     bool flag = false;
     const char *gp = cp;
-    unsigned char dots[4];
+    uint8_t dots[4];
     uint32_t mask;
 
     switch(Family) {
@@ -976,7 +892,7 @@ Socket::address::address(const in_addr& address, in_port_t port) : list(NULL)
     addr.sin_family = AF_INET;
     addr.sin_addr = address;
     addr.sin_port = htons(port);
-    insert((sockaddr&)addr);
+    insert((const struct sockaddr *)&addr);
 }
 
 Socket::address::address(const in6_addr& address, in_port_t port) : list(NULL)
@@ -985,7 +901,7 @@ Socket::address::address(const in6_addr& address, in_port_t port) : list(NULL)
     addr.sin6_family = AF_INET6;
     addr.sin6_addr = address;
     addr.sin6_port = htons(port);
-    insert((sockaddr&)addr);
+    insert((const struct sockaddr *)&addr);
 }
 
 Socket::address::address()
@@ -1201,7 +1117,15 @@ proc:
 	strfree(addr);
 }
 
-struct sockaddr *Socket::address::get(void) const
+struct sockaddr *Socket::address::modify(void)
+{
+    if(!list)
+        return NULL;
+
+    return list->ai_addr;
+}
+
+const struct sockaddr *Socket::address::get(void) const
 {
     if(!list)
         return NULL;
@@ -1243,7 +1167,7 @@ Socket::address Socket::address::withPort(in_port_t port) const
     return copy;
 }
 
-struct sockaddr *Socket::address::get(int family) const
+struct sockaddr *Socket::address::modify(int family)
 {
     struct sockaddr *ap;
     struct addrinfo *lp;
@@ -1259,6 +1183,22 @@ struct sockaddr *Socket::address::get(int family) const
     return NULL;
 }
 
+const struct sockaddr *Socket::address::get(int family) const
+{
+    const struct sockaddr *ap;
+    const struct addrinfo *lp;
+
+    lp = list;
+
+    while(lp) {
+        ap = lp->ai_addr;
+        if(ap && ap->sa_family == family)
+            return ap;
+        lp = lp->ai_next;
+    }
+    return NULL;
+}
+
 void Socket::address::set(struct sockaddr *addr)
 {
     clear();
@@ -1326,7 +1266,7 @@ bool Socket::address::insert(const struct sockaddr *addr)
 
     char buf[256], svc[16];
     query(addr, buf, sizeof(buf));
-    snprintf(svc, sizeof(svc), "%d", service(addr));
+    snprintf(svc, sizeof(svc), "%d", port(addr));
     memset(&hints, 0, sizeof(hints));
     hints.ai_family = addr->sa_family;
     hints.ai_flags = AI_NUMERICHOST | AI_NUMERICSERV;
@@ -1353,21 +1293,6 @@ void Socket::address::copy(const struct addrinfo *addr)
     }
 }
 
-in_port_t Socket::address::getPort(const struct sockaddr *address)
-{
-    if (address == NULL)
-        return 0;
-
-    switch (address->sa_family) {
-    case AF_INET:
-        return ntohs(reinterpret_cast<const sockaddr_in* >(address)->sin_port);
-    case AF_INET6:
-        return ntohs(reinterpret_cast<const sockaddr_in6*>(address)->sin6_port);
-    default:
-        return 0;
-    }
-}
-
 void Socket::address::setPort(struct sockaddr *address, in_port_t port)
 {
     if (address == NULL)
@@ -1517,7 +1442,7 @@ void Socket::address::add(struct sockaddr *addr)
     char svc[8];
 
     Socket::query(addr, buffer, sizeof(buffer));
-    snprintf(svc, sizeof(svc), "%d", Socket::service(addr));
+    snprintf(svc, sizeof(svc), "%d", Socket::port(addr));
     add(buffer, svc, addr->sa_family);
 }
 
@@ -1685,7 +1610,7 @@ Socket::Socket(const struct addrinfo *addr)
         so = ::socket(addr->ai_family, addr->ai_socktype, addr->ai_protocol);
         socket_mapping(addr->ai_family, so);
         if(so != INVALID_SOCKET) {
-            if(!_connect_(so, addr->ai_addr, (socklen_t)addr->ai_addrlen))
+            if(!::connect(so, addr->ai_addr, (socklen_t)addr->ai_addrlen))
                 return;
         }
         addr = addr->ai_next;
@@ -1767,7 +1692,7 @@ socket_t Socket::create(const char *iface, const char *port, int family, int typ
         so = create(AF_UNIX, type, 0);
         if(so == INVALID_SOCKET)
             return INVALID_SOCKET;
-        if(_bind_(so, (struct sockaddr *)&uaddr, len)) {
+        if(::bind(so, (struct sockaddr *)&uaddr, len)) {
             release(so);
             return INVALID_SOCKET;
         }
@@ -1789,7 +1714,7 @@ socket_t Socket::create(const char *iface, const char *port, int family, int typ
     }
     setsockopt(so, SOL_SOCKET, SO_REUSEADDR, (caddr_t)&reuse, sizeof(reuse));
     if(res->ai_addr) {
-        if(_bind_(so, res->ai_addr, (socklen_t)res->ai_addrlen)) {
+        if(::bind(so, res->ai_addr, (socklen_t)res->ai_addrlen)) {
             release(so);
             so = INVALID_SOCKET;
         }
@@ -1927,7 +1852,7 @@ int Socket::error(void)
 }
 #endif
 
-Socket::operator bool()
+Socket::operator bool() const
 {
     if(so == INVALID_SOCKET)
         return false;
@@ -1956,28 +1881,19 @@ size_t Socket::peek(void *data, size_t len) const
     if(iowait && iowait != Timer::inf && !Socket::wait(so, iowait))
         return 0;
 
-    ssize_t rtn = _recv_(so, (caddr_t)data, 1, MSG_DONTWAIT | MSG_PEEK);
+    ssize_t rtn = ::recv(so, (caddr_t)data, 1, MSG_DONTWAIT | MSG_PEEK);
     if(rtn < 1)
         return 0;
     return (size_t)rtn;
 }
 
-ssize_t Socket::recvinet(socket_t so, void *data, size_t len, int flags, struct sockaddr_internet *addr)
-{
-    assert(data != NULL);
-    assert(len > 0);
-
-    socklen_t slen = sizeof(struct sockaddr_internet);
-    return _recvfrom_(so, (caddr_t)data, (socksize_t)len, flags, (struct sockaddr *)addr, &slen);
-}
-
 ssize_t Socket::recvfrom(socket_t so, void *data, size_t len, int flags, struct sockaddr_storage *addr)
 {
     assert(data != NULL);
     assert(len > 0);
 
     socklen_t slen = sizeof(struct sockaddr_storage);
-    return _recvfrom_(so, (caddr_t)data, (socksize_t)len, flags, (struct sockaddr *)addr, (socklen_t *)&slen);
+    return ::recvfrom(so, (caddr_t)data, (socksize_t)len, flags, (struct sockaddr *)addr, (socklen_t *)&slen);
 }
 
 size_t Socket::readfrom(void *data, size_t len, struct sockaddr_storage *from)
@@ -1990,7 +1906,7 @@ size_t Socket::readfrom(void *data, size_t len, struct sockaddr_storage *from)
         return 0;
 
     socklen_t slen = sizeof(struct sockaddr_storage);
-    ssize_t result = _recvfrom_(so, (caddr_t)data, (socksize_t)len, 0, (struct sockaddr *)from, (socklen_t *)&slen);
+    ssize_t result = ::recvfrom(so, (caddr_t)data, (socksize_t)len, 0, (struct sockaddr *)from, (socklen_t *)&slen);
 
     if(result < 0) {
         ioerr = Socket::error();
@@ -2008,7 +1924,7 @@ size_t Socket::writeto(const void *data, size_t dlen, const struct sockaddr *des
     if(dest)
         slen = len(dest);
 
-    ssize_t result = _sendto_(so, (caddr_t)data, (socksize_t)dlen, MSG_NOSIGNAL, dest, (socklen_t)slen);
+    ssize_t result = ::sendto(so, (caddr_t)data, (socksize_t)dlen, MSG_NOSIGNAL, dest, (socklen_t)slen);
 
     if(result < 0) {
         ioerr = Socket::error();
@@ -2026,7 +1942,7 @@ ssize_t Socket::sendto(socket_t so, const void *data, size_t dlen, int flags, co
     if(dest)
         slen = len(dest);
 
-    return _sendto_(so, (caddr_t)data, (socksize_t)dlen, MSG_NOSIGNAL | flags, dest, (socklen_t)slen);
+    return ::sendto(so, (caddr_t)data, (socksize_t)dlen, MSG_NOSIGNAL | flags, dest, (socklen_t)slen);
 }
 
 size_t Socket::writes(const char *str)
@@ -2057,10 +1973,10 @@ size_t Socket::readline(char *data, size_t max)
 
 size_t Socket::readline(String& s)
 {
-    if(!s.c_mem())
+    if(!s.data())
         return 0;
 
-    ssize_t result = Socket::readline(so, s.c_mem(), s.size() + 1, iowait);
+    ssize_t result = Socket::readline(so, s.data(), s.size() + 1, iowait);
     if(result < 0) {
         ioerr = Socket::error();
         s.clear();
@@ -2070,6 +1986,21 @@ size_t Socket::readline(String& s)
     return (size_t)result;
 }
 
+stringref_t Socket::readline(size_t size)
+{
+	charvalues_t buf = stringref<auto_release>::create(size);
+	if(!buf)
+		return stringref_t();
+
+	ssize_t result = Socket::readline(so, buf->get(), buf->max() + 1, iowait);
+	if(result < 0)
+		return stringref_t();
+
+	stringref_t out;
+	out.assign(buf);
+	return out;
+}
+
 ssize_t Socket::readline(socket_t so, char *data, size_t max, timeout_t timeout)
 {
     assert(data != NULL);
@@ -2089,7 +2020,7 @@ ssize_t Socket::readline(socket_t so, char *data, size_t max, timeout_t timeout)
             if(!wait(so, timeout))
                 return 0;
         }
-        nstat = _recv_(so, data, (socksize_t)nleft, MSG_PEEK);
+        nstat = ::recv(so, data, (socksize_t)nleft, MSG_PEEK);
         if(nstat < 0)
             return -1;
 
@@ -2106,7 +2037,7 @@ ssize_t Socket::readline(socket_t so, char *data, size_t max, timeout_t timeout)
             }
         }
 
-        nstat = _recv_(so, (caddr_t)data, c, 0);
+        nstat = ::recv(so, (caddr_t)data, c, 0);
         if(nstat < 0)
             break;
 
@@ -2144,7 +2075,7 @@ int Socket::loopback(socket_t so, bool enable)
     if(so == INVALID_SOCKET)
         return EBADF;
 
-    _getsockname_(so, addr, &len);
+    ::getsockname(so, addr, &len);
     family = us.inaddr.sin_family;
     switch(family) {
     case AF_INET:
@@ -2163,7 +2094,7 @@ int Socket::loopback(socket_t so, bool enable)
     return err;
 }
 
-int Socket::ttl(socket_t so, unsigned char t)
+int Socket::ttl(socket_t so, uint8_t t)
 {
     union {
         struct sockaddr_storage saddr;
@@ -2177,7 +2108,7 @@ int Socket::ttl(socket_t so, unsigned char t)
     if(so == INVALID_SOCKET)
         return EBADF;
 
-    _getsockname_(so, addr, &len);
+    ::getsockname(so, addr, &len);
     family = us.inaddr.sin_family;
     switch(family) {
     case AF_INET:
@@ -2295,7 +2226,7 @@ int Socket::multicast(socket_t so, unsigned ttl)
     else
         enable = false;
 
-    _getsockname_(so, (struct sockaddr *)&addr, &len);
+    ::getsockname(so, (struct sockaddr *)&addr, &len);
     if(!enable)
         switch(addr.address.sa_family)
         {
@@ -2410,7 +2341,7 @@ int Socket::disconnect(socket_t so)
     socklen_t len = sizeof(us.saddr);
 
 #if defined(_MSWINDOWS_)
-    _getsockname_(so, addr, &len);
+    ::getsockname(so, addr, &len);
     int family = us.inaddr.sin_family;
     memset(addr, 0, sizeof(us.saddr));
     us.inaddr.sin_family = family;
@@ -2422,7 +2353,7 @@ int Socket::disconnect(socket_t so)
         len = sizeof(us.saddr);
     if(so == INVALID_SOCKET)
         return EBADF;
-    if(!_connect_(so, addr, len))
+    if(!::connect(so, addr, len))
         return 0;
     int err = Socket::error();
     if(!err)
@@ -2444,7 +2375,7 @@ int Socket::join(socket_t so, const struct addrinfo *node, const int ifindex)
     if(so == INVALID_SOCKET)
         return EBADF;
 
-    _getsockname_(so, (struct sockaddr *)&addr, &len);
+    ::getsockname(so, (struct sockaddr *)&addr, &len);
     while(!rtn && node && node->ai_addr) {
         target = (const struct sockaddr_internet *)node->ai_addr;
         family = node->ai_family;
@@ -2492,7 +2423,7 @@ int Socket::drop(socket_t so, const struct addrinfo *node, const int ifindex)
     if(so == INVALID_SOCKET)
         return EBADF;
 
-    _getsockname_(so, (struct sockaddr *)&addr, &len);
+    ::getsockname(so, (struct sockaddr *)&addr, &len);
     while(!rtn && node && node->ai_addr) {
         target = (const struct sockaddr_internet *)node->ai_addr;
         family = node->ai_family;
@@ -2558,7 +2489,7 @@ socket_t Socket::create(const struct addrinfo *node, int stype, int sprotocol)
             so = Socket::create(sfamily, ctype, cprotocol);
         }
         if(so != INVALID_SOCKET) {
-            if(!_connect_(so, node->ai_addr, (socklen_t)node->ai_addrlen))
+            if(!::connect(so, node->ai_addr, (socklen_t)node->ai_addrlen))
                 return so;
         }
 next:
@@ -2593,7 +2524,7 @@ int Socket::connectto(socket_t so, struct addrinfo *node)
 
     while(node) {
         if(node->ai_family == socket_family) {
-            if(!_connect_(so, node->ai_addr, (socklen_t)node->ai_addrlen)) {
+            if(!::connect(so, node->ai_addr, (socklen_t)node->ai_addrlen)) {
                 rtn = 0;
                 goto exit;
             }
@@ -2614,7 +2545,7 @@ exit:
     return rtn;
 }
 
-int Socket::error(socket_t so)
+int Socket::error(const socket_t so)
 {
     assert(so != INVALID_SOCKET);
 
@@ -2683,7 +2614,7 @@ bool Socket::connected(void) const
     if(!wait())
         return true;
 
-    if(_recv_(so, &buf, 1, MSG_DONTWAIT | MSG_PEEK) < 1)
+    if(::recv(so, &buf, 1, MSG_DONTWAIT | MSG_PEEK) < 1)
         return false;
 
     return true;
@@ -2726,9 +2657,9 @@ socket_t Socket::acceptfrom(socket_t so, struct sockaddr_storage *addr)
 {
     socklen_t len = sizeof(struct sockaddr_storage);
     if(addr)
-        return _accept_(so, (struct sockaddr *)addr, &len);
+        return ::accept(so, (struct sockaddr *)addr, &len);
     else
-        return _accept_(so, NULL, NULL);
+        return ::accept(so, NULL, NULL);
 }
 
 bool Socket::wait(timeout_t timeout) const
@@ -2753,9 +2684,9 @@ bool Socket::wait(socket_t so, timeout_t timeout)
     status = 0;
     while(status < 1) {
         if(timeout == Timer::inf)
-            status = _poll_(&pfd, 1, -1);
+            status = ::poll(&pfd, 1, -1);
         else
-            status = _poll_(&pfd, 1, timeout);
+            status = ::poll(&pfd, 1, timeout);
         if(status == -1 && errno == EINTR)
             continue;
         if(status < 0)
@@ -2782,7 +2713,7 @@ bool Socket::wait(socket_t so, timeout_t timeout)
 
     FD_ZERO(&grp);
     FD_SET(so, &grp);
-    status = _select_((int)(so + 1), &grp, NULL, NULL, tvp);
+    status = ::select((int)(so + 1), &grp, NULL, NULL, tvp);
     if(status < 1)
         return false;
     if(FD_ISSET(so, &grp))
@@ -2807,9 +2738,9 @@ bool Socket::waitSending(timeout_t timeout) const
     status = 0;
     while(status < 1) {
         if(timeout == Timer::inf)
-            status = _poll_(&pfd, 1, -1);
+            status = ::poll(&pfd, 1, -1);
         else
-            status = _poll_(&pfd, 1, timeout);
+            status = ::poll(&pfd, 1, timeout);
         if(status == -1 && errno == EINTR)
             continue;
         if(status < 0)
@@ -2836,7 +2767,7 @@ bool Socket::waitSending(timeout_t timeout) const
 
     FD_ZERO(&grp);
     FD_SET(so, &grp);
-    status = _select_((int)(so + 1), NULL, &grp, NULL, tvp);
+    status = ::select((int)(so + 1), NULL, &grp, NULL, tvp);
     if(status < 1)
         return false;
     if(FD_ISSET(so, &grp))
@@ -2868,7 +2799,7 @@ socket_t ListenSocket::create(const char *iface, const char *svc, unsigned backl
     if(so == INVALID_SOCKET)
         return so;
 
-    if(_listen_(so, backlog)) {
+    if(::listen(so, backlog)) {
         release(so);
         return INVALID_SOCKET;
     }
@@ -2879,9 +2810,9 @@ socket_t ListenSocket::accept(struct sockaddr_storage *addr) const
 {
     socklen_t len = sizeof(struct sockaddr_storage);
     if(addr)
-        return _accept_(so, (struct sockaddr *)addr, &len);
+        return ::accept(so, (struct sockaddr *)addr, &len);
     else
-        return _accept_(so, NULL, NULL);
+        return ::accept(so, NULL, NULL);
 }
 
 TCPServer::TCPServer(const char *address, const char *service, unsigned backlog) :
@@ -2906,7 +2837,7 @@ struct ::addrinfo *Socket::hinting(socket_t so, struct addrinfo *hint)
 
     memset(hint, 0, sizeof(struct addrinfo));
     memset(sa, 0, sizeof(us.st));
-    if(_getsockname_(so, sa, &slen))
+    if(::getsockname(so, sa, &slen))
         return NULL;
     hint->ai_family = us.in.sin_family;
     slen = sizeof(hint->ai_socktype);
@@ -2914,7 +2845,7 @@ struct ::addrinfo *Socket::hinting(socket_t so, struct addrinfo *hint)
     return hint;
 }
 
-int Socket::type(socket_t so)
+int Socket::type(const socket_t so)
 {
     int sotype;
     socklen_t slen = sizeof(sotype);
@@ -3049,16 +2980,16 @@ exit:
 
 int Socket::bindto(socket_t so, const struct sockaddr *iface)
 {
-    if(!_bind_(so, iface, len(iface)))
+    if(!::bind(so, iface, len(iface)))
         return 0;
     return Socket::error();
 }
 
 int Socket::listento(socket_t so, const struct sockaddr *iface, int backlog)
 {
-    if(_bind_(so, iface, len(iface)))
+    if(::bind(so, iface, len(iface)))
         return Socket::error();
-    if(_listen_(so, backlog))
+    if(::listen(so, backlog))
         return Socket::error();
     return 0;
 }
@@ -3080,7 +3011,7 @@ int Socket::bindto(socket_t so, const char *host, const char *svc, int protocol)
     if(host && strchr(host, '/')) {
         struct sockaddr_storage uaddr;
         socklen_t len = unixaddr((struct sockaddr_un *)&uaddr, host);
-        rtn = _bind_(so, (struct sockaddr *)&uaddr, len);
+        rtn = ::bind(so, (struct sockaddr *)&uaddr, len);
         goto exit;
     };
 #endif
@@ -3114,7 +3045,7 @@ int Socket::bindto(socket_t so, const char *host, const char *svc, int protocol)
     if(rtn)
         goto exit;
 
-    rtn = _bind_(so, res->ai_addr, (socklen_t)res->ai_addrlen);
+    rtn = ::bind(so, res->ai_addr, (socklen_t)res->ai_addrlen);
 exit:
     if(res)
         freeaddrinfo(res);
@@ -3165,13 +3096,13 @@ switch(addr->sa_family) {
     case AF_INET6:
         cp = (caddr_t)(&((const struct sockaddr_in6 *)(addr))->sin6_addr);
         len = 16;
-        key = service(addr);
+        key = port(addr);
         break;
 #endif
     case AF_INET:
         cp = (caddr_t)(&((const struct sockaddr_in *)(addr))->sin_addr);
         len = 4;
-        key = service(addr);
+        key = port(addr);
         break;
     default:
         return 0;
@@ -3183,9 +3114,10 @@ switch(addr->sa_family) {
     return key % keysize;
 }
 
-short Socket::service(const struct sockaddr *addr)
+in_port_t Socket::port(const struct sockaddr *addr)
 {
-    assert(addr != NULL);
+	if(!addr)
+		return 0;
 
     switch(addr->sa_family) {
 #ifdef  AF_INET6
@@ -3254,7 +3186,7 @@ char *Socket::query(const struct sockaddr *addr, char *name, socklen_t size)
     return NULL;
 }
 
-int Socket::via(struct sockaddr *iface, const struct sockaddr *dest)
+int Socket::via(struct sockaddr *iface, const struct sockaddr *dest, socklen_t size)
 {
     assert(iface != NULL);
     assert(dest != NULL);
@@ -3263,8 +3195,11 @@ int Socket::via(struct sockaddr *iface, const struct sockaddr *dest)
     socket_t so = INVALID_SOCKET;
     socklen_t slen = len(dest);
 
-    if(slen)
-        memset(iface, 0, slen);
+    if(size)
+        memset(iface, 0, size);
+
+	if(size && size < slen)
+		return ENOMEM;
 
     iface->sa_family = AF_UNSPEC;
     switch(dest->sa_family) {
@@ -3276,8 +3211,8 @@ int Socket::via(struct sockaddr *iface, const struct sockaddr *dest)
         if((socket_t)so == INVALID_SOCKET)
             return -1;
         socket_mapping(dest->sa_family, so);
-        if(!_connect_(so, dest, slen))
-            rtn = _getsockname_(so, iface, &slen);
+        if(!::connect(so, dest, slen))
+            rtn = ::getsockname(so, iface, &slen);
         break;
     default:
         return ENOSYS;
@@ -3310,15 +3245,15 @@ bool Socket::eq_subnet(const struct sockaddr *s1, const struct sockaddr *s2)
 {
     assert(s1 != NULL && s2 != NULL);
 
-    unsigned char *a1, *a2;
+    uint8_t *a1, *a2;
     if(s1->sa_family != s2->sa_family)
         return false;
 
     if(s1->sa_family != AF_INET)
         return true;
 
-    a1 = (unsigned char *)&(((const struct sockaddr_in *)(s1))->sin_addr);
-    a2 = (unsigned char *)&(((const struct sockaddr_in *)(s1))->sin_addr);
+    a1 = (uint8_t *)&(((const struct sockaddr_in *)(s1))->sin_addr);
+    a2 = (uint8_t *)&(((const struct sockaddr_in *)(s1))->sin_addr);
 
     if(*a1 == *a2 && *a1 < 128)
         return true;
@@ -3502,7 +3437,7 @@ int Socket::family(socket_t so)
 
     socklen_t len = sizeof(us.saddr);
 	struct sockaddr *addr = (struct sockaddr *)(&us.saddr);
-    if(_getsockname_(so, addr, &len))
+    if(::getsockname(so, addr, &len))
         return AF_UNSPEC;
 
     return us.inaddr.sin_family;
@@ -3559,30 +3494,30 @@ int Socket::local(socket_t sock, struct sockaddr_storage *addr)
 {
     socklen_t slen = sizeof(sockaddr_storage);
 	memset(addr, 0, slen);
-    return _getsockname_(sock, (struct sockaddr *)addr, &slen);
+    return ::getsockname(sock, (struct sockaddr *)addr, &slen);
 }
 
 int Socket::remote(socket_t sock, struct sockaddr_storage *addr)
 {
     socklen_t slen = sizeof(sockaddr_storage);
 	memset(addr, 0, slen);
-    return _getpeername_(sock, (struct sockaddr *)addr, &slen);
+    return ::getpeername(sock, (struct sockaddr *)addr, &slen);
 }
 
-String str(Socket& so, strsize_t size)
+String str(Socket& so, size_t size)
 {
     String s(size);
-    so.readline(s.c_mem(), s.size());
+    so.readline(s.data(), s.size());
     String::fix(s);
     return s;
 }
 
-struct sockaddr *_getaddrinfo(struct addrinfo *list)
+const struct sockaddr *linked_sockaddr_operations::_getaddrinfo(const struct addrinfo *list) const
 {
     return list->ai_addr;
 }
 
-struct addrinfo *_nextaddrinfo(struct addrinfo *list)
+const struct addrinfo *linked_sockaddr_operations::_nextaddrinfo(const struct addrinfo *list) const
 {
     if(!list)
         return NULL;
@@ -3590,7 +3525,7 @@ struct addrinfo *_nextaddrinfo(struct addrinfo *list)
     return list->ai_next;
 }
 
-socket_t _getaddrsock(struct addrinfo *list)
+socket_t linked_sockaddr_operations::_getaddrsock(const struct addrinfo *list) const
 {
     if(!list)
         return INVALID_SOCKET;
