@@ -47,6 +47,12 @@ using std::iostream;
 using std::streambuf;
 using std::ios;
 
+#undef  EOF
+#define EOF         std::streambuf::traits_type::eof()
+#define IS_EOF(x)    std::streambuf::traits_type::eq_int_type(x, EOF)
+#define GET(x)      std::streambuf::traits_type::to_int_type(x)
+#define PUT(x)      std::streambuf::traits_type::to_char_type(x)
+
 StreamBuffer::StreamBuffer() :
 streambuf(), iostream((streambuf *)this)
 {
@@ -58,7 +64,7 @@ int StreamBuffer::uflow()
 {
     int ret = underflow();
 
-    if (ret == EOF)
+    if(IS_EOF(ret))
         return EOF;
 
     if (bufsize != 1)
@@ -187,7 +193,7 @@ ssize_t tcpstream::_write(const char *buffer, size_t size)
 int tcpstream::underflow(void)
 {
     ssize_t rlen = 1;
-    unsigned char ch;
+    char ch;
 
     if(bufsize == 1) {
         if(!_wait()) {
@@ -195,20 +201,20 @@ int tcpstream::underflow(void)
             return EOF;
         }
         else
-            rlen = _read((char *)&ch, 1);
+            rlen = _read(&ch, 1);
         if(rlen < 1) {
             if(rlen < 0)
                 reset();
             return EOF;
         }
-        return ch;
+        return GET(ch);
     }
 
     if(!gptr())
         return EOF;
 
     if(gptr() < egptr())
-        return (unsigned char)*gptr();
+        return GET(*gptr());
 
     rlen = (ssize_t)((gbuf + bufsize) - eback());
     if(!_wait()) {
@@ -228,19 +234,19 @@ int tcpstream::underflow(void)
     }
 
     setg(eback(), eback(), eback() + rlen);
-    return (unsigned char) *gptr();
+    return GET(*gptr());
 }
 
 int tcpstream::overflow(int c)
 {
-    unsigned char ch;
+    char ch;
     ssize_t rlen, req;
 
     if(bufsize == 1) {
-        if(c == EOF)
-            return 0;
+        if(IS_EOF(c))
+            return EOF;
 
-        ch = (unsigned char)(c);
+        ch = PUT(c);
         rlen = _write((const char *)&ch, 1);
         if(rlen < 1) {
             if(rlen < 0)
@@ -273,7 +279,7 @@ int tcpstream::overflow(int c)
     pbump(req);
 
     if(c != EOF) {
-        *pptr() = (unsigned char)c;
+        *pptr() = PUT(c);
         pbump(1);
     }
     return c;
@@ -465,7 +471,7 @@ void pipestream::allocate(size_t size, access_t mode)
 int pipestream::underflow(void)
 {
     ssize_t rlen = 1;
-    unsigned char ch;
+    char ch;
 
     if(!gbuf)
         return EOF;
@@ -477,14 +483,14 @@ int pipestream::underflow(void)
                 close();
             return EOF;
         }
-        return ch;
+        return GET(ch);
     }
 
     if(!gptr())
         return EOF;
 
     if(gptr() < egptr())
-        return (unsigned char)*gptr();
+        return GET(*gptr());
 
     rlen = (ssize_t)((gbuf + bufsize) - eback());
     rlen = rd.read(eback(), rlen);
@@ -498,22 +504,22 @@ int pipestream::underflow(void)
     }
 
     setg(eback(), eback(), eback() + rlen);
-    return (unsigned char) *gptr();
+    return GET(*gptr());
 }
 
 int pipestream::overflow(int c)
 {
-    unsigned char ch;
+    char ch;
     ssize_t rlen, req;
 
     if(!pbuf)
         return EOF;
 
     if(bufsize == 1) {
-        if(c == EOF)
-            return 0;
+        if(IS_EOF(c))
+            return EOF;
 
-        ch = (unsigned char)(c);
+        ch = PUT(c);
         rlen = wr.write(&ch, 1);
         if(rlen < 1) {
             if(rlen < 0)
@@ -546,7 +552,7 @@ int pipestream::overflow(int c)
     pbump(req);
 
     if(c != EOF) {
-        *pptr() = (unsigned char)c;
+        *pptr() = PUT(c);
         pbump(1);
     }
     return c;
@@ -650,6 +656,14 @@ void filestream::seek(fsys::offset_t offset)
     }
 }
 
+void filestream::rewind(void)
+{
+    sync();
+    if(bufsize) {
+        fd.seek(0);
+    }
+}
+
 void filestream::close(void)
 {
     sync();
@@ -747,7 +761,7 @@ int filestream::underflow(void)
         return EOF;
 
     if(gptr() < egptr())
-        return (unsigned char)*gptr();
+        return GET(*gptr());
 
     rlen = (ssize_t)((gbuf + bufsize) - eback());
     rlen = fd.read(eback(), rlen);
@@ -761,7 +775,7 @@ int filestream::underflow(void)
     }
 
     setg(eback(), eback(), eback() + rlen);
-    return (unsigned char) *gptr();
+    return GET(*gptr());
 }
 
 int filestream::overflow(int c)
@@ -793,12 +807,100 @@ int filestream::overflow(int c)
     pbump(req);
 
     if(c != EOF) {
-        *pptr() = (unsigned char)c;
+        *pptr() = PUT(c);
         pbump(1);
     }
     return c;
 }
 
+omemstream::omemstream(char *mem, size_t size) :
+std::streambuf(), std::ostream(this)
+{
+    count = size;
+    pos = (uint8_t *)mem;
+    bp = (uint8_t *)mem;
+    zb = true;
+    *mem = 0;
+    --count;
+}
+
+omemstream::omemstream(uint8_t *mem, size_t size) :
+std::streambuf(), std::ostream(this)
+{
+    count = size;
+    pos = mem;
+    bp = mem;
+    zb = false;
+}
+
+int omemstream::overflow(int ch)
+{
+    if(ch == EOF)
+        return EOF;
+
+    if(!count || !pos)
+        return EOF;
+
+    --count;
+    *(pos++) = PUT(ch);
+    if(zb)
+        *(pos) = 0;
+    return ch;
+}
+
+imemstream::imemstream(const char *str) :
+std::streambuf(), std::istream(this)
+{
+    bp = (const uint8_t *)str;
+    count = strlen(str);
+    pos = (const uint8_t *)str;
+}
+
+imemstream::imemstream(const uint8_t *str, size_t size) :
+std::streambuf(), std::istream(this)
+{
+    bp = str;
+    pos = str;
+    count = size;
+}
+
+int imemstream::underflow() 
+{
+    if(!count || !pos)
+        return EOF;
+    return GET(*pos);
+}
+
+int imemstream::uflow()
+{
+    if(!count || !pos)
+        return EOF;
+    --count;
+    return GET(*(pos++));
+} 
+
+bool getline(std::istream& in, char *buffer, size_t size)
+{
+    *buffer = 0;
+    if(!in.good())
+        return false;
+
+    in.getline(buffer, size);
+    if(!buffer[0])
+        return false;
+
+    return true;
+}
+
+bool putline(std::ostream& out, char *buffer)
+{
+    if(!out.good())
+        return false;
+
+    out << buffer << std::endl;
+    return out.good();
+}
+
 std::istream& _stream_operators::input(std::istream& inp, InputProtocol& fmt)
 {
     int c = 0;
@@ -832,7 +934,7 @@ std::ostream& _stream_operators::print(std::ostream& out, const PrintProtocol& f
 
 std::istream& _stream_operators::input(std::istream& inp, string_t& str)
 {
-    inp.getline(str.c_mem(), str.size());
+    inp.getline(str.data(), str.size());
     return inp;
 }
 
@@ -882,7 +984,52 @@ std::string& _stream_operators::append(std::string& target, String& source)
     return target;
 }
 
+class __LOCAL NullBuffer : public std::streambuf
+{
+private:
+	__DELETE_COPY(NullBuffer);
+
+    NullBuffer();
+
+public:
+	int overflow(int c) __OVERRIDE;
+
+    int underflow() __OVERRIDE;
+
+    int uflow() __OVERRIDE;
+
+	static NullBuffer null;
+};
+
+NullBuffer::NullBuffer() : streambuf()
+{
+}
+
+int NullBuffer::overflow(int c)
+{
+    return c;
+}
+
+int NullBuffer::underflow()
+{
+    return EOF;
+}
+
+int NullBuffer::uflow()
+{
+    return EOF;
+}
+
+NullBuffer NullBuffer::null;
+
+static std::iostream nullstream(&NullBuffer::null);
+
 } // namespace ucommon
 
+namespace std {
+    using namespace ucommon;
+    iostream& null = nullstream;
+}
+
 #endif
 
