@@ -29,63 +29,29 @@
 
 namespace ucommon {
 
-#if _MSC_VER > 1400        // windows broken dll linkage issue...
-#else
-const strsize_t String::npos = (strsize_t)(-1);
-const size_t memstring::header = sizeof(cstring);
-const char String::eos = '\0';
-#endif
-
-String::cstring::cstring(strsize_t size) :
+String::cstring::cstring(size_t size) :
 CountedObject()
 {
     max = size;
     len = 0;
-    fill = 0;
     text[0] = 0;
 }
 
-String::cstring::cstring(strsize_t size, char f) :
-CountedObject()
-{
-    max = size;
-    len = 0;
-    fill = f;
-
-    if(fill) {
-        memset(text, fill, max);
-        len = max;
-    }
-}
-
 void String::cstring::fix(void)
 {
-    while(fill && len < max)
-        text[len++] = fill;
     text[len] = 0;
 }
 
-void String::cstring::unfix(void)
+void String::cstring::clear(size_t offset)
 {
-    while(len && fill) {
-        if(text[len - 1] == fill)
-            --len;
-    }
-    text[len] = 0;
-}
-
-void String::cstring::clear(strsize_t offset, strsize_t size)
-{
-    if(!fill || offset >= max)
+    if(offset >= len)
         return;
 
-    while(size && offset < max) {
-        text[offset++] = fill;
-        --size;
-    }
+    text[offset] = 0;
+    len = offset;
 }
 
-void String::cstring::dec(strsize_t offset)
+void String::cstring::dec(size_t offset)
 {
     if(!len)
         return;
@@ -97,20 +63,10 @@ void String::cstring::dec(strsize_t offset)
         return;
     }
 
-    if(!fill) {
-        text[--len] = 0;
-        return;
-    }
-
-    while(len) {
-        if(text[--len] == fill)
-            break;
-    }
-    text[len] = 0;
-    fix();
+    text[--len] = 0;
 }
 
-void String::cstring::inc(strsize_t offset)
+void String::cstring::inc(size_t offset)
 {
     if(!offset)
         ++offset;
@@ -129,14 +85,11 @@ void String::cstring::inc(strsize_t offset)
 
 void String::cstring::add(const char *str)
 {
-    strsize_t size = (strsize_t)strlen(str);
+    size_t size = strlen(str);
 
     if(!size)
         return;
 
-    while(fill && len && text[len - 1] == fill)
-        --len;
-
     if(len + size > max)
         size = max - len;
 
@@ -153,9 +106,6 @@ void String::cstring::add(char ch)
     if(!ch)
         return;
 
-    while(fill && len && text[len - 1] == fill)
-        --len;
-
     if(len == max)
         return;
 
@@ -163,7 +113,7 @@ void String::cstring::add(char ch)
     fix();
 }
 
-void String::cstring::set(strsize_t offset, const char *str, strsize_t size)
+void String::cstring::set(size_t offset, const char *str, size_t size)
 {
     assert(str != NULL);
 
@@ -178,26 +128,13 @@ void String::cstring::set(strsize_t offset, const char *str, strsize_t size)
         --size;
     }
 
-    while(size && fill) {
-        text[offset++] = fill;
-        --size;
-    }
-
     if(offset > len) {
         len = offset;
         text[len] = 0;
     }
 }
 
-char String::fill(void)
-{
-    if(!str)
-        return 0;
-
-    return str->fill;
-}
-
-strsize_t String::size(void) const
+size_t String::size(void) const
 {
     if(!str)
         return 0;
@@ -205,7 +142,7 @@ strsize_t String::size(void) const
     return str->max;
 }
 
-strsize_t String::len(void) const
+size_t String::len(void) const
 {
     if(!str)
         return 0;
@@ -217,7 +154,7 @@ void String::cstring::set(const char *str)
 {
     assert(str != NULL);
 
-    strsize_t size = (strsize_t)strlen(str);
+    size_t size = strlen(str);
     if(size > max)
         size = max;
 
@@ -236,14 +173,14 @@ String::String()
 
 String::String(const char *s, const char *end)
 {
-    strsize_t size = 0;
+    size_t size = 0;
 
     if(!s)
         s = "";
     else if(!end)
-        size = (strsize_t)strlen(s);
+        size = strlen(s);
     else if(end > s)
-        size = (strsize_t)(end - s);
+        size = (size_t)(end - s);
     str = create(size);
     str->retain();
     str->set(s);
@@ -251,7 +188,7 @@ String::String(const char *s, const char *end)
 
 String::String(const char *s)
 {
-    strsize_t size = (strsize_t)count(s);
+    size_t size = count(s);
     if(!s)
         s = "";
     str = create(size);
@@ -259,48 +196,24 @@ String::String(const char *s)
     str->set(s);
 }
 
-String::String(const char *s, strsize_t size)
+String::String(const char *s, size_t size)
 {
     if(!s)
         s = "";
     if(!size)
-        size = (strsize_t)strlen(s);
+        size = strlen(s);
     str = create(size);
     str->retain();
     str->set(s);
 }
 
-String::String(strsize_t size)
+String::String(size_t size)
 {
     str = create(size);
     str->retain();
 }
 
-String::String(long value)
-{
-    str = create(20);
-    str->retain();
-    snprintf(&str->text[0], 20, "%ld", value);
-    str->len = (strsize_t)strlen(str->text);
-    str->fix();
-}
-
-String::String(double value)
-{
-    str = create(32);
-    str->retain();
-    snprintf(&str->text[0], 32, "%f", value);
-    str->len = (strsize_t)strlen(str->text);
-    str->fix();
-}
-
-String::String(strsize_t size, char fill)
-{
-    str = create(size, fill);
-    str->retain();
-}
-
-String::String(strsize_t size, const char *format, ...)
+String::String(size_t size, const char *format, ...)
 {
     assert(format != NULL);
 
@@ -330,7 +243,7 @@ String::cstring *String::c_copy(void) const
     return str;
 }
 
-String String::get(strsize_t offset, strsize_t len) const
+String String::get(size_t offset, size_t len) const
 {
     if(!str || offset >= str->len)
         return String("");
@@ -340,13 +253,10 @@ String String::get(strsize_t offset, strsize_t len) const
     return String(str->text + offset, len);
 }
 
-String::cstring *String::create(strsize_t size, char fill) const
+String::cstring *String::create(size_t size) const
 {
     void *mem = ::malloc(size + sizeof(cstring));
-    if(fill) 
-        return new(mem) cstring(size, fill);
-    else
-        return new(mem) cstring(size);
+    return new(mem) cstring(size);
 }
 
 void String::cstring::dealloc(void)
@@ -368,7 +278,7 @@ void String::release(void)
     str = NULL;
 }
 
-char *String::c_mem(void) const
+char *String::data(void)
 {
     if(!str)
         return NULL;
@@ -454,7 +364,7 @@ const char *String::rchr(char ch) const
     return ::strrchr(str->text, ch);
 }
 
-const char *String::skip(const char *clist, strsize_t offset) const
+const char *String::skip(const char *clist, size_t offset) const
 {
     if(!str || !clist || !*clist || !str->len || offset > str->len)
         return NULL;
@@ -467,7 +377,7 @@ const char *String::skip(const char *clist, strsize_t offset) const
     return NULL;
 }
 
-const char *String::rskip(const char *clist, strsize_t offset) const
+const char *String::rskip(const char *clist, size_t offset) const
 {
     if(!str || !clist || !*clist)
         return NULL;
@@ -485,7 +395,7 @@ const char *String::rskip(const char *clist, strsize_t offset) const
     return NULL;
 }
 
-const char *String::rfind(const char *clist, strsize_t offset) const
+const char *String::rfind(const char *clist, size_t offset) const
 {
     if(!str || !clist || !*clist)
         return NULL;
@@ -505,7 +415,7 @@ const char *String::rfind(const char *clist, strsize_t offset) const
 
 void String::chop(const char *clist)
 {
-    strsize_t offset;
+    size_t offset;
 
     if(!str)
         return;
@@ -568,17 +478,17 @@ unsigned String::replace(const char *substring, const char *cp, unsigned flags)
 {
     const char *result = "";
     unsigned count = 0;
-    strsize_t cpl = 0;
+    size_t cpl = 0;
 
     if(cp)
-        cpl = (strsize_t)strlen(cp);
+        cpl = strlen(cp);
 
     if(!str || !substring || str->len == 0)
         return 0;
 
-    strsize_t offset = 0;
+    size_t offset = 0;
 
-    strsize_t tcl = (strsize_t)strlen(substring);
+    size_t tcl = strlen(substring);
 
     while(result) {
         const char *text = str->text + offset;
@@ -597,7 +507,7 @@ unsigned String::replace(const char *substring, const char *cp, unsigned flags)
 
         if(result) {
             ++count;
-            offset = (strsize_t)(text - str->text);
+            offset = (size_t)(text - str->text);
             cut(offset, tcl);
             if(cpl) {
                 paste(offset, cp);
@@ -639,7 +549,7 @@ const char *String::search(const char *substring, unsigned instance, unsigned fl
     return result;
 }
 
-const char *String::find(const char *clist, strsize_t offset) const
+const char *String::find(const char *clist, size_t offset) const
 {
     if(!str || !clist || !*clist || !str->len || offset > str->len)
         return NULL;
@@ -661,7 +571,6 @@ bool String::unquote(const char *clist)
     if(!str)
         return false;
 
-    str->unfix();
     s = unquote(str->text, clist);
     if(!s) {
         str->fix();
@@ -692,7 +601,7 @@ void String::erase(void)
     }
 }
 
-strsize_t String::offset(const char *s) const
+size_t String::offset(const char *s) const
 {
     if(!str || !s)
         return npos;
@@ -700,20 +609,20 @@ strsize_t String::offset(const char *s) const
     if(s < str->text || s > str->text + str->max)
         return npos;
 
-    if(s - str->text > str->len)
+    if((size_t)(s - str->text) > str->len)
         return str->len;
 
-    return (strsize_t)(s - str->text);
+    return (size_t)(s - str->text);
 }
 
-strsize_t String::count(void) const
+size_t String::count(void) const
 {
     if(!str)
         return 0;
     return str->len;
 }
 
-strsize_t String::ccount(const char *clist) const
+size_t String::ccount(const char *clist) const
 {
     if(!str)
         return 0;
@@ -721,7 +630,7 @@ strsize_t String::ccount(const char *clist) const
     return ccount(str->text, clist);
 }
 
-strsize_t String::printf(const char *format, ...)
+size_t String::printf(const char *format, ...)
 {
     assert(format != NULL);
 
@@ -729,20 +638,20 @@ strsize_t String::printf(const char *format, ...)
     va_start(args, format);
     if(str) {
         vsnprintf(str->text, str->max + 1, format, args);
-        str->len = (strsize_t)strlen(str->text);
+        str->len = strlen(str->text);
         str->fix();
     }
     va_end(args);
     return len();
 }
 
-strsize_t String::vprintf(const char *format, va_list args)
+size_t String::vprintf(const char *format, va_list args)
 {
     assert(format != NULL);
 
     if(str) {
         vsnprintf(str->text, str->max + 1, format, args);
-        str->len = (strsize_t)strlen(str->text);
+        str->len = strlen(str->text);
         str->fix();
     }
     return len();
@@ -791,7 +700,7 @@ void String::rsplit(const char *s)
     str->set(s);
 }
 
-void String::rsplit(strsize_t pos)
+void String::rsplit(size_t pos)
 {
     if(!str || pos > str->len || !pos)
         return;
@@ -814,7 +723,7 @@ void String::split(const char *s)
     str->fix();
 }
 
-void String::split(strsize_t pos)
+void String::split(size_t pos)
 {
     if(!str || pos >= str->len)
         return;
@@ -823,18 +732,30 @@ void String::split(strsize_t pos)
     str->fix();
 }
 
-void String::set(strsize_t offset, const char *s, strsize_t size)
+void String::fill(size_t size, char fill)
+{
+    if(!str) {
+        str = create(size);
+        str->retain();
+    }
+    while(str->len < str->max && size--)
+        str->text[str->len++] = fill;
+    str->fix();
+}
+
+void String::set(size_t offset, const char *s, size_t size)
 {
     if(!s || !*s || !str)
         return;
 
     if(!size)
-        size = (strsize_t)strlen(s);
+        size = strlen(s);
 
-    str->set(offset, s, size);
+    if(str)
+        str->set(offset, s, size);
 }
 
-void String::set(const char *s, char overflow, strsize_t offset, strsize_t size)
+void String::set(const char *s, char overflow, size_t offset, size_t size)
 {
     size_t len = count(s);
 
@@ -858,11 +779,10 @@ void String::set(const char *s, char overflow, strsize_t offset, strsize_t size)
         str->text[offset + size - 1] = overflow;
 }
 
-void String::rset(const char *s, char overflow, strsize_t offset, strsize_t size)
+void String::rset(const char *s, char overflow, size_t offset, size_t size)
 {
-    strsize_t len = (strsize_t)count(s);
-    strsize_t dif;
-
+    size_t len = count(s);
+    
     if(!s || !*s || !str)
         return;
 
@@ -872,12 +792,6 @@ void String::rset(const char *s, char overflow, strsize_t offset, strsize_t size
     if(!size || size > str->max - offset)
         size = str->max - offset;
 
-    dif = len;
-    while(dif < size && str->fill) {
-        str->text[offset++] = str->fill;
-        ++dif;
-    }
-
     if(len > size)
         s += len - size;
     set(offset, s, size);
@@ -887,13 +801,13 @@ void String::rset(const char *s, char overflow, strsize_t offset, strsize_t size
 
 void String::set(const char *s)
 {
-    strsize_t len;
+    size_t len;
 
     if(!s)
         s = "";
 
     if(!str) {
-        len = (strsize_t)strlen(s);
+        len = strlen(s);
         str = create(len);
         str->retain();
     }
@@ -901,13 +815,13 @@ void String::set(const char *s)
     str->set(s);
 }
 
-void String::paste(strsize_t offset, const char *cp, strsize_t size)
+void String::paste(size_t offset, const char *cp, size_t size)
 {
     if(!cp)
         return;
 
     if(!size)
-        size = (strsize_t)strlen(cp);
+        size = strlen(cp);
 
     if(!size)
         return;
@@ -932,7 +846,7 @@ void String::paste(strsize_t offset, const char *cp, strsize_t size)
     str->fix();
 }
 
-void String::cut(strsize_t offset, strsize_t size)
+void String::cut(size_t offset, size_t size)
 {
     if(!str || offset >= str->len)
         return;
@@ -996,10 +910,8 @@ void String::cut(char *text, size_t offset, size_t size)
     text[len] = 0;
 }
 
-bool String::resize(strsize_t size)
+bool String::resize(size_t size)
 {
-    char fill = 0;
-
     if(!size) {
         release();
         str = NULL;
@@ -1007,27 +919,23 @@ bool String::resize(strsize_t size)
     }
 
     if(!str) {
-        str = create(size, fill);
+        str = create(size);
         str->retain();
     }
     else if(str->is_copied() || str->max < size) {
-        fill = str->fill;
         str->release();
-        str = create(size, fill);
+        str = create(size);
         str->retain();
     }
     return true;
 }
 
-void String::clear(strsize_t offset, strsize_t size)
+void String::clear(size_t offset)
 {
     if(!str)
         return;
 
-    if(!size)
-        size = str->max;
-
-    str->clear(offset, size);
+    str->clear(offset);
 }
 
 void String::clear(void)
@@ -1036,14 +944,10 @@ void String::clear(void)
         str->set("");
 }
 
-void String::cow(strsize_t size)
+void String::cow(size_t size)
 {
-    if(str) {
-        if(str->fill)
-            size = str->max;
-        else
-            size += str->len;
-    }
+    if(str)
+        size += str->len;
 
     if(!size)
         return;
@@ -1095,7 +999,7 @@ void String::add(const char *s)
         return;
     }
 
-    cow((strsize_t)strlen(s));
+    cow(strlen(s));
     str->add(s);
 }
 
@@ -1110,20 +1014,20 @@ char String::at(int offset) const
     if(offset > -1)
         return str->text[offset];
 
-    if((strsize_t)(-offset) >= str->len)
+    if((size_t)(-offset) >= str->len)
         return str->text[0];
 
     return str->text[(int)(str->len) + offset];
 }
 
-String String::operator()(int offset, strsize_t len) const
+String String::operator()(int offset, size_t len) const
 {
     const char *cp = operator()(offset);
     if(!cp)
         cp = "";
 
     if(!len)
-        len = (strsize_t)strlen(cp);
+        len = strlen(cp);
 
     return String(cp, len);
 }
@@ -1139,7 +1043,7 @@ const char *String::operator()(int offset) const
     if(offset > -1)
         return str->text + offset;
 
-    if((strsize_t)(-offset) >= str->len)
+    if((size_t)(-offset) >= str->len)
         return str->text;
 
     return str->text + str->len + offset;
@@ -1156,7 +1060,7 @@ const char String::operator[](int offset) const
     if(offset > -1)
         return str->text[offset];
 
-    if((strsize_t)(-offset) >= str->len)
+    if((size_t)(-offset) >= str->len)
         return *str->text;
 
     return str->text[str->len + offset];
@@ -1176,14 +1080,14 @@ String &String::operator--()
     return *this;
 }
 
-String &String::operator+=(strsize_t offset)
+String &String::operator+=(size_t offset)
 {
     if(str)
         str->inc(offset);
     return *this;
 }
 
-String &String::operator-=(strsize_t offset)
+String &String::operator-=(size_t offset)
 {
     if(str)
         str->dec(offset);
@@ -1251,8 +1155,7 @@ bool String::full(void) const
     if(!str)
         return false;
 
-    if(str->len == str->max &&
-       str->text[str->len - 1] != str->fill)
+    if(str->len == str->max)
         return true;
 
     return false;
@@ -1264,7 +1167,6 @@ bool String::operator!() const
     if(!str)
         return true;
 
-    str->unfix();
     if(!str->len)
         rtn = true;
 
@@ -1279,7 +1181,6 @@ String::operator bool() const
     if(!str)
         return false;
 
-    str->unfix();
     if(str->len)
         rtn = true;
     str->fix();
@@ -1336,7 +1237,7 @@ String &String::operator|(const char *s)
     return *this;
 }
 
-String String::operator+(const char *s)
+const String String::operator+(const char *s) const
 {
     String tmp;
     if(str && str->text[0])
@@ -1357,12 +1258,12 @@ String &String::operator+=(const char *s)
     return *this;
 }
 
-memstring::memstring(void *mem, strsize_t size, char fill)
+memstring::memstring(void *mem, size_t size)
 {
     assert(mem != NULL);
     assert(size > 0);
 
-    str = new(mem) cstring(size, fill);
+    str = new(mem) cstring(size);
     str->set("");
 }
 
@@ -1371,20 +1272,20 @@ memstring::~memstring()
     str = NULL;
 }
 
-memstring *memstring::create(strsize_t size, char fill)
+memstring *memstring::create(size_t size)
 {
     assert(size > 0);
 
     void *mem = ::malloc(size + sizeof(memstring) + sizeof(cstring));
-    return new(mem) memstring((caddr_t)mem + sizeof(memstring), size, fill);
+    return new(mem) memstring((caddr_t)mem + sizeof(memstring), size);
 }
 
-memstring *memstring::create(MemoryProtocol *mpager, strsize_t size, char fill)
+memstring *memstring::create(MemoryProtocol *mpager, size_t size)
 {
     assert(size > 0);
 
     void *mem = mpager->alloc(size + sizeof(memstring) + sizeof(cstring));
-    return new(mem) memstring((caddr_t)mem + sizeof(memstring), size, fill);
+    return new(mem) memstring((caddr_t)mem + sizeof(memstring), size);
 }
 
 void memstring::release(void)
@@ -1392,12 +1293,12 @@ void memstring::release(void)
     str = NULL;
 }
 
-bool memstring::resize(strsize_t size)
+bool memstring::resize(size_t size)
 {
     return false;
 }
 
-void memstring::cow(strsize_t adj)
+void memstring::cow(size_t adj)
 {
 }
 
@@ -1466,7 +1367,7 @@ char *String::token(char *text, char **token, const char *clist, const char *quo
 
 String::cstring *memstring::c_copy(void) const
 {
-    cstring *tmp = String::create(str->max, str->fill);
+    cstring *tmp = String::create(str->max);
     tmp->set(str->text);
     return tmp;
 }
@@ -1474,7 +1375,7 @@ String::cstring *memstring::c_copy(void) const
 void String::fix(String &s)
 {
     if(s.str) {
-        s.str->len = (strsize_t)strlen(s.str->text);
+        s.str->len = strlen(s.str->text);
         s.str->fix();
     }
 }
@@ -1495,7 +1396,8 @@ char *String::dup(const char *cp)
 
     size_t len = strlen(cp) + 1;
     mem = (char *)malloc(len);
-    crit(mem != NULL, "string dup allocation error");
+    if(!mem)
+        __THROW_ALLOC();
     String::set(mem, len, cp);
     return mem;
 }
@@ -1511,7 +1413,8 @@ char *String::left(const char *cp, size_t size)
         size = strlen(cp);
 
     mem = (char *)malloc(++size);
-    crit(mem != NULL, "string dup allocation error");
+    if(!mem)
+        __THROW_ALLOC();
     String::set(mem, size, cp);
     return mem;
 }
@@ -1881,36 +1784,6 @@ bool String::eq_case(const char *s1, const char *s2)
 #endif
 }
 
-int String::case_compare(const char *s1, const char *s2)
-{
-    if(!s1)
-        s1 = "";
-
-    if(!s2)
-        s2 = "";
-
-#ifdef  HAVE_STRICMP
-    return stricmp(s1, s2);
-#else
-    return strcasecmp(s1, s2);
-#endif
-}
-
-int String::case_compare(const char *s1, const char *s2, size_t size)
-{
-    if(!s1)
-        s1 = "";
-
-    if(!s2)
-        s2 = "";
-
-#ifdef  HAVE_STRICMP
-    return strnicmp(s1, s2, size);
-#else
-    return strncasecmp(s1, s2, size);
-#endif
-}
-
 bool String::eq_case(const char *s1, const char *s2, size_t size)
 {
     if(!s1)
@@ -1989,9 +1862,37 @@ char *String::fill(char *str, size_t size, char fill)
     return str;
 }
 
-unsigned String::hexsize(const char *format)
+static int hexcode(char ch)
 {
-    unsigned count = 0;
+    ch = toupper(ch);
+    if(ch >= '0' && ch <= '9')
+        return ch - '0';
+    else if(ch >= 'A' && ch <= 'F')
+        return ch - 'A' + 10;
+    return -1;  // error flag
+}
+
+size_t String::hexcount(const char *str, bool ws)
+{
+    size_t count = 0;
+
+    while(str && *str) {
+        if(ws && isspace(*str)) {
+            ++str;
+            continue;
+        }
+        if(hexcode(str[0]) < 0 || hexcode(str[1]) < 0)
+            break;
+        str += 2;
+        ++count;
+    }
+
+    return count;
+}
+
+size_t String::hexsize(const char *format)
+{
+    size_t count = 0;
     char *ep;
     unsigned skip;
 
@@ -2009,11 +1910,10 @@ unsigned String::hexsize(const char *format)
     return count;
 }
 
-String String::hex(const unsigned char *binary, size_t size)
+String String::hex(const uint8_t *binary, size_t size)
 {
-    strsize_t ssize = (strsize_t)(size * 2);
-    String out(ssize, ' ');
-    char *buf = out.c_mem();
+    String out(size * 2);
+    char *buf = out.data();
     while(size--) {
         snprintf(buf, 3, "%02x", *(binary++));
         buf += 2;
@@ -2021,9 +1921,9 @@ String String::hex(const unsigned char *binary, size_t size)
     return out;
 } 
 
-unsigned String::hexdump(const unsigned char *binary, char *string, const char *format)
+size_t String::hexdump(const uint8_t *binary, char *string, const char *format)
 {
-    unsigned count = 0;
+    size_t count = 0;
     char *ep;
     unsigned skip;
 
@@ -2046,17 +1946,33 @@ unsigned String::hexdump(const unsigned char *binary, char *string, const char *
     return count;
 }
 
-static unsigned hexcode(char ch)
+size_t String::hex2bin(const char *str, uint8_t *bin, size_t max, bool ws)
 {
-    if(ch >= '0' && ch <= '9')
-        return ch - '0';
-    else
-        return toupper(ch) - 'A' + 10;
+    size_t count = 0;
+    size_t out = 0;
+    int hi, lo;
+    while(str && *str) {
+        if(ws && isspace(*str)) {
+            ++count;
+            ++str;
+            continue;
+        }
+        hi = hexcode(str[0]);
+        lo = hexcode(str[1]);
+        if(hi < 0 || lo < 0)
+            break;
+        *(bin++) = (hi << 4) | lo;
+        str += 2;
+        count += 2;
+        if(++out > max)
+            break;
+    }
+    return count;
 }
 
-unsigned String::hexpack(unsigned char *binary, const char *string, const char *format)
+size_t String::hexpack(uint8_t *binary, const char *string, const char *format)
 {
-    unsigned count = 0;
+    size_t count = 0;
     char *ep;
     unsigned skip;
 
@@ -2181,7 +2097,7 @@ String &String::operator%(const char *get)
     if(!str || !str->text[0] || !get)
         return *this;
 
-    unsigned len = (strsize_t)strlen(get);
+    size_t len = strlen(get);
     const char *cp = str->text;
 
     while(isspace(*cp))
@@ -2195,44 +2111,15 @@ String &String::operator%(const char *get)
     return *this;
 }
 
-String str(CharacterProtocol *p, strsize_t size)
-{
-    String temp(size);
-    char *cp = temp.c_mem();
-    int ch;
-    bool cr = false;
-
-    while(--size) {
-        ch = p->getchar();
-        if(ch == 0 || ch == EOF || ch == '\n')
-            break;
-
-        if(cr) {
-            cr = false;
-            *(cp++) = '\r';
-        }
-
-        if(ch == '\r')
-            cr = true;
-        else
-            *(cp++) = ch;
-    }
-
-    *cp = 0;
-
-    String::fix(temp);
-    return temp;
-}
-
-static const unsigned char alphabet[65] =
+static const uint8_t alphabet[65] =
     "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 
 String String::b64(const uint8_t *bin, size_t size)
 {
-    strsize_t dsize = (strsize_t)((size * 4 / 3) + 1);
-    String out(dsize, String::eos);
+    size_t dsize = (size * 4 / 3) + 1;
+    String out(dsize);
 
-    b64encode(out.c_mem(), bin, size);
+    b64encode(out.data(), bin, size);
     return out;
 }
 
@@ -2243,7 +2130,7 @@ size_t String::b64encode(char *dest, const uint8_t *bin, size_t size, size_t dsi
     size_t count = 0;
 
     if(!dsize)
-        dsize = (size * 4 / 3) + 1;
+        dsize = b64size(size);
 
     if (!dsize || !size)
         goto end;
@@ -2285,7 +2172,12 @@ end:
     return count;
 }
 
-size_t String::b64decode(uint8_t *dest, const char *src, size_t size)
+size_t String::b64size(size_t size)
+{
+    return (size * 4 / 3) + 4;
+}
+
+size_t String::b64count(const char *src, bool ws)
 {
     char decoder[256];
     unsigned long bits;
@@ -2302,37 +2194,94 @@ size_t String::b64decode(uint8_t *dest, const char *src, size_t size)
     bits = 1;
 
     while(*src) {
-        c = (uint8_t)(*(src++));
-        if (c == '=') {
-            if (bits & 0x40000) {
-                if (size < 2)
-                    break;
-                *(dest++) = (uint8_t)((bits >> 10) & 0xff);
-                *(dest++) = (uint8_t)((bits >> 2) & 0xff);
-                count += 2;
-                break;
-            }
-            if ((bits & 0x1000) && size) {
-                *(dest++) = (uint8_t)((bits >> 4) & 0xff);
-                ++count;
+        if(isspace(*src)) {
+            if(ws) {
+                ++src;
+                continue;
             }
             break;
         }
-        // end on invalid chars
-        if (decoder[c] == 64)
+        c = (uint8_t)(*(src++));
+        if (c == '=')
             break;
+        // end on invalid chars
+        if (decoder[c] == 64) {
+            break;
+        }
         bits = (bits << 6) + decoder[c];
         if (bits & 0x1000000) {
-            if (size < 3)
+            bits = 1;
+            count += 3;
+        }
+    }
+
+    if (bits & 0x40000)
+        count += 2;
+    else if (bits & 0x1000)
+        ++count;
+
+    return count;
+}
+
+size_t String::b64decode(uint8_t *dest, const char *src, size_t size, bool ws)
+{
+    char decoder[256];
+    unsigned long bits;
+    uint8_t c;
+    unsigned i;
+    size_t count = 0;
+
+    for (i = 0; i < 256; ++i)
+        decoder[i] = 64;
+
+    for (i = 0; i < 64 ; ++i)
+        decoder[alphabet[i]] = i;
+
+    bits = 1;
+
+    while(*src) {
+        if(isspace(*src)) {
+            if(ws) {
+                ++count;
+                ++src;
+                continue;
+            }
+            break;
+        }
+        c = (uint8_t)(*(src++));
+        if (c == '=') {
+            ++count;
+            if(*src == '=')
+                ++count;
+            break;
+        }
+        // end on invalid chars
+        if (decoder[c] == 64) {
+            break;
+        }
+        ++count;
+        bits = (bits << 6) + decoder[c];
+        if (bits & 0x1000000) {
+            if (size < 3) {
+                bits = 1;
                 break;
+            }
             *(dest++) = (uint8_t)((bits >> 16) & 0xff);
             *(dest++) = (uint8_t)((bits >> 8) & 0xff);
             *(dest++) = (uint8_t)((bits & 0xff));
             bits = 1;
             size -= 3;
-            count += 3;
         }
     }
+    if (bits & 0x40000) {
+        if (size >= 2) {
+            *(dest++) = (uint8_t)((bits >> 10) & 0xff);
+            *(dest++) = (uint8_t)((bits >> 2) & 0xff);
+        }
+    }
+    else if ((bits & 0x1000) && size) {
+        *(dest++) = (uint8_t)((bits >> 4) & 0xff);
+    }
     return count;
 }
 
