@@ -34,8 +34,6 @@ static int realtime_policy = SCHED_FIFO;
 
 #undef  _POSIX_SPIN_LOCKS
 
-static unsigned max_sharing = 0;
-
 namespace ucommon {
 
 #if _POSIX_TIMERS > 0 && defined(POSIX_TIMERS)
@@ -51,7 +49,7 @@ struct mutex_entry
     unsigned count;
 };
 
-class __LOCAL rwlock_entry : public ThreadLock
+class __LOCAL rwlock_entry : public RWLock
 {
 public:
     rwlock_entry();
@@ -82,16 +80,7 @@ static mutex_index single_table;
 static mutex_index *mutex_table = &single_table;
 static unsigned mutex_indexing = 1;
 static unsigned rwlock_indexing = 1;
-
-#ifdef  __PTH__
-static pth_key_t threadmap;
-#else
-#ifdef  _MSTHREADS_
-static DWORD threadmap;
-#else
 static pthread_key_t threadmap;
-#endif
-#endif
 
 mutex_index::mutex_index() : Mutex()
 {
@@ -103,37 +92,11 @@ rwlock_index::rwlock_index() : Mutex()
     list = NULL;
 }
 
-rwlock_entry::rwlock_entry() : ThreadLock()
+rwlock_entry::rwlock_entry() : RWLock()
 {
     count = 0;
 }
 
-#if !defined(_MSTHREADS_) && !defined(__PTH__)
-Conditional::attribute Conditional::attr;
-#endif
-
-#ifdef  __PTH__
-static int pthread_cond_timedwait(pthread_cond_t *cond, pthread_mutex_t *mutex, const struct timespec *abstime)
-{
-    static pth_key_t ev_key = PTH_KEY_INIT;
-    pth_event_t ev = pth_event(PTH_EVENT_TIME|PTH_MODE_STATIC, &ev_key,
-        pth_time(abstime->tv_sec, (abstime->tv_nsec) / 1000));
-
-    if(!pth_cond_await(cond, mutex, ev))
-        return errno;
-    return 0;
-}
-
-static void pthread_shutdown(void)
-{
-    pth_kill();
-}
-
-inline pthread_t pthread_self(void)
-    {return pth_self();};
-
-#endif
-
 static unsigned hash_address(const void *ptr, unsigned indexing)
 {
     assert(ptr != NULL);
@@ -182,127 +145,81 @@ void ReusableAllocator::release(ReusableObject *obj)
     unlock();
 }
 
-void ConditionalAccess::limit_sharing(unsigned max)
+LinkedObject *Thread::Local::list = NULL;
+
+#ifdef  _MSTHREADS_
+
+Thread::Local::Local() : LinkedObject(&list)
 {
-    max_sharing = max;
+    Thread::init();
+    key = TlsAlloc();
 }
 
-void Conditional::set(struct timespec *ts, timeout_t msec)
+Thread::Local::~Local()
 {
-    assert(ts != NULL);
+    TlsFree(key);
+}
+
+void Thread::Local::set(void *ptr)
+{
+    void *current = TlsGetValue(key);
+    if(current)
+        release(current);
+    TlsSetValue(key, ptr);
+}
+
+void *Thread::Local::get(void)
+{
+    return TlsGetValue(key);
+}
 
-#if _POSIX_TIMERS > 0 && defined(POSIX_TIMERS)
-    clock_gettime(_posix_clocking, ts);
 #else
-    timeval tv;
-    gettimeofday(&tv, NULL);
-    ts->tv_sec = tv.tv_sec;
-    ts->tv_nsec = tv.tv_usec * 1000l;
+
+Thread::Local::Local() : LinkedObject(&list)
+{
+    Thread::init();
+    pthread_key_create(&key, NULL);
+}
+
+Thread::Local::~Local()
+{
+    pthread_key_delete(key);
+}
+
+void Thread::Local::set(void *ptr)
+{
+    void *current = pthread_getspecific(key);
+    if(current)
+        release(current);
+    pthread_setspecific(key, ptr);
+}
+
+void *Thread::Local::get(void)
+{
+    return pthread_getspecific(key);
+}
 #endif
-    ts->tv_sec += msec / 1000;
-    ts->tv_nsec += (msec % 1000) * 1000000l;
-    while(ts->tv_nsec >= 1000000000l) {
-        ++ts->tv_sec;
-        ts->tv_nsec -= 1000000000l;
+
+void *Thread::Local::operator*()
+{
+    void *current = get();
+    if(!current) {
+        set(current = allocate());
     }
+    return current;
 }
 
-Semaphore::Semaphore(unsigned limit) :
-Conditional()
+void *Thread::Local::allocate()
 {
-	waits = 0;
-	count = limit;
-	used = 0;
+    return nullptr;
 }
 
-Semaphore::Semaphore(unsigned limit, unsigned avail) :
-Conditional()
+void Thread::release(void)
 {
-	assert(limit > 0);
-	assert(avail <= limit);
-
-	waits = 0;
-	count = limit;
-	used = limit - avail;
-}
-
-void Semaphore::_share(void)
-{
-    wait();
-}
-
-void Semaphore::_unlock(void)
-{
-    release();
-}
-
-bool Semaphore::wait(timeout_t timeout)
-{
-    bool result = true;
-    struct timespec ts;
-    Conditional::set(&ts, timeout);
-
-    lock();
-    while(used >= count && result) {
-        ++waits;
-        result = Conditional::wait(&ts);
-        --waits;
-		if(!count)
-			break;
-    }
-    if(result && count)
-        ++used;
-    unlock();
-    return result;
-}
-
-void Semaphore::wait(void)
-{
-    lock();
-    if(used >= count) {
-        ++waits;
-        Conditional::wait();
-        --waits;
-    }
-	if(count)
-	    ++used;
-    unlock();
-}
-
-void Semaphore::release(void)
-{
-    lock();
-    if(used)
-        --used;
-    if(waits) {
-		if(count)
-	        signal();
-		else
-			broadcast();
-	}
-    unlock();
-}
-
-void Semaphore::set(unsigned value)
-{
-    assert(value > 0);
-
-    unsigned diff;
-
-    lock();
-    count = value;
-    if(used >= count || !waits) {
-        unlock();
-        return;
-    }
-    diff = count - used;
-    if(diff > waits)
-        diff = waits;
-    unlock();
-    while(diff--) {
-        lock();
-        signal();
-        unlock();
+    linked_pointer<Thread::Local> lp = Thread::Local::list;
+    while(is(lp)) {
+        lp->clear();
+        lp.next();
     }
 }
 
@@ -313,218 +230,12 @@ bool Thread::equal(pthread_t t1, pthread_t t2)
     return t1 == t2;
 }
 
-Conditional::Conditional()
-{
-    InitializeCriticalSection(&mutex);
-    InitializeConditionVariable(&cond);
-}
-
-Conditional::~Conditional()
-{
-    DeleteCriticalSection(&mutex);
-}
-
-void Conditional::wait(void)
-{
-    SleepConditionVariableCS(&cond, &mutex, INFINITE);
-}
-
-bool Conditional::wait(timeout_t timeout)
-{
-    if(SleepConditionVariableCS(&cond, &mutex, timeout))
-        return true;
-
-    return false;
-}
-
-void Conditional::signal(void)
-{
-    WakeConditionVariable(&cond);
-}
-
-void Conditional::broadcast(void)
-{
-    WakeAllConditionVariable(&cond);
-}
-
-bool Conditional::wait(struct timespec *ts)
-{
-    assert(ts != NULL);
-
-    return wait((timeout_t)(ts->tv_sec * 1000 + (ts->tv_nsec / 1000000l)));
-}
-
 #else
 
 #include <stdio.h>
 bool Thread::equal(pthread_t t1, pthread_t t2)
 {
-#ifdef  __PTH__
-    return (t1 == t2);
-#else
     return pthread_equal(t1, t2) != 0;
-#endif
-}
-
-#ifndef __PTH__
-Conditional::attribute::attribute()
-{
-    Thread::init();
-    pthread_condattr_init(&attr);
-#if _POSIX_TIMERS > 0 && defined(HAVE_PTHREAD_CONDATTR_SETCLOCK) && defined(POSIX_TIMERS)
-#if defined(_POSIX_MONOTONIC_CLOCK)
-    if(!pthread_condattr_setclock(&attr, CLOCK_MONOTONIC))
-        _posix_clocking = CLOCK_MONOTONIC;
-#else
-    pthread_condattr_setclock(&attr, CLOCK_REALTIME);
-#endif
-#endif
-}
-#endif
-
-Conditional::Conditional()
-{
-#ifdef  __PTH__
-    Thread::init();
-    pth_cond_init(&cond);
-    pth_mutex_init(&mutex);
-#else
-    crit(pthread_cond_init(&cond, &attr.attr) == 0, "conditional init failed");
-    crit(pthread_mutex_init(&mutex, NULL) == 0, "mutex init failed");
-#endif
-}
-
-Conditional::~Conditional()
-{
-#ifndef __PTH__
-    pthread_cond_destroy(&cond);
-    pthread_mutex_destroy(&mutex);
-#endif
-}
-
-bool Conditional::wait(timeout_t timeout)
-{
-    struct timespec ts;
-    set(&ts, timeout);
-    return wait(&ts);
-}
-
-bool Conditional::wait(struct timespec *ts)
-{
-    assert(ts != NULL);
-
-    if(pthread_cond_timedwait(&cond, &mutex, ts) == ETIMEDOUT)
-        return false;
-
-    return true;
-}
-
-#endif
-
-
-#if defined(_MSTHREADS_)
-
-ConditionalAccess::ConditionalAccess()
-{
-    waiting = pending = sharing = 0;
-    InitializeConditionVariable(&bcast);
-}
-
-ConditionalAccess::~ConditionalAccess()
-{
-}
-
-bool ConditionalAccess::waitBroadcast(timeout_t timeout)
-{
-    if(SleepConditionVariableCS(&bcast, &mutex, timeout))
-        return true;
-
-    return false;
-}
-
-void ConditionalAccess::waitBroadcast()
-{
-    SleepConditionVariableCS(&bcast, &mutex, INFINITE);
-}
-
-void ConditionalAccess::waitSignal()
-{
-    SleepConditionVariableCS(&cond, &mutex, INFINITE);
-}
-
-bool ConditionalAccess::waitSignal(timeout_t timeout)
-{
-    if(SleepConditionVariableCS(&cond, &mutex, timeout))
-        return true;
-
-    return false;
-}
-
-bool ConditionalAccess::waitBroadcast(struct timespec *ts)
-{
-    assert(ts != NULL);
-
-    return waitBroadcast((timeout_t)(ts->tv_sec * 1000 + (ts->tv_nsec / 1000000l)));
-}
-
-bool ConditionalAccess::waitSignal(struct timespec *ts)
-{
-    assert(ts != NULL);
-
-    return waitSignal((timeout_t)(ts->tv_sec * 1000 + (ts->tv_nsec / 1000000l)));
-}
-
-#else
-
-ConditionalAccess::ConditionalAccess()
-{
-    waiting = pending = sharing = 0;
-#ifdef  __PTH__
-    pth_cond_init(&bcast);
-#else
-    crit(pthread_cond_init(&bcast, &attr.attr) == 0, "conditional init failed");
-#endif
-}
-
-ConditionalAccess::~ConditionalAccess()
-{
-#ifndef __PTH__
-    pthread_cond_destroy(&bcast);
-#endif
-}
-
-bool ConditionalAccess::waitSignal(timeout_t timeout)
-{
-    struct timespec ts;
-    set(&ts, timeout);
-    return waitSignal(&ts);
-}
-
-bool ConditionalAccess::waitBroadcast(struct timespec *ts)
-{
-    assert(ts != NULL);
-
-    if(pthread_cond_timedwait(&bcast, &mutex, ts) == ETIMEDOUT)
-        return false;
-
-    return true;
-}
-
-bool ConditionalAccess::waitBroadcast(timeout_t timeout)
-{
-    struct timespec ts;
-    set(&ts, timeout);
-    return waitBroadcast(&ts);
-}
-
-bool ConditionalAccess::waitSignal(struct timespec *ts)
-{
-    assert(ts != NULL);
-
-    if(pthread_cond_timedwait(&cond, &mutex, ts) == ETIMEDOUT)
-        return false;
-
-    return true;
 }
 
 #endif
@@ -549,51 +260,6 @@ bool DetachedThread::is_active(void) const
     return active;
 }
 
-void ConditionalAccess::modify(void)
-{
-    lock();
-    while(sharing) {
-        ++pending;
-        waitSignal();
-        --pending;
-    }
-}
-
-void ConditionalAccess::commit(void)
-{
-    if(pending)
-        signal();
-    else if(waiting)
-        broadcast();
-    unlock();
-}
-
-void ConditionalAccess::access(void)
-{
-    lock();
-    assert(!max_sharing || sharing < max_sharing);
-    while(pending) {
-        ++waiting;
-        waitBroadcast();
-        --waiting;
-    }
-    ++sharing;
-    unlock();
-}
-
-void ConditionalAccess::release(void)
-{
-   lock();
-    assert(sharing);
-
-    --sharing;
-    if(pending && !sharing)
-        signal();
-    else if(waiting && !pending)
-        broadcast();
-    unlock();
-}
-
 RecursiveMutex::RecursiveMutex() :
 Conditional()
 {
@@ -662,28 +328,13 @@ void RecursiveMutex::release(void)
     Conditional::unlock();
 }
 
-ThreadLock::ThreadLock() :
+RWLock::RWLock() :
 ConditionalAccess()
 {
     writers = 0;
 }
 
-void ThreadLock::_lock(void)
-{
-    modify();
-}
-
-void ThreadLock::_share(void)
-{
-    access();
-}
-
-void ThreadLock::_unlock(void)
-{
-    release();
-}
-
-bool ThreadLock::modify(timeout_t timeout)
+bool RWLock::modify(timeout_t timeout)
 {
     bool rtn = true;
     struct timespec ts;
@@ -714,7 +365,7 @@ bool ThreadLock::modify(timeout_t timeout)
     return rtn;
 }
 
-bool ThreadLock::access(timeout_t timeout)
+bool RWLock::access(timeout_t timeout)
 {
     struct timespec ts;
     bool rtn = true;
@@ -740,7 +391,7 @@ bool ThreadLock::access(timeout_t timeout)
     return rtn;
 }
 
-void ThreadLock::release(void)
+void RWLock::release(void)
 {
     lock();
     assert(sharing || writers);
@@ -766,60 +417,24 @@ void ThreadLock::release(void)
     unlock();
 }
 
-auto_protect::auto_protect()
+AutoProtect::AutoProtect()
 {
     object = NULL;
 }
 
-auto_protect::auto_protect(const void *obj)
-{
-    object = obj;
-    if(object)
-        Mutex::protect(obj);
-}
-
-auto_protect::~auto_protect()
-{
-    release();
-}
-
-void auto_protect::release()
-{
-    if(object) {
-        Mutex::release(object);
-        object = NULL;
-    }
-}
-
-void auto_protect::operator=(const void *obj)
-{
-    if(obj == object)
-        return;
-
-    release();
-    object = obj;
-    if(object)
-        Mutex::protect(object);
-}
-
-Mutex::guard::guard()
-{
-    object = NULL;
-}
-
-Mutex::guard::guard(const void *obj)
+AutoProtect::AutoProtect(const void *obj)
 {
     object = obj;
     if(obj)
         Mutex::protect(object);
 }
 
-Mutex::guard::~guard()
+AutoProtect::~AutoProtect()
 {
     release();
 }
 
-void Mutex::guard::set(const void *obj)
+void AutoProtect::set(const void *obj)
 {
     release();
     object = obj;
@@ -827,7 +442,7 @@ void Mutex::guard::set(const void *obj)
         Mutex::protect(object);
 }
 
-void Mutex::guard::release(void)
+void AutoProtect::release(void)
 {
     if(object) {
         Mutex::release(object);
@@ -837,11 +452,8 @@ void Mutex::guard::release(void)
 
 Mutex::Mutex()
 {
-#ifdef  __PTH__
-    pth_mutex_init(&mlock);
-#else
-    crit(pthread_mutex_init(&mlock, NULL) == 0, "mutex init failed");
-#endif
+    if(pthread_mutex_init(&mlock, NULL))
+         __THROW_RUNTIME("mutex init failed");
 }
 
 Mutex::~Mutex()
@@ -857,7 +469,7 @@ void Mutex::indexing(unsigned index)
     }
 }
 
-void ThreadLock::indexing(unsigned index)
+void RWLock::indexing(unsigned index)
 {
     if(index > 1) {
         rwlock_table = new rwlock_index[index];
@@ -865,77 +477,77 @@ void ThreadLock::indexing(unsigned index)
     }
 }
 
-ThreadLock::guard_reader::guard_reader()
+RWLock::reader::reader()
 {
     object = NULL;
 }
 
-ThreadLock::guard_reader::guard_reader(const void *obj)
+RWLock::reader::reader(const void *obj)
 {
     object = obj;
     if(obj)
-        if(!ThreadLock::reader(object))
+        if(!lock(object))
             object = NULL;
 }
 
-ThreadLock::guard_reader::~guard_reader()
+RWLock::reader::~reader()
 {
     release();
 }
 
-void ThreadLock::guard_reader::set(const void *obj)
+void RWLock::reader::set(const void *obj)
 {
     release();
     object = obj;
     if(obj)
-        if(!ThreadLock::reader(object))
+        if(!lock(object))
             object = NULL;
 }
 
-void ThreadLock::guard_reader::release(void)
+void RWLock::reader::release(void)
 {
     if(object) {
-        ThreadLock::release(object);
+        RWLock::release(object);
         object = NULL;
     }
 }
 
-ThreadLock::guard_writer::guard_writer()
+RWLock::writer::writer()
 {
     object = NULL;
 }
 
-ThreadLock::guard_writer::guard_writer(const void *obj)
+RWLock::writer::writer(const void *obj)
 {
     object = obj;
     if(obj)
-        if(!ThreadLock::writer(object))
+        if(!lock(object))
             object = NULL;
 }
 
-ThreadLock::guard_writer::~guard_writer()
+RWLock::writer::~writer()
 {
     release();
 }
 
-void ThreadLock::guard_writer::set(const void *obj)
+void RWLock::writer::set(const void *obj)
 {
     release();
     object = obj;
     if(obj)
-        if(!ThreadLock::writer(object))
+        if(!lock(object))
             object = NULL;
 }
 
-void ThreadLock::guard_writer::release(void)
+void RWLock::writer::release(void)
 {
     if(object) {
-        ThreadLock::release(object);
+        RWLock::release(object);
         object = NULL;
     }
 }
 
-bool ThreadLock::reader(const void *ptr, timeout_t timeout)
+bool RWLock::reader::lock(const void *ptr, timeout_t timeout)
 {
     rwlock_index *index = &rwlock_table[hash_address(ptr, rwlock_indexing)];
     rwlock_entry *entry, *empty = NULL;
@@ -972,7 +584,7 @@ bool ThreadLock::reader(const void *ptr, timeout_t timeout)
     return false;
 }
 
-bool ThreadLock::writer(const void *ptr, timeout_t timeout)
+bool RWLock::writer::lock(const void *ptr, timeout_t timeout)
 {
     rwlock_index *index = &rwlock_table[hash_address(ptr, rwlock_indexing)];
     rwlock_entry *entry, *empty = NULL;
@@ -1009,6 +621,26 @@ bool ThreadLock::writer(const void *ptr, timeout_t timeout)
     return false;
 }
 
+void RWLock::_lock(void)
+{
+    modify();
+}
+
+void RWLock::_share(void)
+{
+    access();
+}
+
+void RWLock::_unlock(void)
+{
+    release();
+}
+
+void RWLock::_unshare(void)
+{
+    release();
+}
+
 bool Mutex::protect(const void *ptr)
 {
     mutex_index *index = &mutex_table[hash_address(ptr, mutex_indexing)];
@@ -1045,7 +677,7 @@ bool Mutex::protect(const void *ptr)
 	return true;
 }
 
-bool ThreadLock::release(const void *ptr)
+bool RWLock::release(const void *ptr)
 {
     rwlock_index *index = &rwlock_table[hash_address(ptr, rwlock_indexing)];
     rwlock_entry *entry;
@@ -1198,14 +830,10 @@ TimedEvent::TimedEvent() :
 Timer()
 {
     signalled = false;
-#ifdef  __PTH__
-    Thread::init();
-    pth_cond_init(&cond);
-    pth_mutex_init(&mutex);
-#else
-    crit(pthread_cond_init(&cond, Conditional::initializer()) == 0, "conditional init failed");
-    crit(pthread_mutex_init(&mutex, NULL) == 0, "mutex init failed");
-#endif
+    if(pthread_cond_init(&cond, Conditional::initializer()))
+        __THROW_RUNTIME("conditional init failed");
+    if(pthread_mutex_init(&mutex, NULL))
+        __THROW_RUNTIME("mutex init failed");
     set();
 }
 
@@ -1213,36 +841,26 @@ TimedEvent::TimedEvent(timeout_t timeout) :
 Timer(timeout)
 {
     signalled = false;
-#ifdef  __PTH__
-    Thread::init();
-    pth_cond_init(&cond);
-    pth_mutex_init(&mutex);
-#else
-    crit(pthread_cond_init(&cond, Conditional::initializer()) == 0, "conditional init failed");
-    crit(pthread_mutex_init(&mutex, NULL) == 0, "mutex init failed");
-#endif
+    if(pthread_cond_init(&cond, Conditional::initializer()))
+        __THROW_RUNTIME("conditional init failed");
+    if(pthread_mutex_init(&mutex, NULL))
+        __THROW_RUNTIME("mutex init failed");
 }
 
 TimedEvent::TimedEvent(time_t timer) :
 Timer(timer)
 {
     signalled = false;
-#ifdef  __PTH__
-    Thread::init();
-    pth_cond_init(&cond);
-    pth_mutex_init(&mutex);
-#else
-    crit(pthread_cond_init(&cond, Conditional::initializer()) == 0, "conditional init failed");
-    crit(pthread_mutex_init(&mutex, NULL) == 0, "mutex init failed");
-#endif
+    if(pthread_cond_init(&cond, Conditional::initializer()))
+        __THROW_RUNTIME("conditional init failed");
+    if(pthread_mutex_init(&mutex, NULL))
+        __THROW_RUNTIME("mutex init failed");
 }
 
 TimedEvent::~TimedEvent()
 {
-#ifndef __PTH__
     pthread_cond_destroy(&cond);
     pthread_mutex_destroy(&mutex);
-#endif
 }
 
 void TimedEvent::reset(void)
@@ -1315,324 +933,6 @@ void TimedEvent::release(void)
     pthread_mutex_unlock(&mutex);
 }
 
-ConditionalLock::ConditionalLock() :
-ConditionalAccess()
-{
-    contexts = NULL;
-}
-
-ConditionalLock::~ConditionalLock()
-{
-    linked_pointer<Context> cp = contexts, next;
-    while(cp) {
-        next = cp->getNext();
-        delete *cp;
-        cp = next;
-    }
-}
-
-ConditionalLock::Context *ConditionalLock::getContext(void)
-{
-    Context *slot = NULL;
-    pthread_t tid = Thread::self();
-    linked_pointer<Context> cp = contexts;
-
-    while(cp) {
-        if(cp->count && Thread::equal(cp->thread, tid))
-            return *cp;
-        if(!cp->count)
-            slot = *cp;
-        cp.next();
-    }
-    if(!slot) {
-        slot = new Context(&this->contexts);
-        slot->count = 0;
-    }
-    slot->thread = tid;
-    return slot;
-}
-
-void ConditionalLock::_share(void)
-{
-    access();
-}
-
-void ConditionalLock::_unlock(void)
-{
-    release();
-}
-
-void ConditionalLock::modify(void)
-{
-    Context *context;
-
-    lock();
-    context = getContext();
-
-    assert(context && sharing >= context->count);
-
-    sharing -= context->count;
-    while(sharing) {
-        ++pending;
-        waitSignal();
-        --pending;
-    }
-    ++context->count;
-}
-
-void ConditionalLock::commit(void)
-{
-    Context *context = getContext();
-    --context->count;
-
-    if(context->count) {
-        sharing += context->count;
-        unlock();
-    }
-    else
-        ConditionalAccess::commit();
-}
-
-void ConditionalLock::release(void)
-{
-    Context *context;
-
-    lock();
-    context = getContext();
-    assert(sharing && context && context->count > 0);
-    --sharing;
-    --context->count;
-    if(pending && !sharing)
-        signal();
-    else if(waiting && !pending)
-        broadcast();
-    unlock();
-}
-
-void ConditionalLock::access(void)
-{
-    Context *context;
-    lock();
-    context = getContext();
-    assert(context && (!max_sharing || sharing < max_sharing));
-
-    // reschedule if pending exclusives to make sure modify threads are not
-    // starved.
-
-    ++context->count;
-
-    while(context->count < 2 && pending) {
-        ++waiting;
-        waitBroadcast();
-        --waiting;
-    }
-    ++sharing;
-    unlock();
-}
-
-void ConditionalLock::exclusive(void)
-{
-    Context *context;
-
-    lock();
-    context = getContext();
-    assert(sharing && context && context->count > 0);
-    sharing -= context->count;
-    while(sharing) {
-        ++pending;
-        waitSignal();
-        --pending;
-    }
-}
-
-void ConditionalLock::share(void)
-{
-    Context *context = getContext();
-    assert(!sharing && context && context->count);
-    sharing += context->count;
-    unlock();
-}
-
-barrier::barrier(unsigned limit) :
-Conditional()
-{
-    count = limit;
-    waits = 0;
-}
-
-barrier::~barrier()
-{
-    lock();
-    if(waits)
-        broadcast();
-    unlock();
-}
-
-void barrier::set(unsigned limit)
-{
-    assert(limit > 0);
-
-    lock();
-    count = limit;
-    if(count <= waits) {
-        waits = 0;
-        broadcast();
-    }
-    unlock();
-}
-
-void barrier::dec(void)
-{
-    lock();
-    if(count)
-        --count;
-    unlock();
-}
-
-unsigned barrier::operator--(void)
-{
-    unsigned result;
-    lock();
-    if(count)
-        --count;
-    result = count;
-    unlock();
-    return result;
-}
-
-void barrier::inc(void)
-{
-    lock();
-    count++;
-    if(count <= waits) {
-        waits = 0;
-        broadcast();
-    }
-    unlock();
-}
-
-unsigned barrier::operator++(void)
-{
-    unsigned result;
-    lock();
-    count++;
-    if(count <= waits) {
-        waits = 0;
-        broadcast();
-    }
-    result = count;
-    unlock();
-    return result;
-}
-
-bool barrier::wait(timeout_t timeout)
-{
-    bool result;
-
-    Conditional::lock();
-    if(!count) {
-        Conditional::unlock();
-        return true;
-    }
-    if(++waits >= count) {
-        waits = 0;
-        Conditional::broadcast();
-        Conditional::unlock();
-        return true;
-    }
-    result = Conditional::wait(timeout);
-    Conditional::unlock();
-    return result;
-}
-
-void barrier::wait(void)
-{
-    Conditional::lock();
-    if(!count) {
-        Conditional::unlock();
-        return;
-    }
-    if(++waits >= count) {
-        waits = 0;
-        Conditional::broadcast();
-        Conditional::unlock();
-        return;
-    }
-    Conditional::wait();
-    Conditional::unlock();
-}
-
-LockedPointer::LockedPointer()
-{
-#if defined(_MSTHREADS_)
-    InitializeCriticalSection((LPCRITICAL_SECTION)&mutex);
-#else
-#ifdef  __PTH__
-    static pthread_mutex_t lock = PTH_MUTEX_INIT;
-#else
-    static pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;
-#endif
-    memcpy(&mutex, &lock, sizeof(mutex));
-#endif
-    pointer = NULL;
-}
-
-void LockedPointer::replace(ObjectProtocol *obj)
-{
-    assert(obj != NULL);
-
-    pthread_mutex_lock(&mutex);
-    obj->retain();
-    if(pointer)
-        pointer->release();
-    pointer = obj;
-    pthread_mutex_unlock(&mutex);
-}
-
-ObjectProtocol *LockedPointer::dup(void)
-{
-    ObjectProtocol *temp;
-    pthread_mutex_lock(&mutex);
-    temp = pointer;
-    if(temp)
-        temp->retain();
-    pthread_mutex_unlock(&mutex);
-    return temp;
-}
-
-SharedObject::~SharedObject()
-{
-}
-
-SharedPointer::SharedPointer() :
-ConditionalAccess()
-{
-    pointer = NULL;
-}
-
-SharedPointer::~SharedPointer()
-{
-}
-
-void SharedPointer::replace(SharedObject *ptr)
-{
-    modify();
-
-    if(pointer)
-        delete pointer;
-    pointer = ptr;
-    if(ptr)
-        ptr->commit(this);
-
-    commit();
-}
-
-SharedObject *SharedPointer::share(void)
-{
-    access();
-    return pointer;
-}
-
 Thread::Thread(size_t size)
 {
     stack = (stacksize_t)size;
@@ -1662,7 +962,6 @@ void Thread::setPriority(void)
 
 void Thread::setPriority(void)
 {
-#ifndef __PTH__
     int policy;
     struct sched_param sp;
     pthread_t ptid = pthread_self();
@@ -1691,7 +990,6 @@ void Thread::setPriority(void)
 
     sp.sched_priority = pri;
     pthread_setschedparam(ptid, policy, &sp);
-#endif
 }
 
 #else
@@ -1739,9 +1037,7 @@ DetachedThread::DetachedThread(size_t size)
 
 void Thread::sleep(timeout_t timeout)
 {
-#if defined(__PTH__)
-    pth_usleep(timeout * 1000);
-#elif defined(_MSTHREADS_)
+#if defined(_MSTHREADS_)
 	::Sleep(timeout);
 #elif defined(HAVE_PTHREAD_DELAY)
     timespec ts;
@@ -1762,8 +1058,6 @@ void Thread::yield(void)
 {
 #if defined(_MSTHREADS_)
     SwitchToThread();
-#elif defined(__PTH__)
-    pth_yield(NULL);
 #elif defined(HAVE_PTHREAD_YIELD_NP)
     pthread_yield_np();
 #elif defined(HAVE_PTHREAD_YIELD)
@@ -1885,14 +1179,12 @@ void JoinableThread::start(int adj)
     joining = false;
     priority = adj;
 
-#ifndef __PTH__
     pthread_attr_t attr;
     pthread_attr_init(&attr);
     pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE);
 #if HAVE_PTHREAD_ATTR_SETINHRITSCHED
     pthread_attr_setinheritsched(&attr, PTHREAD_INHERIT_SCHED);
 #endif
-#endif
 // we typically use "stack 1" for min stack...
 #ifdef  PTHREAD_STACK_MIN
     if(stack && stack < PTHREAD_STACK_MIN)
@@ -1901,31 +1193,23 @@ void JoinableThread::start(int adj)
     if(stack && stack < 2)
         stack = 0;
 #endif
-#ifdef  __PTH__
-    pth_attr_t attr = PTH_ATTR_DEFAULT;
-    pth_attr_set(attr, PTH_ATTR_JOINABLE);
-    tid = pth_spawn(attr, &exec_thread, this);
-#else
     if(stack)
         pthread_attr_setstacksize(&attr, stack);
     result = pthread_create(&tid, &attr, &exec_thread, this);
     pthread_attr_destroy(&attr);
     if(!result)
         running = true;
-#endif
 }
 
 void DetachedThread::start(int adj)
 {
     priority = adj;
-#ifndef __PTH__
     pthread_attr_t attr;
     pthread_attr_init(&attr);
     pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
 #if HAVE_PTHREAD_ATTR_SETINHRITSCHED
     pthread_attr_setinheritsched(&attr, PTHREAD_INHERIT_SCHED);
 #endif
-#endif
 // we typically use "stack 1" for min stack...
 #ifdef  PTHREAD_STACK_MIN
     if(stack && stack < PTHREAD_STACK_MIN)
@@ -1934,14 +1218,10 @@ void DetachedThread::start(int adj)
     if(stack && stack < 2)
         stack = 0;
 #endif
-#ifdef  __PTH__
-    tid = pth_spawn(PTH_ATTR_DEFAULT, &exec_thread, this);
-#else
     if(stack)
         pthread_attr_setstacksize(&attr, stack);
     pthread_create(&tid, &attr, &exec_thread, this);
     pthread_attr_destroy(&attr);
-#endif
     active = true;
 }
 
@@ -1960,129 +1240,35 @@ void JoinableThread::join(void)
 
     joining = true;
 
-#ifdef  __PTH__
-    if(pth_join(tid, NULL))
-        running = false;
-#else
     if(!pthread_join(tid, NULL))
         running = false;
-#endif
 }
 
 #endif
 
 void Thread::exit(void)
 {
+    release();
     pthread_exit(NULL);
 }
 
-locked_release::locked_release(const locked_release &copy)
-{
-    object = copy.object;
-    object->retain();
-}
-
-locked_release::locked_release()
-{
-    object = NULL;
-}
-
-locked_release::locked_release(LockedPointer &p)
-{
-    object = p.dup();
-}
-
-locked_release::~locked_release()
-{
-    release();
-}
-
-void locked_release::release(void)
-{
-    if(object)
-        object->release();
-    object = NULL;
-}
-
-locked_release &locked_release::operator=(LockedPointer &p)
-{
-    release();
-    object = p.dup();
-    return *this;
-}
-
-shared_release::shared_release(const shared_release &copy)
-{
-    ptr = copy.ptr;
-}
-
-shared_release::shared_release()
-{
-    ptr = NULL;
-}
-
-SharedObject *shared_release::get(void)
-{
-    if(ptr)
-        return ptr->pointer;
-    return NULL;
-}
-
-void SharedObject::commit(SharedPointer *spointer)
-{
-}
-
-shared_release::shared_release(SharedPointer &p)
-{
-    ptr = &p;
-    p.share(); // create rdlock
-}
-
-shared_release::~shared_release()
-{
-    release();
-}
-
-void shared_release::release(void)
-{
-    if(ptr)
-        ptr->release();
-    ptr = NULL;
-}
-
-shared_release &shared_release::operator=(SharedPointer &p)
-{
-    release();
-    ptr = &p;
-    p.share();
-    return *this;
-}
-
 void Thread::map(void)
 {
     Thread::init();
-#ifdef  __PTH__
-    pth_key_setdata(threadmap, this);
-#else
 #ifdef  _MSTHREADS_
     TlsSetValue(threadmap, this);
 #else
     pthread_setspecific(threadmap, this);
 #endif
-#endif
 }
 
 Thread *Thread::get(void)
 {
-#ifdef  __PTH__
-    return (Thread *)pth_key_setdata(threadmap);
-#else
 #ifdef  _MSTHREADS_
     return (Thread *)TlsGetValue(threadmap);
 #else
     return (Thread *)pthread_getspecific(threadmap);
 #endif
-#endif
 }
 
 void Thread::init(void)
@@ -2090,31 +1276,69 @@ void Thread::init(void)
     static volatile bool initialized = false;
 
     if(!initialized) {
-#ifdef  __PTH__
-        pth_init();
-        pth_key_create(&threadmap, NULL);
-        atexit(pthread_shutdown);
-#else
 #ifdef  _MSTHREADS_
         threadmap = TlsAlloc();
 #else
         pthread_key_create(&threadmap, NULL);
-#endif
 #endif
         initialized = true;
     }
 }
 
-#ifdef  __PTH__
-pthread_t Thread::self(void)
+size_t Thread::cache(void)
 {
-    return pth_self();
-}
+    static volatile size_t line_size = 0;
+
+    if(line_size)
+        return line_size;
+
+#ifdef  _MSWINDOWS_
+    DWORD buffer_size = 0;
+    DWORD i = 0;
+    SYSTEM_LOGICAL_PROCESSOR_INFORMATION * buffer = 0;
+
+    GetLogicalProcessorInformation(0, &buffer_size);
+    buffer = (SYSTEM_LOGICAL_PROCESSOR_INFORMATION *)malloc(buffer_size);
+    GetLogicalProcessorInformation(&buffer[0], &buffer_size);
+
+    for (i = 0; i != buffer_size / sizeof(SYSTEM_LOGICAL_PROCESSOR_INFORMATION); ++i) {
+        if (buffer[i].Relationship == RelationCache && buffer[i].Cache.Level == 1) {
+            line_size = buffer[i].Cache.LineSize;
+            break;
+        }
+    }
+
+    free(buffer);
+    return line_size;
+#elif defined(__APPLE__)
+    size_t line_size = 0;
+    size_t sizeof_line_size = sizeof(line_size);
+    sysctlbyname("hw.cachelinesize", &line_size, &sizeof_line_size, 0, 0);
+    return line_size;
+#elif defined(_SC_LEVEL1_DCACHE_LINESIZE)
+    line_size = sysconf(_SC_LEVEL1_DCACHE_LINESIZE);
+    if(!line_size)
+        line_size = 64;
+    return line_size;
+#elif defined(linux)
+    FILE * fp = 0;
+    fp = fopen("/sys/devices/system/cpu/cpu0/cache/index0/coherency_line_size", "r");
+    if (fp) {
+        if(fscanf(fp, "%lu", &line_size) < 1)
+            line_size = 64;
+        fclose(fp);
+    }
+    else
+        line_size = 64;
+    return line_size;
 #else
+    return line_size = 64;
+#endif
+}
+
 pthread_t Thread::self(void)
 {
     return pthread_self();
 }
-#endif
 
 } // namespace ucommon
