@@ -20,6 +20,7 @@
 #include <ucommon/export.h>
 #include <ucommon/timers.h>
 #include <ucommon/thread.h>
+#include <ucommon/cpr.h>
 
 namespace ucommon {
 
@@ -59,65 +60,14 @@ static void adj(struct timeval *ts)
 }
 #endif
 
-#ifdef  WIN32
-#ifdef  _WIN32_WCE
-} // namespace ucommon
-extern "C" int gettimeofday(struct timeval *tv_,  void *tz_)
-{
-    assert(tv_ != NULL);
-
-    // We could use _ftime(), but it is not available on WinCE.
-    // (WinCE also lacks time.h)
-    // Note also that the average error of _ftime is around 20 ms :)
-    DWORD ms = GetTickCount();
-    tv_->tv_sec = ms / 1000;
-    tv_->tv_usec = ms * 1000;
-    return 0;
-}
-namespace ucommon {
-#else
-} // namespace ucommon
-#ifdef  HAVE_SYS_TIMEB_H
-#include <sys/timeb.h>
-#endif
-extern "C" int gettimeofday(struct timeval *tv_, void *tz_)
-{
-    assert(tv_ != NULL);
-
-#if defined(_MSC_VER) && _MSC_VER >= 1300
-    struct __timeb64 tb;
-    _ftime64(&tb);
-#else
-# ifndef __BORLANDC__
-    struct _timeb tb;
-    _ftime(&tb);
-# else
-    struct timeb tb;
-    ftime(&tb);
-# endif
-#endif
-    tv_->tv_sec = (long)tb.time;
-    tv_->tv_usec = tb.millitm * 1000;
-    return 0;
-}
-namespace ucommon {
-#endif
-#endif
-
-#if _MSC_VER > 1400        // windows broken dll linkage issue...
-#else
-const timeout_t Timer::inf = ((timeout_t)(-1));
-const time_t Timer::reset = ((time_t)0);
-#endif
-
 TimerQueue::event::event(timeout_t timeout) :
-Timer(), LinkedList()
+Timer(), DLinkedObject()
 {
     set(timeout);
 }
 
 TimerQueue::event::event(TimerQueue *tq, timeout_t timeout) :
-Timer(), LinkedList()
+Timer(), DLinkedObject()
 {
     set(timeout);
     Timer::update();
