@@ -19,19 +19,29 @@
 #include <ucommon/export.h>
 #include <ucommon/typeref.h>
 #include <ucommon/string.h>
+#include <ucommon/thread.h>
 #include <cstdlib>
 
 namespace ucommon {
 
-TypeRef::Counted::Counted(void *addr, size_t objsize) : 
+TypeRef::Counted::Counted(void *addr, size_t objsize, TypeRelease *ar) : 
 ObjectProtocol()
 {
-    this->memory = addr;
+    this->offset = (unsigned)((char *)this - (char *)addr);
     this->size = objsize;
+    this->autorelease = ar;
 }
 
 void TypeRef::Counted::dealloc()
 {
+    TypeRelease *rel = autorelease;
+    if(rel) {
+        autorelease = nullptr;
+        rel->dealloc(this);
+        return;
+    }
+        
+    void *memory = (void *)((char *)this - offset);
     delete this;
     ::free(memory);
 }
@@ -42,12 +52,12 @@ void TypeRef::Counted::operator delete(void *addr)
 
 void TypeRef::Counted::retain(void)
 {
-    count.fetch_add();
+    count.fetch_retain();
 }
 
 void TypeRef::Counted::release(void)
 {
-    if(count.fetch_sub() < 2) {
+    if(count.fetch_release() < 2) {
 	    dealloc();
     }
 }
@@ -83,6 +93,17 @@ void TypeRef::clear(void)
     ref = NULL;
 }
 
+bool TypeRef::is_released()
+{
+    if(!ref)
+        return true;
+
+    if(ref->autorelease)
+        return false;
+
+    return true;
+}
+
 void TypeRef::set(const TypeRef& ptr)
 {
     if(ptr.ref)
@@ -91,6 +112,16 @@ void TypeRef::set(const TypeRef& ptr)
     ref = ptr.ref;
 }
 
+void TypeRef::assign(const typeref_guard& global)
+{
+    global.sync.acquire();
+    if(global.ref)
+        global.ref->retain();
+    clear();
+    ref = global.ref;
+    global.sync.release();
+}
+
 void TypeRef::set(TypeRef::Counted *object)
 {
     if(object)
@@ -99,20 +130,23 @@ void TypeRef::set(TypeRef::Counted *object)
     ref = object;
 }
 
-caddr_t TypeRef::alloc(size_t size)
-{
-    return (caddr_t)::malloc(size + 16);
-}
-
 caddr_t TypeRef::mem(caddr_t addr)
 {
-    while(((uintptr_t)addr) & 0xf)
+    size_t mask = (Thread::cache() - 1);
+    while(((uintptr_t)addr) & mask)
         ++addr;
     return addr;
 }
 
-typeref<const char *>::value::value(caddr_t addr, size_t objsize, const char *str) : 
-TypeRef::Counted(addr, objsize)
+void typeref_guard::set(const TypeRef& pointer)
+{
+    sync.lock();
+    TypeRef::set(pointer);
+    sync.unlock();
+}
+
+typeref<const char *>::value::value(caddr_t addr, size_t objsize, const char *str, TypeRelease *ar) : 
+TypeRef::Counted(addr, objsize, ar)
 {
     if(str)
     	String::set(mem, objsize + 1, str);
@@ -132,7 +166,7 @@ TypeRef() {}
 typeref<const char *>::typeref(const typeref<const char *>& copy) :
 TypeRef(copy) {}
 
-typeref<const char *>::typeref(const char *str) :
+typeref<const char *>::typeref(const char *str, TypeRelease *ar) :
 TypeRef()
 {
     size_t size = 0;
@@ -140,8 +174,15 @@ TypeRef()
     if(str)
         size = strlen(str);
 
-    caddr_t p = TypeRef::alloc(sizeof(value) + size);
-    TypeRef::set(new(mem(p)) value(p, size, str));
+    caddr_t p = ar->allocate(sizeof(value) + size);
+    TypeRef::set(new(mem(p)) value(p, size, str, ar));
+}
+
+typeref<const char *>::typeref(size_t size, TypeRelease *ar) :
+TypeRef()
+{
+    caddr_t p = ar->allocate(sizeof(value) + size);
+    TypeRef::set(new(mem(p)) value(p, size, "", ar));
 }
 
 const char *typeref<const char *>::operator()(ssize_t offset) const
@@ -180,20 +221,23 @@ size_t typeref<const char *>::len() const
     return v->len();
 }
 
-typeref<const char *> typeref<const char *>::operator+(const char *str2) const 
+const typeref<const char *> typeref<const char *>::operator+(const char *str2) const 
 {
     value *v1 = polystatic_cast<value *>(ref);
     const char *str1 = "";
+    TypeRelease *ar = nullptr;
 
-    if(v1)
+    if(v1) {
         str1 = &v1->mem[0];
+        ar = v1->autorelease;
+    }
 
     if(!str2)
         str2 = "";
 
     size_t ss = strlen(str1);
     ss += strlen(str2);
-    charvalues_t results = stringref::create(ss);
+    charvalues_t results = create(ss, ar);
     snprintf(results->get(), results->max() + 1, "%s%s", str1, str2);
     stringref_t result;
     result.assign(results);
@@ -212,7 +256,7 @@ typeref<const char *>& typeref<const char *>::operator=(const char *str)
     return *this;
 }
 
-void typeref<const char *>::set(const char *str)
+void typeref<const char *>::set(const char *str, TypeRelease *ar)
 {
     clear();
     size_t size = 0;
@@ -220,8 +264,34 @@ void typeref<const char *>::set(const char *str)
     if(str)
         size = strlen(str);
 
-    caddr_t p = TypeRef::alloc(sizeof(value) + size);
-    TypeRef::set(new(mem(p)) value(p, size, str));
+    caddr_t p = ar->allocate(sizeof(value) + size);
+    TypeRef::set(new(mem(p)) value(p, size, str, ar));
+}
+
+void typeref<const char *>::b64(const uint8_t *bytes, size_t bsize, TypeRelease *ar)
+{
+    clear();
+    size_t len = String::b64size(bsize);
+
+    caddr_t p = ar->allocate(sizeof(value) + len);
+    value *s = new(mem(p)) value(p, len, "", ar);
+    String::b64encode(&s->mem[0], bytes, bsize);
+    TypeRef::set(s);
+}
+
+void typeref<const char *>::hex(const uint8_t *bytes, size_t bsize, TypeRelease *ar)
+{
+    clear();
+    size_t len = bsize * 2;
+
+    caddr_t p = ar->allocate(sizeof(value) + len);
+    value *s = new(mem(p)) value(p, len, "", ar);
+
+    for(size_t index = 0; index < bsize; ++index) {
+        snprintf(&s->mem[index * 2], 3, "%2.2x", bytes[index]);
+    }
+
+    TypeRef::set(s);
 }
 
 void typeref<const char *>::assign(value *chars)
@@ -237,10 +307,10 @@ typeref<const char *>& typeref<const char *>::operator=(value *chars)
     return *this;
 }
 
-typeref<const char *>::value *typeref<const char *>::create(size_t size)
+typeref<const char *>::value *typeref<const char *>::create(size_t size, TypeRelease *ar)
 {
-    caddr_t p = TypeRef::alloc(sizeof(value) + size);
-    return new(mem(p)) value(p, size, NULL);
+    caddr_t p = ar->allocate(sizeof(value) + size);
+    return new(mem(p)) value(p, size, NULL, ar);
 }
 
 void typeref<const char *>::destroy(typeref<const char *>::value *chars)
@@ -307,40 +377,42 @@ bool typeref<const char *>::operator<(const typeref<const char *>& ptr) const
 #endif
 }
     
-typeref<const uint8_t *>::value::value(caddr_t addr, size_t objsize, const uint8_t *str) : 
-TypeRef::Counted(addr, objsize)
+typeref<const uint8_t *>::value::value(caddr_t addr, size_t objsize, const uint8_t *str, TypeRelease *ar) : 
+TypeRef::Counted(addr, objsize, ar)
 {
-    if(objsize)
+    if(objsize && str)
         memcpy(mem, str, objsize);
 }
 
-typeref<const uint8_t *>::value::value(caddr_t addr, size_t size) : 
-TypeRef::Counted(addr, size)
-{
-}
-
 typeref<const uint8_t *>::typeref() :
 TypeRef() {}
 
 typeref<const uint8_t *>::typeref(const typeref<const uint8_t *>& copy) :
 TypeRef(copy) {}
 
-typeref<const uint8_t *>::typeref(uint8_t *str, size_t size) :
+typeref<const uint8_t *>::typeref(uint8_t *str, size_t size, TypeRelease *ar) :
 TypeRef()
 {
-    caddr_t p = TypeRef::alloc(sizeof(value) + size);
-    TypeRef::set(new(mem(p)) value(p, size, str));
+    caddr_t p = ar->allocate(sizeof(value) + size);
+    TypeRef::set(new(mem(p)) value(p, size, str, ar));
 }
 
-typeref<const uint8_t *>::typeref(bool mode, size_t bits) :
+typeref<const uint8_t *>::typeref(size_t size, TypeRelease *ar) :
+TypeRef()
+{
+    caddr_t p = ar->allocate(sizeof(value) + size);
+    TypeRef::set(new(mem(p)) value(p, size, nullptr, ar));
+}
+
+typeref<const uint8_t *>::typeref(bool mode, size_t bits, TypeRelease *ar) :
 TypeRef()
 {
     size_t size = (bits / 8);
     if(bits % 8)
         ++size;
 
-    caddr_t p = TypeRef::alloc(sizeof(value) + size);
-    TypeRef::set(new(mem(p)) value(p, size, NULL));
+    caddr_t p = ar->allocate(sizeof(value) + size);
+    TypeRef::set(new(mem(p)) value(p, size, nullptr, ar));
     set(mode, 0, bits);
 }
 
@@ -350,15 +422,7 @@ void typeref<const uint8_t *>::value::destroy(void)
 	release();
 }
 
-const uint8_t *typeref<const uint8_t *>::operator*() const 
-{
-    if(!ref)
-	    return NULL;
-    value *v = polystatic_cast<value *>(ref);
-    return &v->mem[0];
-}
-
-typeref<const uint8_t *>& typeref<const uint8_t *>::operator=(const byteref& objref)
+typeref<const uint8_t *>& typeref<const uint8_t *>::operator=(const typeref<const uint8_t *>& objref)
 {
     TypeRef::set(objref);
     return *this;
@@ -370,11 +434,53 @@ typeref<const uint8_t *>& typeref<const uint8_t *>::operator=(value *bytes)
     return *this;
 }
 
-void typeref<const uint8_t *>::set(const uint8_t *str, size_t size)
+void typeref<const uint8_t *>::set(const uint8_t *str, size_t size, TypeRelease *ar)
 {
     clear();
-    caddr_t p = TypeRef::alloc(sizeof(value) + size);
-    TypeRef::set(new(mem(p)) value(p, size, str));
+    caddr_t p = ar->allocate(sizeof(value) + size);
+    TypeRef::set(new(mem(p)) value(p, size, str, ar));
+}
+
+size_t typeref<const uint8_t *>::hex(const char *str, bool ws, TypeRelease *ar)
+{
+    clear();
+    size_t size = String::hexcount(str, ws);
+    if(!size)
+        return 0;
+
+    caddr_t p = ar->allocate(sizeof(value) + size);
+    TypeRef::set(new(mem(p)) value(p, size, nullptr, ar));
+    String::hex2bin(str, data(), size, ws);
+    return size;
+}
+
+size_t typeref<const uint8_t *>::b64(const char *str, bool ws, TypeRelease *ar)
+{
+    clear();
+    size_t size = String::b64count(str, ws);
+    if(!size)
+        return 0;
+
+    caddr_t p = ar->allocate(sizeof(value) + size);
+    TypeRef::set(new(mem(p)) value(p, size, nullptr, ar));
+    String::b64decode(data(), str, size, ws);
+    return size;
+}
+
+const uint8_t *typeref<const uint8_t *>::operator*() const
+{
+    if(!ref)
+	    return NULL;
+    value *v = polystatic_cast<value *>(ref);
+    return &v->mem[0];
+}
+
+uint8_t *typeref<const uint8_t *>::data()
+{
+    if(!ref)
+	    return NULL;
+    value *v = polystatic_cast<value *>(ref);
+    return &v->mem[0];
 }
 
 void typeref<const uint8_t *>::assign(value *bytes)
@@ -383,10 +489,10 @@ void typeref<const uint8_t *>::assign(value *bytes)
     TypeRef::set(bytes);
 }
 
-typeref<const uint8_t *>::value *typeref<const uint8_t *>::create(size_t size)
+typeref<const uint8_t *>::value *typeref<const uint8_t *>::create(size_t size, TypeRelease *ar)
 {
-    caddr_t p = TypeRef::alloc(sizeof(value) + size);
-    return new(mem(p)) value(p, size);
+    caddr_t p = ar->allocate(sizeof(value) + size);
+    return new(mem(p)) value(p, size, nullptr, ar);
 }
 
 void typeref<const uint8_t *>::destroy(typeref<const uint8_t *>::value *bytes)
@@ -410,6 +516,26 @@ unsigned TypeRef::copies() const
 	return ref->copies();
 }
 
+typeref<const char *> typeref<const uint8_t *>::hex()
+{
+    typeref<const char *>str;
+    if(ref) {
+        value *v = polystatic_cast<value*>(ref);
+        str.hex(&v->mem[0], v->size, ref->getRelease());
+    }
+    return str;
+}
+
+typeref<const char *> typeref<const uint8_t *>::b64()
+{
+    typeref<const char *>str;
+    if(ref) {
+        value *v = polystatic_cast<value*>(ref);
+        str.b64(&v->mem[0], v->size, ref->getRelease());
+    }
+    return str;
+}
+
 bool typeref<const uint8_t *>::operator==(const typeref<const uint8_t *>& ptr) const 
 {
     value *v1 = polystatic_cast<value*>(ref);
@@ -427,7 +553,7 @@ bool typeref<const uint8_t *>::operator==(value *bytes) const
     return !memcmp(&(v->mem[0]), &(bytes->mem[0]), v->size);
 }
 
-typeref<const uint8_t *> typeref<const uint8_t *>::operator+(const typeref<const uint8_t *>&add) const 
+const typeref<const uint8_t *> typeref<const uint8_t *>::operator+(const typeref<const uint8_t *>&add) const 
 {
     value *v1 = polystatic_cast<value *>(ref);
     value *v2 = polystatic_cast<value *>(add.ref);
@@ -435,10 +561,12 @@ typeref<const uint8_t *> typeref<const uint8_t *>::operator+(const typeref<const
     uint8_t *out;
     size_t s1 = 0, s2 = 0, max;
     typeref<const uint8_t*> result;
+    TypeRelease *ar = nullptr;
 
     if(v1) {
         s1 = v1->max();
         b1 = v1->get();
+        ar = v1->autorelease;
     }
 
     if(v2) {
@@ -450,7 +578,7 @@ typeref<const uint8_t *> typeref<const uint8_t *>::operator+(const typeref<const
     if(!max)
         return result;
 
-    bytevalues_t bytes = byteref::create(max);
+    bytevalues_t bytes = create(max, ar);
     out = const_cast<uint8_t *>(bytes->get());
     if(s1)
         memcpy(out, b1, s1);
@@ -523,4 +651,112 @@ size_t typeref<const uint8_t *>::set(bool mode, size_t offset, size_t bits)
     return total;
 }
 
+caddr_t TypeRelease::allocate(size_t size)
+{
+    return (caddr_t)::malloc(size + Thread::cache());
+}
+
+void TypeRelease::release(TypeRef::Counted *obj)
+{
+    obj->autorelease = nullptr;
+    obj->dealloc();
+}
+
+unsigned TypeRelease::purge()
+{
+    if(delegate)
+        return delegate->purge();
+    return 0;
+}
+
+void TypeRelease::enlist(TypeRef::Counted **root, TypeRef::Counted *obj)
+{
+    obj->linkrelease = *root;
+    *root = obj;
+}
+
+TypeRef::Counted *TypeRelease::delist(TypeRef::Counted **root)
+{
+    TypeRef::Counted *obj = *root;
+    if(obj) {
+        *root = obj->linkrelease;
+        obj->autorelease = nullptr;
+    }
+    else
+        *root = nullptr;
+    return obj;
+}
+
+void TypeRelease::dealloc(TypeRef::Counted *obj)
+{
+    if(delegate)
+        delegate->release(obj);
+    else
+        release(obj);
+} 
+
+class __LOCAL TypeSecure __FINAL : public TypeRelease
+{
+private:
+	void release(TypeRef::Counted *obj) __FINAL;
+
+public:
+	inline TypeSecure() {}
+};
+
+class __LOCAL TypeReleaseLater __FINAL : public TypeRelease
+{
+private:
+    Mutex lock;
+    TypeRef::Counted *list;
+
+    void release(TypeRef::Counted *obj) __FINAL;
+
+public:
+    TypeReleaseLater() { 
+        list = nullptr; 
+    }
+
+    unsigned purge(void) __FINAL;
+};
+
+void TypeReleaseLater::release(TypeRef::Counted *obj)
+{
+    lock.acquire();
+    enlist(&list, obj);
+    lock.release();
+}
+
+unsigned TypeReleaseLater::purge()
+{
+    TypeRef::Counted *obj;
+    unsigned count = 0;
+    TypeRef::Counted *pool;
+    lock.acquire();
+    pool = list;
+    list = nullptr;
+    lock.release();
+
+    while((obj = delist(&pool)) != nullptr) {
+        TypeRelease::release(obj);
+        ++count;
+    }
+    return count;
+}
+
+void TypeSecure::release(TypeRef::Counted *obj)
+{
+    char *addr = (char *)obj + sizeof(TypeRef::Counted);
+    size_t size = TypeRelease::size(obj);
+    memset(addr, 0, size);
+    TypeRelease::release(obj);
+}
+
+static TypeSecure _secure_release;
+static TypeReleaseLater _release_later;
+
+TypeRelease auto_release;
+TypeRelease secure_release(&_secure_release);
+TypeRelease release_later(&_release_later);
+
 } // namespace
