@@ -41,13 +41,13 @@ namespace ucommon {
 const char *utf8::nil = NULL;
 const unsigned utf8::ucsize = sizeof(wchar_t);
 
-ucs4_t utf8::get(CharacterProtocol& cp)
+ucs4_t utf8::get(const char *cp)
 {
-    int ch = cp.getchar();
+    uint8_t ch = (uint8_t)(*(cp++));
     unsigned count = 0;
     ucs4_t code;
 
-    if(ch == EOF)
+    if(!ch)
         return EOF;
 
     if(ch < 0x80)
@@ -77,8 +77,8 @@ ucs4_t utf8::get(CharacterProtocol& cp)
         return EOF;
 
     while(count--) {
-        ch = cp.getchar();
-        if(ch == EOF)
+        ch = (uint8_t)*(cp++); 
+        if(!ch)
             return EOF;
         if((ch & 0xc0) != 0x80)
             return EOF;
@@ -239,69 +239,71 @@ unicode_t unidup(const char *string)
     return (unicode_t) utf8::udup(string);
 }
 
-size_t utf8::pack(unicode_t buffer, CharacterProtocol& cp, size_t len)
+size_t utf8::pack(unicode_t buffer, const char *str, size_t len)
 {
     size_t used = 0;
     wchar_t *target = (wchar_t *)buffer;
-
-    assert(len > 0);
-
     while(--len) {
-        ucs4_t code = utf8::get(cp);
+        ucs4_t code = utf8::get(str);
         if(!code || code == (ucs4_t)EOF)
             break;
+        str += chars(code);
         *(target++) = (wchar_t)code;
         ++used;
     }
-
     *target = (wchar_t)0;
-    return used;
+    return used;        
 }
 
-ucs4_t utf8::put(ucs4_t code, CharacterProtocol& cp)
+void utf8::put(ucs4_t code, char *buffer)
 {
-    char buffer[8];
-    unsigned used = 0, count = 0;
+    if(code == EOF)
+        return;
 
-    if(code < 0x80)
-        return cp.putchar(code);
+    if(code < 0x80) {
+        *buffer = code;
+        return;
+    }
 
+    unsigned used = 0;
     if(code < 0x000007ff) {
         buffer[used++] = (code >> 6) | 0xc0;
         buffer[used++] = (code & 0x3f) | 0x80;
+        return;
     }
-    else if(code <= 0x0000ffff) {
+
+    if(code <= 0x0000ffff) {
         buffer[used++] = (code >> 12) | 0xe0;
         buffer[used++] = (code >> 6 & 0x3f) | 0x80;
         buffer[used++] = (code & 0x3f) | 0x80;
+        return;
     }
-    else if(code <= 0x001fffff) {
+
+    if(code <= 0x001fffff) {
         buffer[used++] = (code >> 18) | 0xf0;
         buffer[used++] = (code >> 12 & 0x3f) | 0x80;
         buffer[used++] = (code >> 6  & 0x3f) | 0x80;
         buffer[used++] = (code & 0x3f) | 0x80;
+        return;
+
     }
-    else if(code <= 0x03ffffff) {
+
+    if(code <= 0x03ffffff) {
         buffer[used++] = (code >> 24) | 0xf8;
         buffer[used++] = (code >> 18 & 0x3f) | 0x80;
         buffer[used++] = (code >> 12 & 0x3f) | 0x80;
         buffer[used++] = (code >> 6  & 0x3f) | 0x80;
         buffer[used++] = (code & 0x3f) | 0x80;
-    }
-    else {
-        buffer[used++] = (code >> 30) | 0xfc;
-        buffer[used++] = (code >> 24 & 0x3f) | 0x80;
-        buffer[used++] = (code >> 18 & 0x3f) | 0x80;
-        buffer[used++] = (code >> 12 & 0x3f) | 0x80;
-        buffer[used++] = (code >> 6  & 0x3f) | 0x80;
-        buffer[used++] = (code & 0x3f) | 0x80;
+        return;
+
     }
 
-    while(count < used) {
-        if(cp.putchar(buffer[count++]) == EOF)
-            return EOF;
-    }
-    return code;
+    buffer[used++] = (code >> 30) | 0xfc;
+    buffer[used++] = (code >> 24 & 0x3f) | 0x80;
+    buffer[used++] = (code >> 18 & 0x3f) | 0x80;
+    buffer[used++] = (code >> 12 & 0x3f) | 0x80;
+    buffer[used++] = (code >> 6  & 0x3f) | 0x80;
+    buffer[used++] = (code & 0x3f) | 0x80;
 }
 
 ucs4_t *utf8::udup(const char *string)
@@ -309,8 +311,8 @@ ucs4_t *utf8::udup(const char *string)
     if(!string)
         return NULL;
 
-    strsize_t len = (strsize_t)count(string);
-    strsize_t pos = 0;
+    size_t len = count(string);
+    size_t pos = 0;
     ucs4_t *out = (ucs4_t *)malloc(sizeof(ucs4_t) * (++len));
     if (!out)
         return NULL;
@@ -328,8 +330,8 @@ ucs2_t *utf8::wdup(const char *string)
     if(!string)
         return NULL;
 
-    strsize_t len = (strsize_t)count(string);
-    strsize_t pos = 0;
+    size_t len = count(string);
+    size_t pos = 0;
     ucs2_t *out = (ucs2_t *)malloc(sizeof(ucs2_t) * (++len));
     ucs4_t ch;
 
@@ -349,17 +351,20 @@ ucs2_t *utf8::wdup(const char *string)
     return out;
 }
 
-size_t utf8::unpack(const unicode_t str, CharacterProtocol& cp)
+size_t utf8::unpack(const unicode_t str, char *buf, size_t bufsize)
 {
     unsigned points = 0;
     ucs4_t code;
     const wchar_t *string = (const wchar_t *)str;
 
     while(0 != (code = (*(string++)))) {
-        if(utf8::put(code, cp) == EOF)
+        size_t ps = chars(code);
+        if(ps > (bufsize - 1))
             break;
-        ++points;
+        put(code, buf);
+        buf += ps;
     }
+    *buf = 0;
     return points;
 }
 
@@ -442,13 +447,13 @@ UString::UString()
 
 UString::~UString() {}
 
-UString::UString(strsize_t size)
+UString::UString(size_t size)
 {
     str = create(size);
     str->retain();
 }
 
-UString::UString(const char *text, strsize_t size)
+UString::UString(const char *text, size_t size)
 {
     str = NULL;
     String::set(0, text, size);
@@ -469,44 +474,45 @@ UString::UString(const UString& copy)
 
 void UString::set(const unicode_t text)
 {
-    strsize_t size = (strsize_t)utf8::chars(text);
+    size_t size = utf8::chars(text);
     str = NULL;
     str = create(size);
     str->retain();
 
-    chartext cp(str->text, str->max);
-    utf8::unpack(text, cp);
+    utf8::unpack(text, str->text, str->max);
     str->fix();
 }
 
 void UString::add(const unicode_t text)
 {
-    strsize_t alloc, size;
+    size_t alloc, size;
 
-    size = alloc = (strsize_t)utf8::chars(text);
+    size = alloc = utf8::chars(text);
     if(str)
         alloc += str->len;
 
     if(!resize(alloc))
         return;
 
-    chartext cp(str->text + str->len, size);
-    utf8::unpack(text, cp);
+    utf8::unpack(text, str->text + str->len, size);
     str->fix();
 }
 
 size_t UString::get(unicode_t output, size_t points) const
 {
-    chartext cp(str->text);
-    return utf8::pack(output, cp, points);
+    const char *buf = "";
+    if(str)
+        buf = str->text;
+        
+    return utf8::pack(output, buf, points);
 }
 
-void UString::cut(strsize_t pos, strsize_t size)
+void UString::cut(size_t pos, size_t size)
 {
     if(!str)
         return;
 
-    strsize_t bpos = 0, blen = 0;
+    size_t bpos = 0, blen = 0;
     if(pos && pos != npos)
          bpos = String::offset(utf8::offset(str->text, (ssize_t)pos));
 
@@ -516,9 +522,9 @@ void UString::cut(strsize_t pos, strsize_t size)
     String::cut(bpos, blen);
 }
 
-void UString::paste(strsize_t pos, const char *text, strsize_t size)
+void UString::paste(size_t pos, const char *text, size_t size)
 {
-    strsize_t bpos = 0, blen = 0;
+    size_t bpos = 0, blen = 0;
     if(pos && pos != npos && str)
          bpos = String::offset(utf8::offset(str->text, (ssize_t)pos));
 
@@ -528,7 +534,7 @@ void UString::paste(strsize_t pos, const char *text, strsize_t size)
     String::paste(bpos, text, blen);
 }
 
-UString UString::get(strsize_t pos, strsize_t size) const
+UString UString::get(size_t pos, size_t size) const
 {
     if(!str)
         return UString("", 0);
@@ -540,11 +546,11 @@ UString UString::get(strsize_t pos, strsize_t size) const
     if(!size)
         return UString(substr, 0);
 
-    const char *end = utf8::offset(substr, size);
+    const char *end = utf8::offset(substr, (ssize_t)size);
     if(!end)
         return UString(substr);
 
-    pos = (strsize_t)(end - substr + 1);
+    pos = (size_t)(end - substr + 1);
     return UString(substr, pos);
 }
 
@@ -563,20 +569,20 @@ ucs4_t UString::at(int offset) const
     return utf8::codepoint(cp);
 }
 
-const char *UString::find(ucs4_t code, strsize_t pos) const
+const char *UString::find(ucs4_t code, size_t pos) const
 {
     if(!str)
         return NULL;
 
-    return utf8::find(str->text, code, (size_t)pos);
+    return utf8::find(str->text, code, pos);
 }
 
-const char *UString::rfind(ucs4_t code, strsize_t pos) const
+const char *UString::rfind(ucs4_t code, size_t pos) const
 {
     if(!str)
         return NULL;
 
-    return utf8::rfind(str->text, code, (size_t)pos);
+    return utf8::rfind(str->text, code, pos);
 }
 
 unsigned UString::ccount(ucs4_t code) const
@@ -587,7 +593,7 @@ unsigned UString::ccount(ucs4_t code) const
     return utf8::ccount(str->text, code);
 }
 
-UString UString::operator()(int codepoint, strsize_t size) const
+UString UString::operator()(int codepoint, size_t size) const
 {
     return UString::get(codepoint, size);
 }
