@@ -18,7 +18,7 @@
 
 #include "local.h"
 
-static const unsigned char *_salt = NULL;
+static const uint8_t *_salt = NULL;
 static unsigned _rounds = 1;
 
 namespace ucommon {
@@ -139,7 +139,7 @@ int __context::map_cipher(const char *cipher)
     }
 }
 
-void Cipher::Key::assign(const char *text, size_t size, const unsigned char *salt, unsigned count)
+void Cipher::Key::assign(const char *text, size_t size, const uint8_t *salt, unsigned count)
 {
     if(!hashid || !algoid) {
         keysize = 0;
@@ -156,7 +156,7 @@ void Cipher::Key::assign(const char *text, size_t size, const unsigned char *sal
     }
 
     char previous[MAX_DIGEST_HASHSIZE / 8];
-    unsigned char temp[MAX_DIGEST_HASHSIZE / 8];
+    uint8_t temp[MAX_DIGEST_HASHSIZE / 8];
     MD_CTX mdc;
 
     unsigned prior = 0;
@@ -199,7 +199,7 @@ void Cipher::Key::assign(const char *text, size_t size)
     assign(text, size, _salt, _rounds);
 }
 
-void Cipher::Key::options(const unsigned char *salt, unsigned rounds)
+void Cipher::Key::options(const uint8_t *salt, unsigned rounds)
 {
     _salt = salt;
     _rounds = rounds;
@@ -224,7 +224,7 @@ void Cipher::Key::set(const char *cipher)
     }
 }
 
-void Cipher::push(unsigned char *address, size_t size)
+void Cipher::push(uint8_t *address, size_t size)
 {
 }
 
@@ -242,7 +242,7 @@ bool Cipher::has(const char *cipher)
     return __context::map_cipher(cipher) != 0;
 }
 
-void Cipher::set(const key_t key, mode_t mode, unsigned char *address, size_t size)
+void Cipher::set(const key_t key, mode_t mode, uint8_t *address, size_t size)
 {
     release();
 
@@ -263,7 +263,7 @@ void Cipher::set(const key_t key, mode_t mode, unsigned char *address, size_t si
     gnutls_cipher_init((CIPHER_CTX *)&context, (CIPHER_ID)keys.algoid, &keyinfo, &ivinfo);
 }
 
-size_t Cipher::put(const unsigned char *data, size_t size)
+size_t Cipher::put(const uint8_t *data, size_t size)
 {
     if(size % keys.iosize() || !bufaddr)
         return 0;
@@ -298,11 +298,11 @@ size_t Cipher::put(const unsigned char *data, size_t size)
     return count;
 }
 
-size_t Cipher::pad(const unsigned char *data, size_t size)
+size_t Cipher::pad(const uint8_t *data, size_t size)
 {
     size_t padsz = 0;
-    unsigned char padbuf[64];
-    const unsigned char *ep;
+    uint8_t padbuf[64];
+    const uint8_t *ep;
 
     if(!bufaddr)
         return 0;
@@ -329,7 +329,7 @@ size_t Cipher::pad(const unsigned char *data, size_t size)
             memset(padbuf, keys.iosize(), keys.iosize());
         }
 
-        put((const unsigned char *)padbuf, keys.iosize());
+        put((const uint8_t *)padbuf, keys.iosize());
         zerofill(padbuf, sizeof(padbuf));
     }
 
