@@ -66,6 +66,9 @@ class MIMEItemPart;
  */
 class __EXPORT MIMEMultipart
 {
+private:
+    __DELETE_COPY(MIMEMultipart);
+
 protected:
     friend class MIMEItemPart;
     char boundry[8];
@@ -105,8 +108,9 @@ public:
      *
      * @return array of headers.
      */
-    char **getHeaders(void)
-        {return header;}
+    char **getHeaders(void) {
+        return header;
+    }
 };
 
 /**
@@ -119,6 +123,9 @@ public:
  */
 class __EXPORT MIMEMultipartForm : public MIMEMultipart
 {
+private:
+    __DELETE_COPY(MIMEMultipartForm);
+
 protected:
     virtual ~MIMEMultipartForm();
 
@@ -140,6 +147,9 @@ public:
  */
 class __EXPORT MIMEItemPart
 {
+private:
+    __DELETE_COPY(MIMEItemPart);
+
 protected:
     friend class MIMEMultipart;
 
@@ -181,6 +191,9 @@ protected:
  */
 class __EXPORT MIMEFormData : public MIMEItemPart
 {
+private:
+    __DELETE_COPY(MIMEFormData);
+
 protected:
     const char *content;
     const char *name;
@@ -193,14 +206,14 @@ public:
      *
      * @param output stream to send header to.
      */
-    void head(std::ostream *output);
+    void head(std::ostream *output) __OVERRIDE;
 
     /**
      * Stream content (value) of this form data field.
      *
      * @param output stream to send body to.
      */
-    void body(std::ostream *output);
+    void body(std::ostream *output) __OVERRIDE;
 
     /**
      * Construct form data field part of multipart form.
