@@ -62,27 +62,35 @@
 
 namespace ost {
 
-class __EXPORT MemPager : private ucommon::memalloc
+class __EXPORT MemPager : protected ucommon::memalloc
 {
+private:
+    __DELETE_COPY(MemPager);
+
 public:
     inline MemPager(size_t pagesize = 4096) : ucommon::memalloc(pagesize) {}
 
-	inline void *alloc(size_t size)
-		{return _alloc(size);}
+	inline void *alloc(size_t size) {
+		return _alloc(size);
+    }
 
 	char *alloc(const char *str);
 
-    inline char *first(const char *str)
-        {return alloc(str);}
+    inline char *first(const char *str) {
+        return alloc(str);
+    }
 
-	inline void *first(size_t size)
-		{return _alloc(size);}
+	inline void *first(size_t size) {
+		return _alloc(size);
+    }
 
-	inline int getPages(void)
-		{return pages();}
+	inline int getPages(void) const {
+		return pages();
+    }
 
-    inline void purge(void)
-        {memalloc::purge();}
+    inline void purge(void) {
+        memalloc::purge();
+    }
 };
 
 /**
@@ -95,6 +103,9 @@ public:
  */
 class __EXPORT SharedMemPager : public MemPager, public Mutex
 {
+private:
+    __DELETE_COPY(SharedMemPager);
+
 protected:
     /**
      * Create a mempager mutex pool.
@@ -116,11 +127,11 @@ protected:
      */
     void* alloc(size_t size);
 
-    inline void *first(size_t size)
-        {return alloc(size);}
+    inline void *first(size_t size) {
+        return alloc(size);
+    }
 };
 
-
 /**
  * This class is used to associate (object) pointers with named strings.
  * A virtual is used to allocate memory which can be overriden in the
@@ -140,6 +151,8 @@ private:
 
     entry *entries[KEYDATA_INDEX_SIZE];
 
+    __DELETE_COPY(Assoc);
+
 protected:
     Assoc();
     virtual ~Assoc();
