@@ -55,11 +55,13 @@ namespace ost {
 class __EXPORT Date : public ucommon::Date
 {
 protected:
-    inline void toJulian(long year, long month, long day)
-        {ucommon::Date::set(year, month, day);}
+    inline void toJulian(long year, long month, long day) {
+        ucommon::Date::set(year, month, day);
+    }
 
-    inline void fromJulian(char *buf) const
-        {put(buf);}
+    inline void fromJulian(char *buf) const {
+        put(buf);
+    }
 
 public:
     inline Date(time_t value) : ucommon::Date(value) {}
@@ -74,39 +76,49 @@ public:
 
     inline Date() : ucommon::Date() {}
 
-    inline int getYear(void) const
-        {return year();}
+    inline int getYear(void) const {
+        return year();
+    }
 
-    inline unsigned getMonth(void) const
-        {return month();}
+    inline unsigned getMonth(void) const {
+        return month();
+    }
 
-    inline unsigned getDay(void) const
-        {return day();}
+    inline unsigned getDay(void) const {
+        return day();
+    }
 
-    inline unsigned getDayOfWeek(void) const
-        {return dow();}
+    inline unsigned getDayOfWeek(void) const {
+        return dow();
+    }
 
-    inline long getJulian(void) const
-        {return julian;}
+    inline long getJulian(void) const {
+        return julian;
+    }
 
-    inline const char *get(char *buffer) const
-        {return put(buffer);}
+    inline const char *get(char *buffer) const {
+        return put(buffer);
+    }
 
-    inline time_t getTime(void) const
-        {return timeref();}
+    inline time_t getTime(void) const {
+        return timeref();
+    }
 
-    inline bool isValid(void) const
-        {return is_valid();}
+    inline bool isValid(void) const {
+        return is_valid();
+    }
 };
 
 class __EXPORT Time : public ucommon::Time
 {
 protected:
-    inline void toSeconds(int h, int m = 0, int s = 0)
-        {set(h, m, s);}
+    inline void toSeconds(int h, int m = 0, int s = 0) {
+        set(h, m, s);
+    }
 
-    inline void fromSeconds(char *buf) const
-        {put(buf);}
+    inline void fromSeconds(char *buf) const {
+        put(buf);
+    }
 
 public:
     inline Time(time_t value) : ucommon::Time(value) {}
@@ -117,22 +129,29 @@ public:
 
     inline Time(int h, int m, int s) : ucommon::Time(h, m, s) {}
 
+    inline Time(const Time& object) : ucommon::Time(object) {}
+
     inline Time() : ucommon::Time() {}
 
-    inline int getHour(void) const
-        {return hour();}
+    inline int getHour(void) const {
+        return hour();
+    }
 
-    inline int getMinute(void) const
-        {return minute();}
+    inline int getMinute(void) const {
+        return minute();
+    }
 
-    inline int getSecond(void) const
-        {return second();}
+    inline int getSecond(void) const {
+        return second();
+    }
 
-    inline const char *get(char *buffer) const
-        {return put(buffer);}
+    inline const char *get(char *buffer) const {
+        return put(buffer);
+    }
 
-    inline bool isValid(void) const
-        {return is_valid();}
+    inline bool isValid(void) const {
+        return is_valid();
+    }
 
 };
 
@@ -155,41 +174,53 @@ public:
 
     inline DateTime() : ucommon::DateTime() {}
 
-    inline int getYear(void) const
-        {return year();}
+    inline int getYear(void) const {
+        return year();
+    }
 
-    inline unsigned getMonth(void) const
-        {return month();}
+    inline unsigned getMonth(void) const {
+        return month();
+    }
 
-    inline unsigned getDay(void) const
-        {return day();}
+    inline unsigned getDay(void) const {
+        return day();
+    }
 
-    inline unsigned getDayOfWeek(void) const
-        {return dow();}
+    inline unsigned getDayOfWeek(void) const {
+        return dow();
+    }
 
-    inline long getJulian(void) const
-        {return julian;}
+    inline long getJulian(void) const {
+        return julian;
+    }
 
-    inline const char *get(char *buffer) const
-        {return ucommon::DateTime::put(buffer);}
+    inline const char *get(char *buffer) const {
+        return ucommon::DateTime::put(buffer);
+    }
 
-    inline time_t getTime(void) const
-        {return ucommon::DateTime::timeref();}
+    inline time_t getTime(void) const {
+        return ucommon::DateTime::timeref();
+    }
 
-    inline bool isValid(void) const
-        {return ucommon::DateTime::is_valid();}
+    inline bool isValid(void) const {
+        return ucommon::DateTime::is_valid();
+    }
 
-    inline int getHour(void) const
-        {return hour();}
+    inline int getHour(void) const {
+        return hour();
+    }
 
-    inline int getMinute(void) const
-        {return minute();}
+    inline int getMinute(void) const {
+        return minute();
+    }
 
-    inline int getSecond(void) const
-        {return second();}
+    inline int getSecond(void) const {
+        return second();
+    }
 
-    inline static tm_t *glt(time_t *time = NULL)
-        {return ucommon::DateTime::local(time);}
+    inline static tm_t *glt(time_t *time = NULL) {
+        return ucommon::DateTime::local(time);
+    }
 };
 
 } // namespace ost
