@@ -197,8 +197,9 @@ protected:
      *
      * @param err string or message to pass.
      */
-    inline void error(char *err)
-        {error(errExtended, err);}
+    inline void error(char *err) {
+        error(errExtended, err);
+    }
 
 
     /**
@@ -207,8 +208,9 @@ protected:
      *
      * @param enable true to enable handler.
      */
-    inline void setError(bool enable)
-        {flags.thrown = !enable;}
+    inline void setError(bool enable) {
+        flags.thrown = !enable;
+    }
 
     /**
      * Set packet read mode and "size" of packet read buffer.
@@ -220,7 +222,7 @@ protected:
      * @param size of packet read request.
      * @param btimer optional inter-byte data packet timeout.
      */
-    int setPacketInput(int size, unsigned char btimer = 0);
+    int setPacketInput(int size, uint8_t btimer = 0);
 
     /**
      * Set "line buffering" read mode and specifies the newline
@@ -270,8 +272,9 @@ protected:
      * This allows later ttystream class to open and close a serial
      * device.
      */
-    Serial()
-        {initSerial();}
+    Serial() {
+        initSerial();
+    }
 
     /**
      * A serial object may be constructed from a named file on the
@@ -356,8 +359,9 @@ public:
      *
      * @return error numbr of last Error.
      */
-    inline Error getErrorNumber(void)
-        {return errid;}
+    inline Error getErrorNumber(void) const {
+        return errid;
+    }
 
     /**
      * Often used by a "catch" to fetch the user set error string
@@ -365,8 +369,9 @@ public:
      *
      * @return string for error message.
      */
-    inline char *getErrorString(void)
-        {return errstr;}
+    inline char *getErrorString(void) const {
+        return errstr;
+    }
 
     /**
      * Get the "buffer" size for buffered operations.  This can
@@ -375,8 +380,9 @@ public:
      *
      * @return number of bytes used for buffering.
      */
-    inline int getBufferSize(void)
-        {return bufsize;}
+    inline int getBufferSize(void) const {
+        return bufsize;
+    }
 
     /**
      * Get the status of pending operations.  This can be used to
@@ -419,6 +425,8 @@ private:
     friend TTYStream& crlf(TTYStream&);
     friend TTYStream& lfcr(TTYStream&);
 
+    __DELETE_COPY(TTYStream);
+
 protected:
     char *gbuf, *pbuf;
     timeout_t timeout;
@@ -447,7 +455,7 @@ protected:
      *
      * @return char from get buffer, EOF also possible.
      */
-    int underflow(void);
+    int underflow(void) __OVERRIDE;
 
     /**
      * This streambuf method is used for doing unbuffered reads
@@ -457,7 +465,7 @@ protected:
      *
      * @return char from tty serial port, EOF also possible.
      */
-    int uflow(void);
+    int uflow(void) __OVERRIDE;
 
     /**
      * This streambuf method is used to write the output
@@ -466,7 +474,7 @@ protected:
      * @param ch char to push through.
      * @return char pushed through.
      */
-    int overflow(int ch);
+    int overflow(int ch) __OVERRIDE;
 
 public:
     /**
@@ -487,8 +495,9 @@ public:
      *
      * @param to timeout to use.
      */
-    inline void setTimeout(timeout_t to)
-        {timeout = to;}
+    inline void setTimeout(timeout_t to) {
+        timeout = to;
+    }
 
     /**
      * Set tty mode to buffered or "interactive".  When interactive,
@@ -518,7 +527,7 @@ public:
      * @param pend ready check to perform.
      * @param timeout in milliseconds.
      */
-    bool isPending(Pending pend, timeout_t timeout = TIMEOUT_INF);
+    bool isPending(Pending pend, timeout_t timeout = TIMEOUT_INF) __OVERRIDE;
 };
 
 /**
@@ -532,6 +541,9 @@ public:
 
 class __EXPORT ttystream : public TTYStream
 {
+private:
+    __DELETE_COPY(ttystream);
+
 public:
     /**
      * Construct an unopened "ttystream" object.
@@ -562,8 +574,9 @@ public:
     /**
      * Test to see if stream is opened.
      */
-    inline bool operator!()
-        {return (dev < 0);}
+    inline bool operator!() {
+        return (dev < 0);
+    }
 };
 
 /**
@@ -578,6 +591,9 @@ public:
 
 class __EXPORT TTYSession : public Thread, public TTYStream
 {
+private:
+    __DELETE_COPY(TTYSession);
+
 public:
     /**
      * Create TTY stream that will be managed by it's own thread.
@@ -633,6 +649,8 @@ private:
 
     friend class SerialService;
 
+    __DELETE_COPY(SerialPort);
+
 protected:
     /**
      * Construct a tty serial port for a named serial device.
@@ -657,8 +675,9 @@ protected:
     /**
      * Get the current state of the DetectPending flag.
      */
-    inline bool getDetectPending( void ) const
-        { return detect_pending; }
+    inline bool getDetectPending( void ) const {
+        return detect_pending; 
+    }
 
     /**
      * Used to indicate if output ready monitoring should be performed
@@ -669,8 +688,9 @@ protected:
     /**
      * Get the current state of the DetectOutput flag.
      */
-    inline bool getDetectOutput( void ) const
-        { return detect_output; }
+    inline bool getDetectOutput( void ) const {
+        return detect_output; 
+    }
 
     /**
      * Called by the service thread when the objects timer
@@ -700,8 +720,9 @@ protected:
      * @param buf address of buffer to send.
      * @param len of bytes to send.
      */
-    inline int output(void *buf, int len)
-        {return aWrite((char *)buf, len);}
+    inline int output(void *buf, int len) {
+        return aWrite((char *)buf, len);
+    }
 
     /**
      * Perform when output is available for sending data.
@@ -717,8 +738,9 @@ protected:
      * @param buf address of buffer to input.
      * @param len of input buffer used.
      */
-    inline int input(void *buf, int len)
-        {return aRead((char *)buf, len);}
+    inline int input(void *buf, int len) {
+        return aRead((char *)buf, len);
+    }
 
 public:
     /**
@@ -769,6 +791,8 @@ private:
     int count;
     SerialPort *first, *last;
 
+    __DELETE_COPY(SerialService);
+
     /**
      * Attach a new serial port to this service thread.
      *
@@ -786,7 +810,7 @@ private:
     /**
      * The service thread itself.
      */
-    void run(void);
+    void run(void) __OVERRIDE;
 
     friend class SerialPort;
 
@@ -797,7 +821,7 @@ protected:
      *
      * @param flag of update request.
      */
-    virtual void onUpdate(unsigned char flag);
+    virtual void onUpdate(uint8_t flag);
 
     /**
      * A virtual handler for event loop calls.  This can be
@@ -823,7 +847,7 @@ public:
      *
      * @param flag event for OnUpdate, termination, or reschedule.
      */
-    void update(unsigned char flag = 0xff);
+    void update(uint8_t flag = 0xff);
 
     /**
      * Create a service thread for attaching serial ports.  The
@@ -846,8 +870,9 @@ public:
      *
      * @return count of active ports.
      */
-    inline int getCount(void)
-        {return count;}
+    inline int getCount(void) const {
+        return count;
+    }
 };
 
 #endif
