@@ -94,6 +94,8 @@ protected:
     int segsize;
     void setSegmentSize(unsigned mss);
 
+    __DELETE_COPY(TCPSocket);
+
 public:
     /**
      * A method to call in a derived TCPSocket class that is acting
@@ -111,14 +113,16 @@ public:
     /**
      * Fetch out the socket.
      */
-    inline SOCKET getSocket(void)
-        {return so;}
+    inline SOCKET getSocket(void) const {
+        return so;
+    }
 
     /**
      * Get the buffer size for servers.
      */
-    inline int getSegmentSize(void)
-        {return segsize;}
+    inline int getSegmentSize(void) const {
+        return segsize;
+    }
 
     /**
      * A TCP "server" is created as a TCP socket that is bound
@@ -154,8 +158,9 @@ public:
      * @return host requesting a connection.
      * @param port number of requestor.
      */
-    inline IPV4Host getRequest(tpport_t *port = NULL) const
-        {return Socket::getIPV4Sender(port);}
+    inline IPV4Host getRequest(tpport_t *port = NULL) const {
+        return Socket::getIPV4Sender(port);
+    }
 
     /**
      * Used to reject the next incoming connection request.
@@ -165,16 +170,18 @@ public:
     /**
      * Used to get local bound address.
      */
-    inline IPV4Host getLocal(tpport_t *port = NULL) const
-        {return Socket::getIPV4Local(port);}
+    inline IPV4Host getLocal(tpport_t *port = NULL) const {
+        return Socket::getIPV4Local(port);
+    }
 
     /**
      * Used to wait for pending connection requests.
      * @return true if data packets available.
      * @param timeout in milliseconds. TIMEOUT_INF if not specified.
      */
-    inline bool isPendingConnection(timeout_t timeout = TIMEOUT_INF) /* not const -- jfc */
-        {return Socket::isPending(Socket::pendingInput, timeout);}
+    inline bool isPendingConnection(timeout_t timeout = TIMEOUT_INF) {
+        return Socket::isPending(Socket::pendingInput, timeout);
+    }
 
     /**
      * Use base socket handler for ending this socket.
@@ -213,6 +220,8 @@ private:
     int segsize;
     void setSegmentSize(unsigned mss);
 
+    __DELETE_COPY(TCPV6Socket);
+
 public:
     /**
      * A method to call in a derived TCPSocket class that is acting
@@ -230,11 +239,13 @@ public:
     /**
      * Fetch out the socket.
      */
-    inline SOCKET getSocket(void)
-        {return so;}
+    inline SOCKET getSocket(void) {
+        return so;
+    }
 
-    inline int getSegmentSize(void)
-        {return segsize;}
+    inline int getSegmentSize(void) {
+        return segsize;
+    }
 
     /**
      * A TCP "server" is created as a TCP socket that is bound
@@ -270,8 +281,9 @@ public:
      * @return host requesting a connection.
      * @param port number of requestor.
      */
-    inline IPV6Host getRequest(tpport_t *port = NULL) const
-        {return Socket::getIPV6Sender(port);}
+    inline IPV6Host getRequest(tpport_t *port = NULL) const {
+        return Socket::getIPV6Sender(port);
+    }
 
     /**
      * Used to reject the next incoming connection request.
@@ -281,16 +293,18 @@ public:
     /**
      * Used to get local bound address.
      */
-    inline IPV6Host getLocal(tpport_t *port = NULL) const
-        {return Socket::getIPV6Local(port);}
+    inline IPV6Host getLocal(tpport_t *port = NULL) const {
+        return Socket::getIPV6Local(port);
+    }
 
     /**
      * Used to wait for pending connection requests.
      * @return true if data packets available.
      * @param timeout in milliseconds. TIMEOUT_INF if not specified.
      */
-    inline bool isPendingConnection(timeout_t timeout = TIMEOUT_INF) /* not const -- jfc */
-        {return Socket::isPending(Socket::pendingInput, timeout);}
+    inline bool isPendingConnection(timeout_t timeout = TIMEOUT_INF) {
+        return Socket::isPending(Socket::pendingInput, timeout);
+    }
 
     /**
      * Use base socket handler for ending this socket.
@@ -370,7 +384,7 @@ protected:
      *
      * @return char from get buffer, EOF if not connected.
      */
-    int underflow();
+    int underflow() __OVERRIDE;
 
     /**
      * This streambuf method is used for doing unbuffered reads
@@ -380,7 +394,7 @@ protected:
      *
      * @return char from tcp socket connection, EOF if not connected.
      */
-    int uflow();
+    int uflow() __OVERRIDE;
 
     /**
      * This streambuf method is used to write the output
@@ -389,7 +403,7 @@ protected:
      * @param ch char to push through.
      * @return char pushed through.
      */
-    int overflow(int ch);
+    int overflow(int ch) __OVERRIDE;
 
     /**
      * Create a TCP stream by connecting to a TCP socket (on
@@ -420,8 +434,9 @@ protected:
      *
      * @return stream pointer of this object.
      */
-    std::iostream *tcp(void)
-        {return ((std::iostream *)this);}
+    std::iostream *tcp(void) {
+        return ((std::iostream *)this);
+    }
 
 public:
     /**
@@ -479,8 +494,9 @@ public:
      *
      * @param timer to change timeout.
      */
-    inline void setTimeout(timeout_t timer)
-        {timeout = timer;}
+    inline void setTimeout(timeout_t timer) {
+        timeout = timer;
+    }
 
 
     /**
@@ -512,7 +528,7 @@ public:
      * contains data then input is ready and if write buffer
      * contains data it is first flushed and then checked.
      */
-    bool isPending(Pending pend, timeout_t timeout = TIMEOUT_INF);
+    bool isPending(Pending pend, timeout_t timeout = TIMEOUT_INF) __OVERRIDE;
 
     /**
       * Examine contents of next waiting packet.
@@ -521,16 +537,18 @@ public:
       * @param len of packet buffer.
       * @return number of bytes examined.
       */
-     inline ssize_t peek(void *buf, size_t len)
-         {return ::recv(so, (char *)buf, (socksize_t)len, MSG_PEEK);}
+     inline ssize_t peek(void *buf, size_t len) {
+         return ::recv(so, (char *)buf, (socksize_t)len, MSG_PEEK);
+    }
 
     /**
      * Return the size of the current stream buffering used.
      *
      * @return size of stream buffers.
      */
-    inline size_t getBufferSize(void) const
-        {return bufsize;}
+    inline size_t getBufferSize(void) const {
+        return bufsize;
+    }
 };
 
 /**
@@ -547,6 +565,7 @@ class __EXPORT TCPSession : public Thread, public TCPStream
 {
 private:
     TCPSession(const TCPSession &rhs); // not defined
+
 protected:
     /**
      * Normally called during the thread Initial() method by default,
@@ -568,7 +587,7 @@ protected:
      * terminates without having to wait for a connection request
      * to complete.
      */
-    void initial(void);
+    void initial(void) __OVERRIDE;
 
 public:
     /**
