@@ -30,7 +30,7 @@
 #include <ucommon/platform.h>
 #endif
 
-#ifdef  _MSWINDOWS_
+#if defined(_MSWINDOWS_)
 typedef LONG atomic_t;
 #else
 typedef int atomic_t;
@@ -46,17 +46,12 @@ namespace ucommon {
  * code is available.
  * @author David Sugar <dyfet@gnutelephony.org>
  */
-class __EXPORT atomic
+class __EXPORT Atomic
 {
 private:
-    __DELETE_DEFAULTS(atomic);
+    __DELETE_DEFAULTS(Atomic);
 
 public:
-    /**
-     * Set to true if atomics have to be simulated with mutexes.
-     */
-    static const bool simulated;
-
     /**
      * Atomic counter class.  Can be used to manipulate value of an
      * atomic counter without requiring explicit thread locking.
@@ -72,6 +67,10 @@ public:
     public:
         counter(atomic_t initial = 0);
 
+        // optimized reference count semantics
+        atomic_t fetch_retain() volatile;
+        atomic_t fetch_release() volatile;
+
         // fetch add/sub optimized semantics
         atomic_t fetch_add(atomic_t offset = 1) volatile;
         atomic_t fetch_sub(atomic_t offset = 1) volatile;
@@ -131,16 +130,49 @@ public:
         void release(void) volatile;
     };
 
-    /**
-     * Atomically aligned heap alloc function.
-     * @param size of memory to allocate.
-     * @return pointer or NULL if cannot alloc.
-     */
-    static void *alloc(size_t size);
-};
+    class __EXPORT Aligned
+    {
+    private:
+        __DELETE_DEFAULTS(Aligned);
 
-// for abi7
-typedef atomic Atomic;
+    protected:
+        void *address;
+        size_t offset;
+
+        Aligned(size_t object, size_t offset = 0);
+    
+    public:
+        virtual ~Aligned();
+    };
+
+    template<typename T, unsigned alignment = 0>
+    class aligned : public Aligned
+    {
+    protected:
+        inline T* get() const {
+            return static_cast<T*>(address);
+        }
+
+    public:
+        inline aligned() : Aligned(sizeof(T), alignment) { 
+            new((caddr_t)address) T;
+        }
+        
+        inline T& operator*() const {
+            return *(static_cast<T*>(address));
+        }
+
+        inline operator T&() {
+            return *get();
+        }
+
+        inline void operator()(T value) {
+            *get() = value;
+        }
+    };
+
+    static bool is_lockfree(void);
+};
 
 } // namespace ucommon
 
