@@ -38,8 +38,8 @@
 #include <ucommon/numbers.h>
 #endif
 
-#ifndef _UCOMMON_STRING_H_
-#include <ucommon/string.h>
+#ifndef _UCOMMON_TYPEREF_H_
+#include <ucommon/typeref.h>
 #endif
 
 #ifndef _MSWINDOWS_
@@ -220,7 +220,7 @@ public:
      * julian date.
      * @return ISO date string.
      */
-    String operator()() const;
+    stringref_t operator()() const;
 
     /**
      * Increment date by one day.
@@ -253,14 +253,14 @@ public:
      * @param days to add.
      * @return new date object with modified days.
      */
-    Date operator+(long days);
+    const Date operator+(long days) const;
 
     /**
      * Subtract days from a julian date in an expression.
      * @param days to subtract.
      * @return new date object with modified days.
      */
-    Date operator-(long days);
+    const Date operator-(long days) const;
 
     /**
      * Operator to compute number of days between two dates.
@@ -499,14 +499,14 @@ public:
      * @param seconds to add.
      * @return new time object with modified value.
      */
-    Time operator+(long seconds);
+    const Time operator+(long seconds) const;
 
     /**
      * Subtract seconds to the current time, wrap if 24 hours.
      * @param seconds to subtract.
      * @return new time object with modified value.
      */
-    Time operator-(long seconds);
+    const Time operator-(long seconds) const;
 
     /**
      * Get time in seconds.
@@ -528,7 +528,7 @@ public:
      * Convert to standard 24 hour time string.
      * @return time string.
      */
-    String operator()() const;
+    stringref_t operator()() const;
 
     /**
      * Incrememnt time by 1 second, wrap on 24 hour period.
@@ -618,7 +618,7 @@ public:
 class __EXPORT DateTime : public Date, public Time
 {
 protected:
-    void update(void);
+    virtual void update(void) __OVERRIDE;
 
 public:
     /**
@@ -728,7 +728,7 @@ public:
      * @param seconds to add to datetime.
      * @return new modified datetime object.
      */
-    DateTime operator+(long seconds);
+    const DateTime operator+(long seconds) const;
 
     /**
      * Subtract seconds from datetime in an expression.  Day underflows
@@ -736,7 +736,7 @@ public:
      * @param seconds to subtract from datetime.
      * @return new modified datetime object.
      */
-    DateTime operator-(long seconds);
+    const DateTime operator-(long seconds) const;
 
     /**
      * Add a day from the current date and time.
@@ -830,7 +830,7 @@ public:
      * @param strftime format to use.
      * @return String object with formatted time.
      */
-    String format(const char *strftime) const;
+    stringref_t format(const char *strftime) const;
 
     /**
      * Fetch an instance of time converted to local time.  If the localtime
@@ -882,7 +882,7 @@ private:
     mode_t mode;
 
 protected:
-    void update(void);
+    virtual void update(void) __OVERRIDE;
 
 public:
     /**
@@ -971,7 +971,7 @@ public:
 class __EXPORT DateNumber : public Number, public Date
 {
 protected:
-    void update(void);
+    virtual void update(void) __OVERRIDE;
 
 public:
     /**
@@ -991,7 +991,7 @@ public:
     void set(void);
 };
 
-class __EXPORT isotime : public PrintProtocol, public InputProtocol
+class __EXPORT isotime : public __PROTOCOL PrintProtocol, public __PROTOCOL InputProtocol
 {
 private:
     Date *d;
@@ -1007,7 +1007,7 @@ private:
 protected:
     const char *_print(void) const;
 
-    int _input(int code);
+    int _input(int code) __OVERRIDE;
 
 public:
     isotime(Date& date, Time& time);
@@ -1037,8 +1037,4 @@ typedef Time        tod_t;
 
 } // namespace ucommon
 
-extern "C" {
-    __EXPORT long tzoffset(struct timezone *tz = NULL);
-}
-
 #endif
