@@ -301,7 +301,7 @@ inline bool is(T& object) {
  */
 template<typename T>
 inline bool isnull(T& object) {
-    return (bool)(object.operator*() == NULL);
+    return (bool)(object.operator*() == nullptr);
 }
 
 /**
@@ -312,7 +312,7 @@ inline bool isnull(T& object) {
  */
 template<typename T>
 inline bool isnullp(T *object) {
-    return (bool)(object->operator*() == NULL);
+    return (bool)(object->operator*() == nullptr);
 }
 
 /**
@@ -379,7 +379,7 @@ inline void store_unsafe(T& target, const T* source) {
 }
 
 /**
- * Convenience function to swap objects.
+ * Convenience function to swap objects.  Can be specialized.
  * @param o1 to swap.
  * @param o2 to swap.
  */
@@ -388,6 +388,37 @@ inline void swap(T& o1, T& o2) {
     cpr_memswap(&o1, &o2, sizeof(T));
 }
 
+/**
+ * Convenience function to copy objects.
+ */
+template<typename T>
+inline T copy(const T& src) {
+    return T(src);
+}
+
+template<typename T>
+inline T& copy(const T& src, T& to) {
+    new((caddr_t)&to) T(src);
+    return to;
+}
+
+/**
+ * Convenience function to move objects.
+ */
+template<typename T>
+inline T& move(T& src, T& to) {
+    memcpy((void *)&to, (void *)&src, sizeof(T));
+    new((caddr_t)&src) T();
+    return to;
+} 
+
+template<typename T>
+inline T& clear(T& o) {
+    o.~T();
+    new((caddr_t)&o) T();
+    return o;
+}
+
 /**
  * Convenience function to check memory arrays.
  * @param pointer to validate.
@@ -446,6 +477,7 @@ inline T& (limit)(T& value, T& low, T& high) {
  */
 template<typename T>
 inline T& deref_pointer(T *pointer) {
+    __THROW_DEREF(pointer);
     return *pointer;
 }
 
