@@ -41,10 +41,6 @@
 #include <ucommon/object.h>
 #endif
 
-#ifdef nil
-#undef nil
-#endif
-
 namespace ucommon {
 
 class OrderedObject;
@@ -56,14 +52,13 @@ class OrderedObject;
  * start of list of objects that are linked together as a list.
  * @author David Sugar <dyfet@gnutelephony.org>
  */
-class __EXPORT LinkedObject : public ObjectProtocol
+class __EXPORT LinkedObject : public __PROTOCOL ObjectProtocol
 {
-protected:
+private:
     friend class OrderedIndex;
-    friend class LinkedRing;
     friend class NamedObject;
-    friend class ObjectStack;
 
+protected:
     LinkedObject *Next;
 
     /**
@@ -79,10 +74,9 @@ protected:
      */
     LinkedObject();
 
-public:
-    static const LinkedObject *nil; /**< Marker for end of linked list. */
-    static const LinkedObject *inv; /**< Marker for invalid list pointer */
+    LinkedObject(const LinkedObject& from);
 
+public:
     virtual ~LinkedObject();
 
     /**
@@ -184,7 +178,6 @@ class __EXPORT OrderedIndex
 protected:
     friend class OrderedObject;
     friend class DLinkedObject;
-    friend class LinkedList;
     friend class NamedObject;
 
     OrderedObject *head, *tail;
@@ -318,12 +311,11 @@ public:
  */
 class __EXPORT OrderedObject : public LinkedObject
 {
-protected:
-    friend class LinkedList;
-    friend class OrderedIndex;
+private:
     friend class DLinkedObject;
-    friend class ObjectQueue;
+    friend class OrderedIndex;
 
+protected:
     /**
      * Construct an ordered object aot end of a an index.
      * @param index we are listed on.
@@ -335,6 +327,8 @@ protected:
      */
     OrderedObject();
 
+    OrderedObject(const OrderedObject& from);
+
 public:
     /**
      * List our ordered object at end of a linked list on an index.
@@ -370,30 +364,6 @@ public:
     }
 };
 
-/**
- * A double-linked Object, used for certain kinds of lists.
- * @author David Sugar <dyfet@gnutelephony.org>
- */
-class __EXPORT DLinkedObject : public OrderedObject
-{
-public:
-    friend class ObjectQueue;
-
-    /**
-     * Construct an empty object.
-     */
-    DLinkedObject();
-
-protected:
-    /**
-     * Remove a cross-linked list from itself.
-     */
-    void delist(void);
-
-private:
-    DLinkedObject *Prev;
-};
-
 /**
  * A linked object base class with members found by name.  This class is
  * used to help form named option lists and other forms of name indexed
@@ -795,30 +765,32 @@ public:
  * convenient insertion and deletion of list members anywhere in the list.
  * @author David Sugar <dyfet@gnutelephony.org>
  */
-class __EXPORT LinkedList : public OrderedObject
+class __EXPORT DLinkedObject : public OrderedObject
 {
 protected:
     friend class ObjectQueue;
 
-    LinkedList *Prev;
+    DLinkedObject *Prev;
     OrderedIndex *Root;
 
     /**
      * Construct and add our object to an existing double linked list at end.
      * @param index of linked list we are listed in.
      */
-    LinkedList(OrderedIndex *index);
+    DLinkedObject(OrderedIndex *index);
 
     /**
      * Construct an unlinked object.
      */
-    LinkedList();
+    DLinkedObject();
+
+    DLinkedObject(const DLinkedObject& from);
 
     /**
      * Delete linked list object.  If it is a member of a list of objects,
      * then the list is reformed around us.
      */
-    virtual ~LinkedList();
+    virtual ~DLinkedObject();
 
 public:
     /**
@@ -852,7 +824,7 @@ public:
      * @return true if we are the first node in a list.
      */
     inline bool is_head(void) const {
-        return polypointer_cast<LinkedList *>(Root->head) == this;
+        return polypointer_cast<DLinkedObject *>(Root->head) == this;
     }
 
     /**
@@ -860,272 +832,68 @@ public:
      * @return true if we are the last node in a list.
      */
     inline bool is_tail(void) const {
-        return polypointer_cast<LinkedList *>(Root->tail) == this;
+        return polypointer_cast<DLinkedObject *>(Root->tail) == this;
     }
 
     /**
      * Get previous node in the list for reverse iteration.
      * @return previous node in list.
      */
-    inline LinkedList *getPrev(void) const {
-        return static_cast<LinkedList*>(Prev);
+    inline DLinkedObject *getPrev(void) const {
+        return static_cast<DLinkedObject*>(Prev);
     }
 
     /**
      * Get next node in the list when iterating.
      * @return next node in list.
      */
-    inline LinkedList *getNext(void) const {
-        return static_cast<LinkedList*>(LinkedObject::getNext());
+    inline DLinkedObject *getNext(void) const {
+        return static_cast<DLinkedObject*>(LinkedObject::getNext());
     }
 
     /**
      * Insert object behind our object.
      * @param object to add to list.
      */
-    void insertTail(LinkedList *object);
+    void insertTail(DLinkedObject *object);
 
     /**
      * Insert object in front of our object.
      * @param object to add to list.
      */
-    void insertHead(LinkedList *object);
+    void insertHead(DLinkedObject *object);
 
     /**
      * Insert object, method in derived object.
      * @param object to add to list.
      */
-    virtual void insert(LinkedList *object);
+    virtual void insert(DLinkedObject *object);
 
     /**
      * Insert object behind our object.
      * @param object to add to list.
      */
-    inline void operator+=(LinkedList *object) {
+    inline DLinkedObject& operator+=(DLinkedObject *object) {
         insertTail(object);
+        return *this;
     }
 
     /**
      * Insert object in front of our object.
      * @param object to add to list.
      */
-    inline void operator-=(LinkedList *object) {
+    inline DLinkedObject& operator-=(DLinkedObject *object) {
         insertHead(object);
+        return *this;
     }
 
     /**
      * Insert object in list with our object.
      * @param object to add to list.
      */
-    inline void operator*=(LinkedList *object) {
+    inline DLinkedObject& operator*=(DLinkedObject *object) {
         insert(object);
-    }
-};
-
-/**
- * A queue of double linked object.  This uses the linkedlist class to
- * form a basic queue of objects.
- * @author David Sugar <dyfet@gnutelephony.org>
- */
-class __EXPORT ObjectQueue : public OrderedIndex
-{
-public:
-    /**
-     * Create an empty object queue.
-     */
-    ObjectQueue();
-
-    /**
-     * Add an object to the end of the queue.
-     * @param object to add.
-     */
-    void add(DLinkedObject *object);
-
-    /**
-     * Push an object to the front of the queue.
-     * @param object to push.
-     */
-    void push(DLinkedObject *object);
-
-    /**
-     * Pull an object from the front of the queue.
-     * @return object pulled or NULL if empty.
-     */
-    DLinkedObject *pull(void);
-
-    /**
-     * Pop an object from the end of the queue.
-     * @return object popped or NULL if empty.
-     */
-    DLinkedObject *pop(void);
-};
-
-class __EXPORT ObjectStack
-{
-protected:
-    LinkedObject *root;
-
-public:
-    /**
-     * Create an empty stack.
-     */
-    ObjectStack();
-
-    /**
-     * Create a stack from an existing list of objects.
-     * @param list of already linked objects.
-     */
-    ObjectStack(LinkedObject *list);
-
-    /**
-     * Push an object onto the stack.
-     * @param object to push.
-     */
-    void push(LinkedObject *object);
-
-    /**
-     * Pull an object from the stack.
-     * @return object popped from stack or NULL if empty.
-     */
-    LinkedObject *pull(void);
-
-    /**
-     * Pop an object from the stack.
-     * @return object popped from stack or NULL if empty.
-     */
-    inline LinkedObject *pop(void) {
-        return ObjectStack::pull();
-    }
-};
-
-
-/**
- * A multipath linked list where membership is managed in multiple
- * lists.
- * @author David Sugar <dyfet@gnutelephony.org>
- */
-class __EXPORT MultiMap : public ReusableObject
-{
-private:
-    typedef struct {
-        const char *key;
-        size_t keysize;
-        MultiMap *next;
-        MultiMap **root;
-    }   link_t;
-
-    unsigned paths;
-    link_t *links;
-
-protected:
-    /**
-     * Initialize a multilist object.
-     * @param count of link paths.
-     */
-    MultiMap(unsigned count);
-
-    /**
-     * Destroy a multilist object.
-     */
-    virtual ~MultiMap();
-
-    /**
-     * Modifiable interface for key matching.
-     * @param path to check.
-     * @param key to check.
-     * @param size of key to check or 0 if NULL terminated string.
-     * @return true if matches key.
-     */
-    virtual bool equal(unsigned path, caddr_t key, size_t size) const;
-
-public:
-    /**
-     * Enlist on a single linked list.
-     * @param path to attach through.
-     * @param root of list to attach.
-     */
-    void enlist(unsigned path, MultiMap **root);
-
-    /**
-     * Enlist binary key on a single map path.
-     * @param path to attach through.
-     * @param index to attach to.
-     * @param key value to use.
-     * @param size of index.
-     * @param keysize of key or 0 if NULL terminated string.
-     */
-    void enlist(unsigned path, MultiMap **index, caddr_t key, unsigned size, size_t keysize = 0);
-
-    /**
-     * De-list from a single map path.
-     * @param path to detach from.
-     */
-    void delist(unsigned path);
-
-    /**
-     * Get next node from single chain.
-     * @param path to follow.
-     */
-    MultiMap *next(unsigned path) const;
-
-    /**
-     * Compute binary key index.
-     * @param key memory to compute.
-     * @param max size of index.
-     * @param size of key or 0 if NULL terminated string.
-     * @return associated hash value.
-     */
-    static unsigned keyindex(caddr_t key, unsigned max, size_t size = 0);
-
-    /**
-     * Find a multikey node.
-     * @return node that is found or NULL if none.
-     * @param path of table.
-     * @param index of hash table.
-     * @param key to locate.
-     * @param max size of index.
-     * @param size of key or 0 if NULL terminated string.
-     */
-    static MultiMap *find(unsigned path, MultiMap **index, caddr_t key, unsigned max, size_t size = 0);
-};
-
-/**
- * Template value class to embed data structure into a named list.
- * This is used to form a class which can be searched by name and that
- * contains a member value object.  Most of the core logic for this
- * template is found in and derived from the object_value template.
- * @author David Sugar <dyfet@gnutelephony.org>
- */
-template <typename T, class O=NamedObject>
-class named_value : public object_value<T, O>
-{
-public:
-    /**
-     * Construct embedded named object on a linked list.
-     * @param root node or pointer for list.
-     * @param name of our object.
-     */
-    inline named_value(LinkedObject **root, char *name) {
-        LinkedObject::enlist(root); 
-        O::id = name;
-    }
-
-    /**
-     * Assign embedded value from related type.
-     * @param typed_value to assign.
-     */
-    inline void operator=(const T& typed_value) {
-        this->set(typed_value);
-    }
-
-    /**
-     * Find embedded object in chain by name.
-     * @param first object in list to search from.
-     * @param name to search for.
-     * @return composite object found by name or NULL if not found.
-     */
-    inline static named_value find(named_value *first, const char *name) {
-        return static_cast<named_value *>(NamedObject::find(first, name));
+        return *this;
     }
 };
 
@@ -1137,10 +905,15 @@ public:
  * template is found in and derived from the object_value template.
  * @author David Sugar <dyfet@gnutelephony.org>
  */
-template <typename T, class O=OrderedObject>
-class linked_value : public object_value<T, O>
+template <typename T, class O = LinkedObject>
+class linked_value : public O
 {
+protected:
+    __DELETE_COPY(linked_value);
+
 public:
+    T value;
+
     /**
      * Create embedded value object unlinked.
      */
@@ -1169,7 +942,7 @@ public:
      */
     inline linked_value(LinkedObject **root, const T& typed_value) {
         LinkedObject::enlist(root); 
-        this->set(typed_value);
+        value = typed_value;
     }
 
     /**
@@ -1178,163 +951,33 @@ public:
      * @param typed_value to assign.
      */
     inline linked_value(OrderedIndex *index, const T& typed_value) {
-        O::enlist(index); 
-        this->set(typed_value);
+        O::enlist(index);
+        value = typed_value; 
+    }
+
+    inline void set(const T& typed_value) {
+        value = typed_value;
     }
 
     /**
      * Assign embedded value from related type.
      * @param typed_value to assign.
      */
-    inline void operator=(const T& typed_value) {
-        this->set(typed_value);
-    }
-};
-
-/**
- * Template for typesafe basic object stack container.  The object type, T,
- * that is contained in the stack must be derived from LinkedObject.
- * @author David Sugar <dyfet@gnutelephony.org>
- */
-template <class T>
-class objstack : public ObjectStack
-{
-public:
-    /**
-     * Create a new object stack.
-     */
-    inline objstack() : ObjectStack() {}
-
-    /**
-     * Create an object stack from a list of objects.
-     */
-    inline objstack(T *list) : ObjectStack(list) {}
-
-    /**
-     * Push an object onto the object stack.
-     * @param object of specified type to push.
-     */
-    inline void push(T *object) {
-        ObjectStack::push(polypointer_cast<LinkedObject*>(object));
+    inline linked_value& operator=(const T& typed_value) {
+        value = typed_value;
+        return *this;
     }
 
-    /**
-     * Add an object onto the object stack.
-     * @param object of specified type to push.
-     */
-    inline void add(T *object) {
-        ObjectStack::push(polypointer_cast<LinkedObject*>(object));
+    inline T& operator*() {
+        return value;
     }
 
-    /**
-     * Pull an object from the object stack.
-     * @return object of specified type or NULL if empty.
-     */
-    inline T *pull(void) {
-        return static_cast<T *>(ObjectStack::pull());
+    inline operator T&() {
+        return value;
     }
 
-    /**
-     * Pull (pop) an object from the object stack.
-     * @return object of specified type or NULL if empty.
-     */
-    inline T *pop(void) {
-        return static_cast<T *>(ObjectStack::pull());
-    }
-};
-
-/**
- * Template for typesafe basic object fifo container.  The object type, T,
- * that is contained in the fifo must be derived from OrderedObject or
- * LinkedObject.
- * @author David Sugar <dyfet@gnutelephony.org>
- */
-template <class T>
-class objfifo : public OrderedIndex
-{
-public:
-    /**
-     * Create a new object stack.
-     */
-    inline objfifo() : OrderedIndex() {}
-
-    /**
-     * Push an object onto the object fifo.
-     * @param object of specified type to push.
-     */
-    inline void push(T *object) {
-        OrderedIndex::add(polypointer_cast<OrderedObject *>(object));
-    }
-
-    /**
-     * Add an object onto the object fifo.
-     * @param object of specified type to push.
-     */
-    inline void add(T *object) {
-        OrderedIndex::add(polypointer_cast<OrderedObject *>(object));
-    }
-
-    /**
-     * Pull an object from the object stack.
-     * @return object of specified type or NULL if empty.
-     */
-    inline T *pull(void) {
-        return static_cast<T *>(OrderedIndex::get());
-    }
-
-    /**
-     * Pull (pop) an object from the object stack.
-     * @return object of specified type or NULL if empty.
-     */
-    inline T *pop(void) {
-        return static_cast<T *>(OrderedIndex::get());
-    }
-};
-
-/**
- * Template for typesafe basic object queue container.  The object type, T,
- * that is contained in the fifo must be derived from DLinkedObject.
- * @author David Sugar <dyfet@gnutelephony.org>
- */
-template <class T>
-class objqueue : public ObjectQueue
-{
-public:
-    /**
-     * Create a new object stack.
-     */
-    inline objqueue() : ObjectQueue() {}
-
-    /**
-     * Push an object to start of queue.
-     * @param object of specified type to push.
-     */
-    inline void push(T *object) {
-        ObjectQueue::push(polypointer_cast<DLinkedObject *>(object));
-    }
-
-    /**
-     * Add an object to the end of the object queue.
-     * @param object of specified type to add.
-     */
-    inline void add(T *object) {
-        ObjectQueue::add(polypointer_cast<DLinkedObject *>(object));
-    }
-
-    /**
-     * Pull an object from the start of the object queue.
-     * @return object of specified type or NULL if empty.
-     */
-    inline T *pull(void) {
-        return static_cast<T *>(ObjectQueue::pull());
-    }
-
-    /**
-     * Pop an object from the end of the object queue.
-     * @return object of specified type or NULL if empty.
-     */
-    inline T *pop(void) {
-        return static_cast<T *>(ObjectQueue::pop());
+    inline void operator()(const T data) {
+        value = data;
     }
 };
 
@@ -1541,93 +1184,6 @@ public:
     }
 };
 
-/**
- * Embed data objects into a multipap structured memory database.  This
- * can be used to form multi-key hash nodes.  Embedded values can either be
- * of direct types that are then stored as part of the template object, or
- * of class types that are data pointers.
- * @author David Sugar <dyfet@gnutelephony.org>
- */
-template <typename T, unsigned P>
-class multimap : public MultiMap
-{
-protected:
-    T value;
-
-public:
-    /**
-     * Construct a multimap node.
-     */
-    inline multimap() : MultiMap(P) {}
-
-    /**
-     * Destroy a multimap object.
-     */
-    inline ~multimap() {}
-
-    /**
-     * Return the typed value of this node.
-     * @return reference to value of node.
-     */
-    inline T &get(void) const {
-        return value;
-    }
-
-    /**
-     * Return next multimap typed object.
-     * @param path to follow.
-     * @return multimap typed.
-     */
-    inline multimap *next(unsigned path) {
-        return static_cast<multimap*>(MultiMap::next(path));
-    }
-
-    /**
-     * Return typed value of this node by pointer reference.
-     * @return value of node.
-     */
-    inline T& operator*() const {
-        return value;
-    }
-
-    /**
-     * Set the pointer of a pointer based value tree.
-     * @param pointer to set.
-     */
-    inline void setPointer(const T pointer) {
-        value = pointer;
-    }
-
-    /**
-     * Set the value of a data based value tree.
-     * @param reference to value to copy into node.
-     */
-    inline void set(const T &reference) {
-        value = reference;
-    }
-
-    /**
-     * Assign the value of our node.
-     * @param data value to assign.
-     */
-    inline void operator=(const T& data) {
-        value = data;
-    }
-
-    /**
-     * Find multimap key entry.
-     * @param path to search through.
-     * @param index of associated keys.
-     * @param key to search for, binary or NULL terminated string.
-     * @param size of index used.
-     * @param keysize or 0 if NULL terminated string.
-     * @return multipath typed object.
-     */
-    inline static multimap *find(unsigned path, MultiMap **index, caddr_t key, unsigned size, unsigned keysize = 0) {
-        return static_cast<multimap*>(MultiMap::find(path, index, key, size, keysize));
-    }
-};
-
 /**
  * Embed data objects into a tree structured memory database.  This can
  * be used to form XML document trees or other data structures that
@@ -1844,265 +1400,12 @@ public:
     }
 };
 
-/**
- * A template class for a hash map.  This provides a has map index object as
- * a chain of keyindex selected linked pointers of a specified size.  This
- * is used for the index and size values for NamedObject's which are listed
- * on a hash map.
- * @author David Sugar <dyfet@gnutelephony.org>
- */
-template <class T, unsigned M = 177>
-class keymap
-{
-private:
-    NamedObject *idx[M];
-
-public:
-    /**
-     * Destroy the hash map by puring the index chains.
-     */
-    inline ~keymap() {
-        NamedObject::purge(idx, M);
-    }
-
-    /**
-     * Retrieve root of index to use in NamedObject constructors.
-     * @return root node of index.
-     */
-    inline NamedObject **root(void) const {
-        return idx;
-    }
-
-    /**
-     * Retrieve key size to use in NamedObject constructors.
-     * @return key size of hash map.
-     */
-    inline unsigned limit(void) const {
-        return M;
-    }
-
-    /**
-     * Find a typed object derived from NamedObject in the hash map by name.
-     * @param name to search for.
-     * @return typed object if found through map or NULL.
-     */
-    inline T *get(const char *name) const {
-        return static_cast<T*>(NamedObject::map(idx, name, M));
-    }
-
-    /**
-     * Find a typed object derived from NamedObject in the hash map by name.
-     * @param name to search for.
-     * @return typed object if found through map or NULL.
-     */
-    inline T& operator[](const char *name) const {
-        return static_cast<T*>(NamedObject::map(idx, name, M));
-    }
-
-    /**
-     * Add a typed object derived from NamedObject to the hash map by name.
-     * @param name to add.
-     * @param object to add.
-     */
-    inline void add(const char *name, T& object) {
-        object.NamedObject::add(idx, name, M);
-    }
-
-    /**
-     * Add a typed object derived from NamedObject to the hash map by name.
-     * @param name to add.
-     * @param object to add.
-     */
-    inline void add(const char *name, T *object) {
-        object->NamedObject::add(idx, name, M);
-    }
-
-    /**
-     * Remove a typed object derived from NamedObject to the hash map by name.
-     * @param name to remove.
-     * @return object removed if found or NULL.
-     */
-    inline T *remove(const char *name) {
-        return static_cast<T*>(NamedObject::remove(idx, name, M));
-    }
-
-    /**
-     * Find first typed object in hash map to iterate.
-     * @return first typed object or NULL if nothing in list.
-     */
-    inline T *begin(void) const {
-        return static_cast<T*>(NamedObject::skip(idx, NULL, M));
-    }
-
-    /**
-     * Find next typed object in hash map for iteration.
-     * @param current typed object we are referencing.
-     * @return next iterative object or NULL if past end of map.
-     */
-    inline T *next(T *current) const {
-        return static_cast<T*>(NamedObject::skip(idx, current, M));
-    }
-
-    /**
-     * Count the number of typed objects in our hash map.
-     * @return count of typed objects.
-     */
-    inline unsigned count(void) const {
-        return NamedObject::count(idx, M);
-    }
-
-    /**
-     * Convert our hash map into a linear object pointer array.  The
-     * object pointer array is created from the heap and must be deleted
-     * when no longer used.
-     * @return array of typed named object pointers.
-     */
-    inline T **index(void) const {
-        return NamedObject::index(idx, M);
-    }
-
-    /**
-     * Convert our hash map into an alphabetically sorted linear object
-     * pointer array.  The object pointer array is created from the heap
-     * and must be deleted when no longer used.
-     * @return sorted array of typed named object pointers.
-     */
-    inline T **sort(void) const {
-        return NamedObject::sort(NamedObject::index(idx, M));
-    }
-
-    /**
-     * Convenience typedef for iterative pointer.
-     */
-    typedef linked_pointer<T> iterator;
-};
-
-/**
- * A template for ordered index of typed name key mapped objects.
- * This is used to hold an iterable linked list of typed named objects
- * where we can find objects by their name as well as through iteration.
- * @author David Sugar <dyfet@gnutelephony.org>
- */
-template <class T>
-class keylist : public OrderedIndex
-{
-public:
-    /**
-     * Return a root node pointer to use in NamedObject constructors.
-     * @return pointer to index root.
-     */
-    inline NamedObject **root(void) {
-        return static_cast<NamedObject**>(&head);
-    }
-
-    /**
-     * Return first item in ordered list.  This is commonly used to
-     * iterate the list.
-     * @return first item in list or NULL if empty.
-     */
-    inline T *begin(void) {
-        return static_cast<T*>(head);
-    }
-
-    /**
-     * Return last item in ordered list.  This is commonly used to determine
-     * end of list iteration.
-     * @return last item in list or NULL if empty.
-     */
-    inline T *end(void) {
-        return static_cast<T*>(tail);
-    }
-
-    /**
-     * Create a new typed named object with default constructor.
-     * This creates a new object which can be deleted.
-     * @param name of object to create.
-     * @return typed named object.
-     */
-    inline T *create(const char *name) {
-        return new T(this, name);
-    }
-
-    /**
-     * Iterate next object in list.
-     * @param current object we are referencing.
-     * @return next logical object in linked list or NULL if end.
-     */
-    inline T *next(LinkedObject *current) {
-        return static_cast<T*>(current->getNext());
-    }
-
-    /**
-     * Find a specific object by name.
-     * @param name to search for.
-     * @return type named object that matches or NULL if not found.
-     */
-    inline T *find(const char *name) {
-        return static_cast<T*>(NamedObject::find(begin(), name));
-    }
-
-    inline T *offset(unsigned offset) {
-        return static_cast<T*>(OrderedIndex::find(offset));
-    }
-
-    /**
-     * Retrieve a specific object by position in list.
-     * @param offset in list for object we want.
-     * @return type named object or NULL if past end of list.
-     */
-    inline T& operator[](unsigned offset) {
-        return static_cast<T&>(OrderedIndex::find(offset));
-    }
-
-    inline T& operator[](const char *name) {
-        return static_cast<T&>(NamedObject::find(begin(), name));
-    }
-
-    /**
-     * Convert our linked list into a linear object pointer array.  The
-     * object pointer array is created from the heap and must be deleted
-     * when no longer used.
-     * @return array of typed named object pointers.
-     */
-    inline T **index(void) {
-        return static_cast<T**>(OrderedIndex::index());
-    }
-
-    /**
-     * Convert our linked list into an alphabetically sorted linear object
-     * pointer array.  The object pointer array is created from the heap
-     * and must be deleted when no longer used.
-     * @return array of typed named object pointers.
-     */
-    inline T **sort(void) {
-        return static_cast<T**>(NamedObject::sort(index()));
-    }
-
-    /**
-     * Convenience typedef for iterative pointer.
-     */
-    typedef linked_pointer<T> iterator;
-};
-
 /**
  * Convenience typedef for root pointers of single linked lists.
  */
 typedef LinkedObject *LinkedIndex;
 
-/**
- * Convenience type for a stack of linked objects.
- */
-typedef ObjectStack objstack_t;
-
-/**
- * Convenience type for a fifo of linked objects.
- */
-typedef OrderedIndex objfifo_t;
-
-/**
- * Convenience type for a queue of linked objects.
- */
-typedef ObjectQueue objqueue_t;
+typedef DLinkedObject LinkedList;   // compatibility for older code
 
 } // namespace ucommon
 
