@@ -53,7 +53,7 @@ namespace ucommon {
  * automatic heap management when used in conjunction with smart pointers.
  * @author David Sugar <dyfet@gnutelephony.org>
  */
-class __EXPORT CountedObject : public ObjectProtocol
+class __EXPORT CountedObject : public __PROTOCOL ObjectProtocol
 {
 private:
     volatile unsigned count;
@@ -115,13 +115,13 @@ public:
     /**
      * Increase reference count when retained.
      */
-    void retain(void);
+    void retain(void) __OVERRIDE;
 
     /**
      * Decrease reference count when released.  If no longer retained, then
      * the object is dealloc'd.
      */
-    void release(void);
+    void release(void) __OVERRIDE;
 };
 
 /**
@@ -134,34 +134,42 @@ public:
  * This is actually a helper class for the typed pointer template.
  * @author David Sugar <dyfet@gnutelephony.org>
  */
-class __EXPORT auto_object
+class __EXPORT AutoObject
 {
 protected:
     ObjectProtocol *object;
 
-    auto_object();
+    AutoObject();
 
-public:
     /**
      * Construct an auto-pointer referencing an existing object.
      * @param object we point to.
      */
-    auto_object(ObjectProtocol *object);
+    AutoObject(ObjectProtocol *object);
 
     /**
      * Construct an auto-pointer as a copy of another pointer.  The
      * retention of the object being pointed to will be increased.
      * @param pointer we are a copy of.
      */
-    auto_object(const auto_object &pointer);
+    AutoObject(const AutoObject &pointer);
 
     /**
      * Delete auto pointer.  When it falls out of scope, the retention
      * of the object it references is reduced.  If it falls to zero in
      * a reference counted object, then the object is auto-deleted.
      */
-    ~auto_object();
+    ~AutoObject();
 
+    /**
+     * Set our pointer to a specific object.  If the pointer currently
+     * references another object, that object is released.  The pointer
+     * references our new object and that new object is retained.
+     * @param object to assign to.
+     */
+    void set(ObjectProtocol *object);
+
+public:
     /**
      * Manually release the pointer.  This reduces the retention level
      * of the object and resets the pointer to point to nobody.
@@ -180,27 +188,6 @@ public:
      */
     operator bool() const;
 
-    /**
-     * test if the object being referenced is the same as the object we specify.
-     * @param object we compare to.
-     * @return true if this is the object our pointer references.
-     */
-    bool operator==(ObjectProtocol *object) const;
-
-    /**
-     * test if the object being referenced is not the same as the object we specify.
-     * @param object we compare to.
-     * @return true if this is not the object our pointer references.
-     */
-    bool operator!=(ObjectProtocol *object) const;
-
-    /**
-     * Set our pointer to a specific object.  If the pointer currently
-     * references another object, that object is released.  The pointer
-     * references our new object and that new object is retained.
-     * @param object to assign to.
-     */
-    void operator=(ObjectProtocol *object);
 };
 
 /**
@@ -220,6 +207,8 @@ private:
     ObjectProtocol **vector;
     unsigned max;
 
+    __DELETE_DEFAULTS(SparseObjects);
+
 protected:
     /**
      * Object factory for creating members of the spare array when they
@@ -274,6 +263,9 @@ public:
 template <class T>
 class sarray : public SparseObjects
 {
+private:
+    __DELETE_DEFAULTS(sarray);
+
 public:
     /**
      * Generate a sparse typed array of specified size.
@@ -301,90 +293,25 @@ public:
         return reference_cast<T>(get(offset));
     }
 
-    inline const T* at(unsigned offset) const {
-        return immutable_cast<T*>(SparseObjects::get(offset));
+    inline T& at(unsigned offset) {
+        return reference_cast<T>(SparseObjects::get(offset));
+    }
+
+    inline const T* operator()(unsigned offset) const {
+        return get(offset);
+    }
+
+    inline void operator()(unsigned offset, T value) {
+        T& ref = at(offset);
+        ref = value;
     }
 
 private:
-    __LOCAL ObjectProtocol *create(void) {
+    __LOCAL ObjectProtocol *create(void) __FINAL {
         return new T;
     }
 };
 
-/**
- * Template for embedding a data structure into a reference counted object.
- * This is a convenient means to create reference counted heap managed data
- * structure.  This template can be used for embedding data into other kinds
- * of managed object classes in addition to reference counting.  For example,
- * it can be used to embed a data structure into a linked list, as shown in
- * the linked_value template.
- * @author David Sugar <dyfet@gnutelephony.org>
- */
-template <typename T, class O = CountedObject>
-class object_value : public O
-{
-protected:
-    /**
-     * Assign our value from a typed data object.  This is a helper method.
-     * @param object to assign our value from.
-     */
-    inline void set(const T& object) {
-        value = object;
-    }
-
-public:
-    T value;    /**< Embedded data value */
-
-    /**
-     * Construct composite value object.
-     */
-    inline object_value() : O(), value() {}
-
-    /**
-     * Construct composite value object and assign from existing data value.
-     * @param existing typed value to assign.
-     */
-    inline object_value(T& existing) : O() {
-        value = existing;
-    }
-
-    /**
-     * Pointer reference to embedded data value.
-     * @return embedded value.
-     */
-    inline T& operator*() {
-        return value;
-    }
-
-    /**
-     * Assign embedded data value.
-     * @param data value to assign.
-     */
-    inline void operator=(const T& data) {
-        value = data;
-    }
-
-    /**
-     * Retrieve data value by casting reference.
-     * @return embedded value.
-     */
-    inline operator T&() {
-        return value;
-    }
-
-    inline T& operator()() {
-        return value;
-    }
-
-    /**
-     * Set data value by expression reference.
-     * @param data value to assign.
-     */
-    inline void operator()(T& data) {
-        value = data;
-    }
-};
-
 /**
  * Typed smart pointer class.  This is used to manage references to
  * a specific typed object on the heap that is derived from the base Object
@@ -397,27 +324,29 @@ public:
  * they are contained in remains in scope as well.
  * @author David Sugar <dyfet@gnutelephony.org>
  */
-template <class T, class P = auto_object>
-class object_pointer : public P
+template <class T>
+class object_pointer : public AutoObject
 {
 public:
     /**
      * Create a pointer with no reference.
      */
-    inline object_pointer() : P() {}
+    inline object_pointer() : AutoObject() {}
 
     /**
      * Create a pointer with a reference to a heap object.
      * @param object we are referencing.
      */
-    inline object_pointer(T* object) : P(object) {}
+    inline object_pointer(T* object) : AutoObject(object) {}
+
+    inline object_pointer(const object_pointer& copy) : AutoObject(copy) {}
 
     /**
      * Reference object we are pointing to through pointer indirection.
      * @return pointer to object we are pointing to.
      */
     inline T* operator*() const {
-        return static_cast<T*>(P::object);
+        return protocol_cast<T*>(object);
     }
 
     /**
@@ -425,7 +354,7 @@ public:
      * @return object we are pointing to.
      */
     inline T& operator()() const {
-        return reference_cast<T>(P::object);
+        return reference_cast<T>(object);
     }
 
     /**
@@ -433,7 +362,7 @@ public:
      * @return reference to member of pointed object.
      */
     inline T* operator->() const {
-        return static_cast<T*>(P::object);
+        return protocol_cast<T*>(object);
     }
 
     /**
@@ -441,76 +370,38 @@ public:
      * @return pointer or NULL if we are not referencing an object.
      */
     inline T* get(void) const {
-        return static_cast<T*>(P::object);
-    }
-
-    /**
-     * Iterate our pointer if we reference an array on the heap.
-     * @return next object in array.
-     */
-    inline T* operator++() {
-        P::operator++(); 
-        return get();
-    }
-
-    /**
-     * Iterate our pointer if we reference an array on the heap.
-     * @return previous object in array.
-     */
-    inline void operator--() {
-        P::operator--(); 
-        return get();
+        return protocol_cast<T*>(object);
     }
 
     /**
      * Perform assignment operator to existing object.
      * @param typed object to assign.
      */
-    inline void operator=(T *typed) {
-        P::operator=(polypointer_cast<ObjectProtocol*>(typed));
+    inline object_pointer& operator=(T *typed) {
+        AutoObject::set(polypointer_cast<ObjectProtocol*>(typed));
+        return *this;
+    }
+
+    inline object_pointer& operator=(const object_pointer& from) {
+        AutoObject::set(polypointer_cast<ObjectProtocol*>(from.object));
+        return *this;
     }
 
     /**
      * See if pointer is set.
      */
     inline operator bool() const {
-        return P::object != NULL;
+        return object != NULL;
     }
 
     /**
      * See if pointer is not set.
      */
     inline bool operator!() const {
-        return P::object == NULL;
+        return object == NULL;
     }
 };
 
-/**
- * Convenience function to access object retention.
- * @param object we are retaining.
- */
-inline void retain(ObjectProtocol *object) 
-{
-    object->retain();
-}
-
-/**
- * Convenience function to access object release.
- * @param object we are releasing.
- */
-inline void release(ObjectProtocol *object) 
-{
-    object->release();
-}
-
-/**
- * Convenience function to access object copy.
- * @param object we are copying.
- */
-inline ObjectProtocol *copy(ObjectProtocol *object) {
-    return object->copy();
-}
-
 } // namespace ucommon
 
 #endif
