@@ -28,19 +28,38 @@
 
 #include <cstdlib>
 #include <cstddef>
+#if __cplusplus >= 201103L
+#include <memory>
+#endif
+
 #if defined(sun) && defined(unix)
 #include <malloc.h>
 #endif
 
 #ifndef _UCOMMON_PLATFORM_H_
 #define _UCOMMON_PLATFORM_H_
-#define UCOMMON_ABI 6
+#define UCOMMON_ABI 7
 
 #ifndef UCOMMON_SYSRUNTIME
 #ifndef NEW_STDCPP
 #define NEW_STDCPP
 #endif
 #define _UCOMMON_EXTENDED_
+#include <stdexcept>
+#define __THROW_SIZE(x)       throw std::length_error(x)
+#define __THROW_RANGE(x)      throw std::out_of_range(x)
+#define __THROW_RUNTIME(x)    throw std::runtime_error(x)
+#define __THROW_ALLOC()       throw std::bad_alloc()
+#define __THROW_DEREF(v)      if(v == nullptr) \
+                                throw std::runtime_error("Dereference NULL")
+#define __THROW_UNDEF(v,x)    if(v == nullptr) throw std::runtime_error(x)
+#else
+#define __THROW_RANGE(x)      abort()
+#define __THROW_SIZE(x)       abort()
+#define __THROW_RUNTIME(x)    abort()
+#define __THROW_ALLOC()       abort()
+#define __THROW_DEREF(v)      if(v == nullptr) abort()
+#define __THROW_UNDEF(v,x)    if(v == nullptr) abort()
 #endif
 
 /**
@@ -89,6 +108,20 @@
 #endif
 #endif
 
+#ifdef  UCOMMON_RTTI
+#define __PROTOCOL   virtual
+template<typename T, typename S>
+T protocol_cast(S *s) {
+    return dynamic_cast<T>(s);
+}
+#else
+#define __PROTOCOL
+template<typename T, typename S>
+T protocol_cast(S *s) {
+    return static_cast<T>(s);
+}
+#endif
+
 #if defined(__GNUC__) && (__GNUC < 3) && !defined(_GNU_SOURCE)
 #define _GNU_SOURCE
 #endif
@@ -105,6 +138,19 @@
 #define __PRINTF(x,y)   __attribute__ ((format (printf, x, y)))
 #define __SCANF(x, y) __attribute__ ((format (scanf, x, y)))
 #define __MALLOC      __attribute__ ((malloc))
+#define __NORETURN    __attribute__ ((__noreturn__))
+#endif
+
+#define __UNUSED(x)         (void)x
+
+#if __cplusplus >= 201103L
+#define     __ALIGNED(x)    alignas(x)
+#else
+#ifdef      _MSC_VER
+#define     __ALIGNED(x)    __declspec(align(x))
+#else
+#define     __ALIGNED(x)    __attribute__(align(x))
+#endif
 #endif
 
 #if __cplusplus < 201103L
@@ -128,7 +174,7 @@
 #if __cplusplus <= 199711L && !defined(_MSC_VER)
 #if defined(__GNUC_MINOR__) && !defined(__clang__)
 #define nullptr __null
-#else
+#elif !defined(__clang__) || (defined(__clang__) && defined(__linux__))
 const class nullptr_t 
 {
 public:
@@ -268,6 +314,7 @@ typedef size_t stacksize_t;
 #else   
 #define _MSTHREADS_
 typedef DWORD pthread_t;
+typedef DWORD pthread_key_t;
 typedef unsigned stacksize_t;
 typedef CRITICAL_SECTION pthread_mutex_t;
 #endif
@@ -287,6 +334,7 @@ inline void sleep(int seconds)
 
 extern "C" {
 
+    #define __SERVICE(id, argc, argv) void WINAPI service_##id(DWORD argc, LPSTR *argv)
     #define SERVICE_MAIN(id, argc, argv) void WINAPI service_##id(DWORD argc, LPSTR *argv)
 
     typedef LPSERVICE_MAIN_FUNCTION cpr_service_t;
@@ -310,15 +358,6 @@ extern "C" {
     inline void pthread_mutex_unlock(pthread_mutex_t *mutex)
         {LeaveCriticalSection(mutex);}
 #endif
-
-    inline char *strdup(const char *s)
-        {return _strdup(s);}
-
-    inline int stricmp(const char *s1, const char *s2)
-        {return _stricmp(s1, s2);}
-
-    inline int strnicmp(const char *s1, const char *s2, size_t l)
-        {return _strnicmp(s1, s2, l);}
 }
 
 #elif defined(__PTH__)
@@ -353,13 +392,13 @@ inline void pthread_mutex_destroy(pthread_mutex_t *mutex)
     {};
 
 inline void pthread_mutex_lock(pthread_mutex_t *mutex)
-    {pth_mutex_acquire(mutex, 0, NULL);};
+    {pth_mutex_acquire(mutex, 0, nullptr);};
 
 inline void pthread_mutex_unlock(pthread_mutex_t *mutex)
     {pth_mutex_release(mutex);};
 
 inline void pthread_cond_wait(pthread_cond_t *cond, pthread_mutex_t *mutex)
-    {pth_cond_await(cond, mutex, NULL);};
+    {pth_cond_await(cond, mutex, nullptr);};
 
 inline void pthread_cond_signal(pthread_cond_t *cond)
     {pth_cond_notify(cond, FALSE);};
@@ -483,12 +522,14 @@ typedef unsigned long timeout_t;
 #endif
 #endif
 
-#ifndef PROGRAM_MAIN
+#ifndef __PROGRAM
+#define __PROGRAM(c,v)              extern "C" int main(int c, char **v)
 #define PROGRAM_MAIN(argc, argv)    extern "C" int main(int argc, char **argv)
 #define PROGRAM_EXIT(code)          return code
 #endif
 
-#ifndef SERVICE_MAIN
+#ifndef __SERVICE
+#define __SERVICE(id, c, v)         void service_##id(int c, char **v)
 #define SERVICE_MAIN(id, argc, argv)    void service_##id(int argc, char **argv)
 typedef void (*cpr_service_t)(int argc, char **argv);
 #endif
@@ -508,7 +549,7 @@ typedef void (*cpr_service_t)(int argc, char **argv);
  */
 template<class T>
 inline T *init(T *memory)
-    {return ((memory) ? new(((void *)memory)) T : NULL);}
+    {return ((memory) ? new(((void *)memory)) T : nullptr);}
 
 typedef long Integer;
 typedef unsigned long Unsigned;
@@ -526,9 +567,14 @@ template<class T, class S>
 inline T polypointer_cast(S *s)
 {
 #if defined(DEBUG) && defined(UCOMMON_RTTI)
-    assert(dynamic_cast<T>(s) != NULL);   // rtti for debug only...
-#endif
+    if(s == nullptr)
+        return nullptr;
+    T ptr = dynamic_cast<T>(s);
+    __THROW_DEREF(ptr);
+    return ptr;
+#else
     return static_cast<T>(s);
+#endif
 }   
 
 template<class T, class S>
@@ -556,17 +602,13 @@ inline T polydynamic_cast(S *s)
 template<class T, class S>
 inline T& polyreference_cast(S *s)
 {
-#if defined(DEBUG) && defined(UCOMMON_RTTI)
-    assert(dynamic_cast<T*>(s) != NULL);   // rtti for debug only...
-#endif
+    __THROW_DEREF(s);
     return *(static_cast<T*>(s));
 }    
 
 template<typename T>
 inline T& reference_cast(T *pointer) {
-#ifdef  DEBUG
-    assert(pointer != NULL);
-#endif
+    __THROW_DEREF(pointer);
     return *pointer;
 }
 
