@@ -55,170 +55,77 @@
 
 namespace ucommon {
 
-namespace Type {
-
-    class SecChars 
-    {
-    public:
-        typedef enum {GENERIC_STRING, B64_STRING, HEX_STRING, MD5_DIGEST, SHA_DIGEST, PEM_PUBLIC, PEM_PRIVATE} strtype_t;  
-    };
-
-    class KeyBytes
-    {
-    public:
-        typedef enum {UNDEFINED_KEYTYPE, IV_BUFFER, UNPAIRED_KEYTYPE, RSA_KEYTYPE, KEY_DIGEST} keytype_t;
-    };
-
-}
-
-template <>
-class __SHARED typeref<Type::SecChars> : protected TypeRef, public Type::SecChars
+class __SHARED AutoClear
 {
+private:
+    __DELETE_DEFAULTS(AutoClear);
+
+protected:
+    size_t size;
+    void *pointer;
+
+    AutoClear(size_t alloc);
+
 public:
-    class storage : public Counted
-    {
-    private:
-        friend class typeref;
+    virtual ~AutoClear();
+};    
 
-        strtype_t type;
-        char mem[1];
+template<typename T>
+class autoclear : public AutoClear
+{
+private:
+    __DELETE_COPY(autoclear);
 
-        __DELETE_COPY(storage);
+public:
+    autoclear() : AutoClear(sizeof(T)) {};
 
-        storage(caddr_t addr, size_t size, const char *str, strtype_t strtype = GENERIC_STRING);
-
-        virtual void dealloc() __FINAL;
-
-        inline const char *get() {
-            return &mem[0];
-        }
-
-        inline size_t len() {
-            return strlen(mem);
-        }
-
-        inline size_t max() {
-            return size;
-        }
-    };
-
-    typeref();
-    
-    typeref(const typeref& copy);
-
-    typeref(const char *str, strtype_t strtype = GENERIC_STRING);
-
-    const char *operator*() const;
-
-    inline operator const char *() const {
-        return operator*();
+    inline operator T() {
+        return *(static_cast<T*>(pointer));
     }
 
-    bool operator==(const typeref& ptr) const;
-
-    bool operator==(const char *obj) const;
-
-    inline bool operator!=(const typeref& ptr) const {
-        return !(*this == ptr);
+    inline T& operator*() {
+        return *(static_cast<T*>(pointer));
     }
 
-    inline bool operator!=(const char *obj) const {
-        return !(*this == obj);
+    inline T* operator->() {
+        return static_cast<T*>(pointer);
     }
-
-    typeref& operator=(const typeref& objref);
-
-    typeref& operator=(const char *str);
-
-    void set(const char *str, strtype_t strtype = GENERIC_STRING);
-
-    void b64(const uint8_t *bytes, size_t bsize);
-
-    void hex(const uint8_t *bytes, size_t bsize);
-
-    strtype_t type(void) const;
-
-    size_t size(void) const;
-
-    size_t len(void) const;
 };
 
 template <>
-class __SHARED typeref<Type::KeyBytes> : protected TypeRef, public Type::KeyBytes
+class autoclear<char *> : public AutoClear
 {
+private:
+    __DELETE_COPY(autoclear);
+
 public:
-    class storage : public Counted
-    {
-    private:
-        friend class typeref;
+    autoclear(size_t len) : AutoClear(len) {};
 
-        __DELETE_COPY(storage);
-
-        keytype_t type;
-        uint8_t mem[1];
-
-        storage(caddr_t addr, size_t size, const uint8_t *key = NULL, keytype_t keytype = UNPAIRED_KEYTYPE);
-
-        virtual void dealloc() __FINAL;
-
-        inline const uint8_t *get() {
-            return &mem[0];
-        }
-    };
-
-    typeref();
-    
-    typeref(const typeref& copy);
-
-    typeref(size_t keysize, keytype_t keytype = UNPAIRED_KEYTYPE);
-
-    typeref(const uint8_t *key, size_t keysize, keytype_t keytype = UNPAIRED_KEYTYPE);
-
-    const uint8_t *operator*() const;
-
-    inline operator const uint8_t *() const {
-        return operator*();
+    inline char *operator*() {
+        return (char *)pointer;
     }
-
-    bool operator==(const typeref& ptr) const;
-
-    inline bool operator!=(const typeref& ptr) const {
-        return !(*this == ptr);
-    }
-
-    typeref& operator=(const typeref& objref);
-
-    void set(const uint8_t *str, size_t keysize, keytype_t keytype = UNPAIRED_KEYTYPE);
-
-    void generate(size_t keysize, keytype_t keytype = UNPAIRED_KEYTYPE);
-
-    keytype_t type(void) const;
-
-    size_t size(void) const;
-
-    size_t bytes(void) const;
 };
 
-template<>
-inline size_t mapkeypath<Type::SecChars>(typeref<Type::SecChars>& object)
+template <>
+class autoclear<uint8_t *> : public AutoClear
 {
-	size_t path = 1;
-	return MapRef::index(path, (const uint8_t *)(*object), object.len());
-}
+private:
+    __DELETE_COPY(autoclear);
 
-template<>
-inline size_t mapkeypath<Type::KeyBytes>(typeref<Type::KeyBytes>& object)
-{
-	size_t path = object.size();
-	return MapRef::index(path, *object, object.size() / 8);
-}
+public:
+    autoclear(size_t len) : AutoClear(len) {};
+
+    inline char *operator*() {
+        return (char *)pointer;
+    }
+};
 
 /**
  * Common secure socket support.  This offers common routines needed for
  * secure/ssl socket support code.
  * @author David Sugar <dyfet@gnutelephony.org>
  */
-class __SHARED secure : public Type::SecChars, public Type::KeyBytes
+class __SHARED secure 
 {
 public:
     /**
@@ -226,17 +133,14 @@ public:
      */
     typedef enum {OK=0, INVALID, MISSING_CERTIFICATE, MISSING_PRIVATEKEY, INVALID_CERTIFICATE, INVALID_AUTHORITY, INVALID_PEERNAME, INVALID_CIPHER} error_t;
 
-    typedef typeref<Type::SecChars> string;
+    typedef enum {NONE, SIGNED, VERIFIED} verify_t;
 
-    typedef arrayref<Type::SecChars> strarray;
+    typedef stringref<secure_release> string;
 
-    typedef queueref<Type::SecChars> strqueue;
+    typedef byteref<secure_release> keybytes;
 
-    typedef typeref<Type::KeyBytes> keybytes;
-    
-    typedef typeref<Type::KeyBytes> keyarray;
-
-    typedef typeref<Type::KeyBytes> keyqueue;
+private:
+    __DELETE_COPY(secure);
 
 protected:
     /**
@@ -265,6 +169,11 @@ public:
      */
     typedef void *session_t;
 
+    /**
+     * Convenience type to represent a ssl certificate object.
+     */
+    typedef void *cert_t;
+
     /**
      * Convenience type to represent a secure socket buf i/o stream.
      */
@@ -298,17 +207,6 @@ public:
      */
     static const char *oscerts(void);
 
-    /**
-     * Verify a certificate chain through your certificate authority.
-     * This uses the ca loaded as an optional argument for client and
-     * server.  Optionally the hostname of the connection can also be
-     * verified by pulling the peer certificate.
-     * @param session that is connected.
-     * @param peername that we expect.
-     * @return secure error level or secure::OK if none.
-     */
-    static error_t verify(session_t session, const char *peername = NULL);
-
     /**
      * Create a sever context.  The certificate file used will be based on
      * the init() method name.  This may often be /etc/ssl/certs/initname.pem.
@@ -324,9 +222,10 @@ public:
      * Create an anonymous client context with an optional authority to
      * validate.
      * @param authority path to use or NULL if none.
+     * @param paths of certificates to use.
      * @return a basic client security context.
      */
-    static client_t client(const char *authority = NULL);
+    static client_t client(const char *authority = NULL, const char *paths = NULL);
 
     /**
      * Create a peer user client context.  This assumes a user certificate
@@ -347,15 +246,17 @@ public:
      * Determine if the current security context is valid.
      * @return true if valid, -1 if not.
      */
-    inline bool is_valid(void) const
-        {return error == OK;};
+    inline bool is_valid(void) const {
+        return error == OK;
+    };
 
     /**
      * Get last error code associated with the security context.
      * @return last error code or 0/OK if none.
      */
-    inline error_t err(void) const
-        {return error;};
+    inline error_t err(void) const {
+        return error;
+    };
 
     /**
      * Create 36 character traditional version 1 uuid.
@@ -363,66 +264,17 @@ public:
      */
     static void uuid(char *string);
 
+    static secure::string pass(const char *prompt, size_t size);
+
     static secure::string uuid(void);
 
-    template <typename T>
-    inline static void erase(T *object)
-        {memset(object, 0, sizeof(T)); delete object;}
+    inline operator bool() const {
+        return is_valid();
+    }
 
-    inline operator bool() const
-        {return is_valid();}
-
-    inline bool operator!() const
-        {return !is_valid();}
-
-};
-
-/**
- * Secure socket buffer.  This is used to create ssl socket connections
- * for both clients and servers.  The use depends in part on the type of
- * context created and passed at construction time.  If no context is
- * passed (NULL), then this reverts to TCPBuffer behavior.
- * @author David Sugar <dyfet@gnutelephony.org>
- */
-class __SHARED SSLBuffer : public TCPBuffer
-{
-private:
-    __DELETE_COPY(SSLBuffer);
-
-protected:
-    secure::session_t ssl;
-    secure::bufio_t bio;
-    bool server;
-    bool verify;
-
-public:
-    SSLBuffer(secure::client_t context);
-    SSLBuffer(const TCPServer *server, secure::server_t context, size_t size = 536);
-    ~SSLBuffer();
-
-    /**
-     * Connect a ssl client session to a specific host uri.  If the socket
-     * was already connected, it is automatically closed first.
-     * @param host we are connecting to.
-     * @param service to connect to.
-     * @param size of buffer and tcp fragments.
-     */
-    void open(const char *host, const char *service, size_t size = 536);
-
-    void close(void);
-
-    void release(void);
-
-    size_t _push(const char *address, size_t size);
-
-    size_t _pull(char *address, size_t size);
-
-    bool _flush(void);
-
-    bool _pending(void);
-
-    inline bool is_secure(void) const
-        {return bio != NULL;}
+    inline bool operator!() const {
+        return !is_valid();
+    }
 };
 
 /**
@@ -464,7 +316,7 @@ public:
         int modeid;
 
         // assume 512 bit cipher keys possible...
-        unsigned char keybuf[MAX_CIPHER_KEYSIZE / 8], ivbuf[MAX_CIPHER_KEYSIZE / 8];
+        uint8_t keybuf[MAX_CIPHER_KEYSIZE / 8], ivbuf[MAX_CIPHER_KEYSIZE / 8];
 
         // generated keysize
         size_t keysize, blksize;
@@ -476,7 +328,7 @@ public:
     public:
         Key();
 
-        Key(const char *cipher, const char *digest, const char *text, size_t size = 0, const unsigned char *salt = NULL, unsigned rounds = 1);
+        Key(const char *cipher, const char *digest, const char *text, size_t size = 0, const uint8_t *salt = NULL, unsigned rounds = 1);
 
         Key(const char *cipher, const uint8_t *iv, size_t ivsize);
 
@@ -486,25 +338,25 @@ public:
 
         ~Key();
 
+        void set(const uint8_t *key, size_t size);
+
         inline secure::keybytes key() {
             return secure::keybytes(keybuf, keysize);
         }
 
         inline secure::keybytes iv() {
-            return secure::keybytes(ivbuf, blksize, secure::IV_BUFFER);
+            return secure::keybytes(ivbuf, blksize);
         }
 
-        void set(const unsigned char *key, size_t size);
-
         bool set(const secure::keybytes& key);
 
         void set(const char *cipher, const char *digest);
 
         void set(const char *cipher, const uint8_t *iv, size_t ivsize);
 
-        bool set(const char *cipher, const secure::keybytes& iv);
+        void assign(const char *key, size_t size, const uint8_t *salt, unsigned rounds);
 
-        void assign(const char *key, size_t size, const unsigned char *salt, unsigned rounds);
+        bool set(const char *cipher, const secure::keybytes& iv);
 
         void assign(const char *key, size_t size = 0);
 
@@ -516,22 +368,34 @@ public:
 
         size_t get(uint8_t *key, uint8_t *ivout = NULL);
 
-        inline size_t size(void) const
-            {return keysize;}
+        inline size_t size(void) const {
+            return keysize;
+        }
 
-        inline size_t iosize(void) const
-            {return blksize;}
+        inline size_t iosize(void) const {
+            return blksize;
+        }
 
-        inline operator bool() const
-            {return keysize > 0;}
+        inline operator bool() const {
+            return keysize > 0;
+        }
 
-        inline bool operator!() const
-            {return keysize == 0;}
+        inline bool operator!() const {
+            return keysize == 0;
+        }
 
-        inline Key& operator=(const char *pass)
-            {assign(pass); return *this;}
+        inline Key& operator=(const char *pass) {
+            assign(pass); 
+            return *this;
+        }
 
-        static void options(const unsigned char *salt = NULL, unsigned rounds = 1);
+        bool operator==(const Key& other) const;
+
+        inline bool operator!=(const Key& other) const {
+            return !operator==(other);
+        }
+
+        static void options(const uint8_t *salt = NULL, unsigned rounds = 1);
     };
 
     typedef Key *key_t;
@@ -540,24 +404,26 @@ private:
     Key keys;
     size_t bufsize, bufpos;
     mode_t bufmode;
-    unsigned char *bufaddr;
+    uint8_t *bufaddr;
     void *context;
 
+    __DELETE_COPY(Cipher);
+
 protected:
-    virtual void push(unsigned char *address, size_t size);
+    virtual void push(uint8_t *address, size_t size);
 
     void release(void);
 
 public:
     Cipher();
 
-    Cipher(key_t key, mode_t mode, unsigned char *address = NULL, size_t size = 0);
+    Cipher(const key_t key, mode_t mode, uint8_t *address = NULL, size_t size = 0);
 
     virtual ~Cipher();
 
-    void set(unsigned char *address, size_t size = 0);
+    void set(uint8_t *address, size_t size = 0);
 
-    void set(key_t key, mode_t mode, unsigned char *address, size_t size = 0);
+    void set(const key_t key, mode_t mode, uint8_t *address, size_t size = 0);
 
     inline secure::keybytes iv() {
         return keys.iv();
@@ -581,7 +447,7 @@ public:
      * @param size of data to process.
      * @return size of processed output, should be same as size or 0 if error.
      */
-    size_t put(const unsigned char *data, size_t size);
+    size_t put(const uint8_t *data, size_t size);
 
     /**
      * This essentially encrypts a single string and pads with NULL bytes
@@ -602,7 +468,7 @@ public:
      * @param size of data to add before final pad.
      * @return actual bytes encrypted or decrypted.
      */
-    size_t pad(const unsigned char *address, size_t size);
+    size_t pad(const uint8_t *address, size_t size);
 
     /**
      * Process encrypted data in-place.  This assumes no need to set the
@@ -612,16 +478,19 @@ public:
      * @param flag if to pad data.
      * @return bytes processed and written back to buffer.
      */
-    size_t process(unsigned char *address, size_t size, bool flag = false);
+    size_t process(uint8_t *address, size_t size, bool flag = false);
 
-    inline size_t size(void) const
-        {return bufsize;}
+    inline size_t size(void) const {
+        return bufsize;
+    }
 
-    inline size_t pos(void) const
-        {return bufpos;}
+    inline size_t pos(void) const {
+        return bufpos;
+    }
 
-    inline size_t align(void) const
-        {return keys.iosize();}
+    inline size_t align(void) const {
+        return keys.iosize();
+    }
 
     /**
      * Check if a specific cipher is supported.
@@ -648,12 +517,16 @@ private:
     };
 
     unsigned bufsize;
-    unsigned char buffer[MAX_DIGEST_HASHSIZE / 8];
+    uint8_t buffer[MAX_DIGEST_HASHSIZE / 8];
     char textbuf[MAX_DIGEST_HASHSIZE / 8 + 1];
 
+    __DELETE_COPY(Digest);
+
 protected:
     void release(void);
 
+    const uint8_t *get(void);
+
 public:
     Digest(const char *type);
 
@@ -661,55 +534,74 @@ public:
 
     ~Digest();
 
-    inline bool puts(const char *str)
-        {return put(str, strlen(str));}
+    inline bool puts(const char *str) {
+        return put(str, strlen(str));
+    }
 
-    inline Digest &operator<<(const char *str)
-        {puts(str); return *this;}
+    inline Digest &operator<<(const char *str) {
+        puts(str); 
+        return *this;
+    }
 
-    inline Digest &operator<<(int16_t value)
-        {int16_t v = htons(value); put(&v, 2); return *this;}
+    inline Digest &operator<<(int16_t value) {
+        int16_t v = htons(value); 
+        put(&v, 2); 
+        return *this;
+    }
 
-    inline Digest &operator<<(int32_t value)
-        {int32_t v = htonl(value); put(&v, 4); return *this;}
+    inline Digest &operator<<(int32_t value) {
+        int32_t v = htonl(value); 
+        put(&v, 4); 
+        return *this;
+    }
 
-    inline Digest &operator<<(const PrintProtocol& p)
-        {const char *cp = p._print(); if(cp) puts(cp); return *this;}
+    inline Digest &operator<<(const PrintProtocol& p) {
+        const char *cp = p._print(); 
+        if(cp) 
+            puts(cp); 
+        return *this;
+    }
 
     bool put(const void *memory, size_t size);
 
-    inline unsigned size() const
-        {return bufsize;}
+    inline unsigned size() const {
+        return bufsize;
+    }
 
-    const unsigned char *get(void);
+    secure::keybytes key(void);
 
-    const char *c_str(void);
+    secure::string str(void);
 
-    inline secure::string str(void)
-        {return secure::string(c_str());}
-
-    inline operator secure::string()
-        {return secure::string(c_str());}
+    inline operator secure::string() {
+        return str();
+    }
 
     void set(const char *id);
 
-    inline void operator=(const char *id)
-        {set(id);};
+    inline Digest& operator=(const char *id) {
+        set(id);
+        return *this;
+    };
 
-    inline bool operator *=(const char *text)
-        {return puts(text);}
+    inline bool operator *=(const char *text) {
+        return puts(text);
+    }
 
-    inline bool operator +=(const char *text)
-        {return puts(text);}
+    inline bool operator +=(const char *text) {
+        return puts(text);
+    }
 
-    inline const char *operator*()
-        {return c_str();}
+    inline secure::string operator*() {
+        return str();
+    }
 
-    inline bool operator!() const
-        {return !bufsize && context == NULL;}
+    inline bool operator!() const {
+        return !bufsize && context == NULL;
+    }
 
-    inline operator bool() const
-        {return bufsize > 0 || context != NULL;}
+    inline operator bool() const {
+        return bufsize > 0 || context != NULL;
+    }
 
     /**
      * Finalize and recycle current digest to start a new
@@ -730,9 +622,7 @@ public:
      */
     static bool has(const char *name);
 
-    static void uuid(char *string, const char *name, const unsigned char *ns = NULL);
-
-    static secure::string uuid(const char *name, const unsigned char *ns = NULL);
+    static secure::string uuid(const char *name, const uint8_t *ns = NULL);
 
     /**
      * Shortcut for short md5 digests if supported...
@@ -774,69 +664,86 @@ private:
     };
 
     unsigned bufsize;
-    unsigned char buffer[MAX_DIGEST_HASHSIZE / 8];
+    uint8_t buffer[MAX_DIGEST_HASHSIZE / 8];
     char textbuf[MAX_DIGEST_HASHSIZE / 8 + 1];
 
+    __DELETE_COPY(HMAC);
+
 protected:
     void release(void);
 
-public:
-    HMAC(const char *digest, const char *key, size_t keylen = 0);
+    const uint8_t *get(void);
 
+public:
     HMAC(const char *digest, const secure::keybytes& key);
 
     HMAC();
 
     ~HMAC();
 
-    inline bool puts(const char *str)
-        {return put(str, strlen(str));}
+    inline bool puts(const char *str) {
+        return put(str, strlen(str));
+    }
 
-    inline HMAC &operator<<(const char *str)
-        {puts(str); return *this;}
+    inline HMAC &operator<<(const char *str) {
+        puts(str); 
+        return *this;
+    }
 
-    inline HMAC &operator<<(int16_t value)
-        {int16_t v = htons(value); put(&v, 2); return *this;}
+    inline HMAC &operator<<(int16_t value) {
+        int16_t v = htons(value); 
+        put(&v, 2); 
+        return *this;
+    }
 
-    inline HMAC &operator<<(int32_t value)
-        {int32_t v = htonl(value); put(&v, 4); return *this;}
+    inline HMAC &operator<<(int32_t value) {
+        int32_t v = htonl(value); 
+        put(&v, 4); 
+        return *this;
+    }
 
-    inline HMAC &operator<<(const PrintProtocol& p)
-        {const char *cp = p._print(); if(cp) puts(cp); return *this;}
+    inline HMAC &operator<<(const PrintProtocol& p) {
+        const char *cp = p._print(); 
+        if(cp) 
+            puts(cp); 
+        return *this;
+    }
 
     bool put(const void *memory, size_t size);
 
-    inline unsigned size() const
-        {return bufsize;}
+    inline unsigned size() const {
+        return bufsize;
+    }
 
-    const unsigned char *get(void);
+    secure::string str(void);
 
-    const char *c_str(void);
+    secure::keybytes key(void);
 
-    inline secure::string str(void)
-        {return secure::string(c_str());}
+    inline operator secure::string() {
+        return str();
+    }
 
-    inline operator secure::string()
-        {return secure::string(c_str());}
-
-    void set(const char *digest, const char *key, size_t len);
+    inline bool operator *=(const char *text) {
+        return puts(text);
+    }
 
     void set(const char *digest, const secure::keybytes& key);
 
-    inline bool operator *=(const char *text)
-        {return puts(text);}
+    inline bool operator +=(const char *text) {
+        return puts(text);
+    }
 
-    inline bool operator +=(const char *text)
-        {return puts(text);}
+    inline secure::string operator*() {
+        return str();
+    }
 
-    inline const char *operator*()
-        {return c_str();}
+    inline bool operator!() const {
+        return !bufsize && context == NULL;
+    }
 
-    inline bool operator!() const
-        {return !bufsize && context == NULL;}
-
-    inline operator bool() const
-        {return bufsize > 0 || context != NULL;}
+    inline operator bool() const {
+        return bufsize > 0 || context != NULL;
+    }
 
     /**
      * Test to see if a specific digest type is supported.
@@ -850,21 +757,6 @@ public:
     static secure::keybytes sha384(secure::keybytes key, const uint8_t *mem, size_t soze);
 };
 
-class __SHARED RSA
-{
-private:
-    void *keypair;
-
-    __DELETE_COPY(RSA);
-
-public:
-    RSA(secure::string privkey, secure::string pubkey); // pem
-    RSA(size_t keysize);
-    ~RSA();
-
-    secure::string pem(secure::strtype_t type);
-};
-
 /**
  * Cryptographically relevant random numbers.  This is used both to gather
  * entropy pools and pseudo-random values.
@@ -882,7 +774,7 @@ public:
      * @param size of buffer.
      * @return true if successful.
      */
-    static bool seed(const unsigned char *buffer, size_t size);
+    static bool seed(const uint8_t *buffer, size_t size);
 
     /**
      * Re-seed pseudo-random generation and entropy pools.
@@ -897,7 +789,7 @@ public:
      * @param size of buffer.
      * @return number of bytes filled.
      */
-    static size_t key(unsigned char *memory, size_t size);
+    static size_t key(uint8_t *memory, size_t size);
 
     /**
      * Fill memory with pseudo-random values.  This is used
@@ -907,7 +799,7 @@ public:
      * @param size of buffer to fill.
      * @return number of bytes set.
      */
-    static size_t fill(unsigned char *memory, size_t size);
+    static size_t fill(uint8_t *memory, size_t size);
 
     /**
      * Get a pseudo-random integer, range 0 - 32767.
@@ -955,7 +847,7 @@ public:
     template <class T>
     inline static T value(void) {
         T tmp;
-        Random::key(reinterpret_cast<unsigned char *>(&tmp), sizeof(tmp));
+        Random::key(reinterpret_cast<uint8_t *>(&tmp), sizeof(tmp));
         return tmp;
     }
 
@@ -979,10 +871,6 @@ public:
     }    
 };
 
-/**
- * Convenience type for secure socket.
- */
-typedef SSLBuffer ssl_t;
 
 /**
  * Convenience type for generic digests.
@@ -1012,11 +900,9 @@ inline void zerofill(void *addr, size_t size)
 #ifndef UCOMMON_SYSRUNTIME
 
 /**
- * Secure socket using std::iostream.  This class is similar to SSLBuffer
- * but uses the libstdc++ library to stream i/o.  Being based on tcpstream,
- * it also inherits the character protocol.  Like SSLBuffer, if no context
- * is given or the handshake fails, then the stream defaults to insecure TCP
- * connection behavior.
+ * Secure socket using std::iostream.  Being based on tcpstream, it also
+ * inherits the character protocol.  If no context is given or the handshake 
+ * fails, then the stream defaults to insecure TCP connection behavior.
  * @author David Sugar <dyfet@gnutelephony.org>
  */
 class __SHARED sstream : public tcpstream
@@ -1027,99 +913,114 @@ private:
 protected:
     secure::session_t ssl;
     secure::bufio_t bio;
+    secure::cert_t cert;
+    secure::verify_t verified;
     bool server;
-    bool verify;
+
+    ssize_t _write(const char *address, size_t size) __OVERRIDE;
+
+    ssize_t _read(char *address, size_t size) __OVERRIDE;
+
+    bool _wait(void) __OVERRIDE;
 
 public:
+    /**
+     * Construct a ssl client stream.  The context will be loaded with
+     * relevant certificates from secure::client().
+     * @param context to use
+     */
     sstream(secure::client_t context);
+
+    /**
+     * Construct a ssl server stream.  The context will be loaded with
+     * relevant certificates from secure::server().
+     * @param server instance of tcp socket.
+     * @param context to use.
+     * @param size of streaming buffer.
+     */
     sstream(const TCPServer *server, secure::server_t context, size_t size = 536);
+    
+    /**
+     * Destroy ssl stream.  Clean up any resources used.
+     */
     ~sstream();
 
+    /**
+     * Open a connection to a ssl server.
+     * @param host name to connect with.
+     * @param service id to connect to.
+     * @param size of stream buffer to use.
+     */
     void open(const char *host, const char *service, size_t size = 536);
 
+    /**
+     * Close a connection with a ssl server.
+     */
     void close(void);
 
-    int sync();
-
+    /**
+     * Release all ssl resources.
+     */
     void release(void);
 
-    ssize_t _write(const char *address, size_t size);
+    int sync() __OVERRIDE;
 
-    ssize_t _read(char *address, size_t size);
-
-    bool _wait(void);
-
-    inline void flush(void)
-        {sync();}
-
-    inline bool is_secure(void) const
-        {return bio != NULL;}
-};
-
-typedef secure::string keystring_t;
-
-/**
- * A template to create a random generated key of specified size.  The
- * key memory is cleared when the object is destroyed.
- * @author David Sugar <dyfet@gnutelephony.org>
- */
-template<size_t S>
-class keyrandom
-{
-private:
-    unsigned char buffer[S];
-
-    __DELETE_COPY(keyrandom);
-
-public:
-    /**
-     * Create a new character buffer with an empty string.
-     */
-    inline keyrandom()
-        {Random::key(buffer, S);}
+    inline void flush(void) {
+        sync();
+    }
 
     /**
-     * Clear memory when destroyed.
+     * Get peer (x509) certificate for current stream if present.
+     * @return certificate of peer or nullptr if none.
      */
-    inline ~keyrandom()
-        {memset(buffer, 0, S);}
+    inline secure::cert_t certificate(void) const {
+        return cert;
+    }
 
     /**
-     * Update with new random key.
+     * Check if ssl session active, otherwise pure tcp.
+     * @return true if ssl session.
      */
-    inline void update(void)
-        {Random::key(buffer, S);}
+    inline bool is_secure(void) const {
+        return bio != NULL;
+    }
 
     /**
-     * Clear current key memory.
+     * Check if a peer certificate is present.
+     * @return true if peer certificate.
      */
-    inline void clear(void)
-        {memset(buffer, 0, S);}
+    inline bool is_certificate(void) const {
+        return cert != NULL;
+    }
 
     /**
-     * Get text by casting reference.
-     * @return pointer to text in object.
+     * Check if peer certificate is verified through an authority.
+     * @return true if verified peer.
      */
-    inline operator unsigned char *()
-        {return buffer;}
+    inline bool is_verified(void) const {
+        return verified == secure::VERIFIED;
+    }
 
     /**
-     * Get text by object pointer reference.
-     * @return pointer to text in object.
+     * Check if peer certificate is present and at least self-signed.
+     * @return true if signed or verified peer.
      */
-    inline unsigned char *operator*()
-        {return buffer;}
-
-    /**
-     * Get allocated size of the object.
-     * @return allocated size.
-     */
-    inline size_t size(void) const
-        {return S;}
+    inline bool is_signed(void) const {
+        return verified != secure::NONE;
+    }
 };
 
 #endif
 
+// can be specialized...
+template<typename T>
+void clearmem(T &var)
+{
+    memset(&var, 0, sizeof(var));
+}
+
+typedef secure::string keystring_t;
+
 } // namespace ucommon
 
 #endif
