@@ -65,11 +65,6 @@
 
 namespace ucommon {
 
-/**
- * A convenience class for size of strings.
- */
-typedef unsigned short strsize_t;
-
 /**
  * A copy-on-write string class that operates by reference count.  This string
  * class anchors a counted object that is managed as a copy-on-write
@@ -80,7 +75,7 @@ typedef unsigned short strsize_t;
  * manipulate ordinary null terminated character arrays directly in memory.
  * @author David Sugar <dyfet@gnutelephony.org>
  */
-class __EXPORT String : public ObjectProtocol
+class __EXPORT String : public __PROTOCOL ObjectProtocol
 {
 protected:
     /**
@@ -117,8 +112,9 @@ public:
         size_t offset(unsigned member);
         size_t size(unsigned member);
 
-        inline size_t members(void) const
-            {return count;}
+        inline size_t members(void) const {
+            return count;
+        }
 
         bool match(const char *text, unsigned flags = 0);
 
@@ -126,11 +122,13 @@ public:
 
         bool operator*=(const char *string);
 
-        operator bool() const
-            {return object != NULL;}
+        operator bool() const {
+            return object != NULL;
+        }
 
-        bool operator!() const
-            {return object == NULL;}
+        bool operator!() const {
+            return object == NULL;
+        }
     };
 
     class __EXPORT cstring : public CountedObject
@@ -143,9 +141,8 @@ public:
 
     public:
 #pragma pack(1)
-        strsize_t max;  /**< Allocated size of cstring text */
-        strsize_t len;  /**< Current length of cstring text */
-        char fill;      /**< Filler character or 0 for none */
+        size_t max;  /**< Allocated size of cstring text */
+        size_t len;  /**< Current length of cstring text */
         char text[1];   /**< Null terminated text, in overdraft space */
 #pragma pack()
 
@@ -154,25 +151,13 @@ public:
          * new operator would also need the size as an overdraft value.
          * @param size of string.
          */
-        cstring(strsize_t size);
+        cstring(size_t size);
 
         /**
-         * Create a filled cstring node allocated for specified string size.
-         * The new operator would also need the size as an overdraft value.
-         * The newly allocated string is filled with the fill value.
-         * @param size of string.
-         * @param fill character value to fill string with.
-         */
-        cstring(strsize_t size, char fill);
-
-        /**
-         * Used to clear a string.  If null terminated, then the string ends
-         * at the offset, otherwise it is simply filled with fill data up to
-         * the specified size.
+         * Used to clear a string at specified offset.
          * @param offset to clear from.
-         * @param size of field to clear.
          */
-        void clear(strsize_t offset, strsize_t size);
+        void clear(size_t offset);
 
         /**
          * Set part or all of a string with new text.
@@ -180,7 +165,7 @@ public:
          * @param text to insert from null terminated string.
          * @param size of field to modify.  This is filled for fill mode.
          */
-        void set(strsize_t offset, const char *text, strsize_t size);
+        void set(size_t offset, const char *text, size_t size);
 
         /**
          * Set our string from null terminated text up to our allocated size.
@@ -205,25 +190,19 @@ public:
          */
         void fix(void);
 
-        /**
-         * Trim filler at end to reduce filled string to null terminated
-         * string for further processing.
-         */
-        void unfix(void);
-
         /**
          * Adjust size of our string buffer by deleting characters from
          * start of buffer.
          * @param number of characters to delete.
          */
-        void inc(strsize_t number);
+        void inc(size_t number);
 
         /**
          * Adjust size of our string buffer by deleting characters from
          * end of buffer.
          * @param number of characters to delete.
          */
-        void dec(strsize_t number);
+        void dec(size_t number);
     };
 
 protected:
@@ -232,10 +211,9 @@ protected:
     /**
      * Factory create a cstring object of specified size.
      * @param size of allocated space for string buffer.
-     * @param fill character to use or 0 if null.
      * @return new cstring object.
      */
-    cstring *create(strsize_t size, char fill = 0) const;
+    cstring *create(size_t size) const;
 
 public:
     /**
@@ -247,9 +225,6 @@ public:
      */
     virtual int compare(const char *string) const;
 
-    inline int collate(const char *string) const
-        {return compare(string);}
-
 protected:
     /**
     * Test if two string values are equal.
@@ -282,49 +257,24 @@ protected:
      * to disable.
      * @param size to add to allocated space when creating new cstring.
      */
-    virtual void cow(strsize_t size = 0);
+    virtual void cow(size_t size = 0);
 
-    strsize_t getStringSize(void) const;
+    size_t getStringSize(void) const;
 
 public:
-#if _MSC_VER > 1400        // windows broken dll linkage issue...
-    const static strsize_t npos = ((strsize_t)-1);
+    const static size_t npos = ((size_t)-1);
     const static char eos = '\0';
-#else
-    static const strsize_t npos;
-    static const char eos;
-#endif
-
 
     /**
      * Create a new empty string object.
      */
     String();
 
-    /**
-     * Create a string from a long integer.
-     * @param value to convert to string.
-     */
-    String(long value);
-
-    /**
-     * Create a string from a floating point.
-     * @param value to convert to string.
-     */
-    String(double value);
-
     /**
      * Create an empty string with a buffer pre-allocated to a specified size.
      * @param size of buffer to allocate.
      */
-    String(strsize_t size);
-
-    /**
-     * Create a filled string with a buffer pre-allocated to a specified size.
-     * @param size of buffer to allocate.
-     * @param fill character to use.
-     */
-    String(strsize_t size, char fill);
+    String(size_t size);
 
     /**
      * Create a string by printf-like formating into a pre-allocated space
@@ -333,7 +283,7 @@ public:
      * @param size of buffer to allocate.
      * @param format control for string.
      */
-    String(strsize_t size, const char *format, ...) __PRINTF(3, 4);
+    String(size_t size, const char *format, ...);
 
 
     /**
@@ -348,7 +298,7 @@ public:
      * @param text to use for string.
      * @param size limit of new string.
      */
-    String(const char *text, strsize_t size);
+    String(const char *text, size_t size);
 
     /**
      * Create a string for a substring.  The end of the substring is a
@@ -377,7 +327,7 @@ public:
      * @param size of substring or 0 if to end.
      * @return string object holding substring.
      */
-    String get(strsize_t offset, strsize_t size = 0) const;
+    String get(size_t offset, size_t size = 0) const;
 
     /**
      * Scan input items from a string object.
@@ -399,7 +349,7 @@ public:
      * @param format string of print format.
      * @return number of bytes written to string.
      */
-    strsize_t printf(const char *format, ...) __PRINTF(2, 3);
+    size_t printf(const char *format, ...) __PRINTF(2, 3);
 
     /**
      * Print items into a string object.
@@ -407,13 +357,17 @@ public:
      * @param args list to print.
      * @return number of bytes written to string.
      */
-    strsize_t vprintf(const char *format, va_list args) __PRINTF(2, 0);
+    size_t vprintf(const char *format, va_list args) __PRINTF(2, 0);
 
     /**
      * Get memory text buffer of string object.
      * @return writable string buffer.
      */
-    char *c_mem(void) const;
+    char *data(void);
+
+    inline char *c_mem() {
+        return data();
+    }
 
     /**
      * Get character text buffer of string object.
@@ -426,7 +380,7 @@ public:
      * @param size to allocate for string.
      * @return true if re-allocated.  False in derived memstring.
      */
-    virtual bool resize(strsize_t size);
+    virtual bool resize(size_t size);
 
     /**
      * Set string object to text of a null terminated string.
@@ -441,7 +395,7 @@ public:
      * @param text to set at offset.
      * @param size of text area to set or 0 until end of text.
      */
-    void set(strsize_t offset, const char *text, strsize_t size = 0);
+    void set(size_t offset, const char *text, size_t size = 0);
 
     /**
      * Set a text field within our string object.
@@ -450,7 +404,7 @@ public:
      * @param offset in object string buffer to set text at.
      * @param size of part of buffer to set with text and overflow.
      */
-    void set(const char *text, char overflow, strsize_t offset, strsize_t size = 0);
+    void set(const char *text, char overflow, size_t offset, size_t size = 0);
 
     /**
      * Set a text field within our string object offset from the end of buffer.
@@ -459,7 +413,7 @@ public:
      * @param offset from end of object string buffer to set text at.
      * @param size of part of buffer to set with text and overflow.
      */
-    void rset(const char *text, char overflow, strsize_t offset, strsize_t size = 0);
+    void rset(const char *text, char overflow, size_t offset, size_t size = 0);
 
     /**
      * Append null terminated text to our string buffer.
@@ -483,8 +437,9 @@ public:
      * Trim lead characters from text.
      * @param count of characters to remove.
      */
-    inline void trim(strsize_t count = 1)
-        {operator+=(count);}
+    inline void trim(size_t count = 1) {
+        operator+=(count);
+    }
 
     /**
      * Chop trailing characters from the string.
@@ -496,8 +451,9 @@ public:
      * Chop trailing characters from text.
      * @param count of characters to remove.
      */
-    inline void chop(strsize_t count = 1)
-        {operator-=(count);}
+    inline void chop(size_t count = 1) {
+        operator-=(count);
+    }
 
     /**
      * Strip lead and trailing characters from the string.
@@ -517,7 +473,7 @@ public:
      * @param offset to start of text field to remove.
      * @param size of text field to remove or 0 to remove to end of string.
      */
-    void cut(strsize_t offset, strsize_t size = 0);
+    void cut(size_t offset, size_t size = 0);
 
     /**
      * Insert (paste) text into string.
@@ -525,14 +481,13 @@ public:
      * @param text to paste.
      * @param size of text to paste.
      */
-    void paste(strsize_t offset, const char *text, strsize_t size = 0);
+    void paste(size_t offset, const char *text, size_t size = 0);
 
     /**
      * Clear a field of a filled string with filler.
      * @param offset to start of field to clear.
-     * @param size of field to fill or 0 to fill to end of string.
      */
-    void clear(strsize_t offset, strsize_t size = 0);
+    void clear(size_t offset);
 
     /**
      * Clear string by setting to empty.
@@ -559,19 +514,19 @@ public:
      * @param list of characters to find.
      * @return count of instances of characters in string.
      */
-    strsize_t ccount(const char *list) const;
+    size_t ccount(const char *list) const;
 
     /**
      * Count all characters in the string (strlen).
      * @return count of characters.
      */
-    strsize_t count(void) const;
+    size_t count(void) const;
 
     /**
      * Get the size of currently allocated space for string.
      * @return size allocated for text.
      */
-    strsize_t size(void) const;
+    size_t size(void) const;
 
     /**
      * Find offset of a pointer into our string buffer.  This can be used
@@ -582,7 +537,7 @@ public:
      * then npos is returned.
      * @param pointer into our object's string buffer.
      */
-    strsize_t offset(const char *pointer) const;
+    size_t offset(const char *pointer) const;
 
     /**
      * Return character found at a specific position in the string.
@@ -609,7 +564,7 @@ public:
      * @param offset to start of scan.
      * @return pointer to first part of string past skipped characters.
      */
-    const char *skip(const char *list, strsize_t offset = 0) const;
+    const char *skip(const char *list, size_t offset = 0) const;
 
     /**
      * Skip trailing characters in the string.  This searches the
@@ -618,7 +573,7 @@ public:
      * @param offset to start of scan.  Default is end of string.
      * @return pointer to first part of string before skipped characters.
      */
-    const char *rskip(const char *list, strsize_t offset = npos) const;
+    const char *rskip(const char *list, size_t offset = npos) const;
 
     /**
      * Search for a substring in the string.
@@ -639,7 +594,7 @@ public:
      * @param offset to start of search.
      * @return pointer to first occurrence of character.
      */
-    const char *find(const char *list, strsize_t offset = 0) const;
+    const char *find(const char *list, size_t offset = 0) const;
 
     /**
      * Find last occurrence of character in the string.
@@ -647,7 +602,7 @@ public:
      * @param offset to start of search.  Default is end of string.
      * @return pointer to last occurrence of character.
      */
-    const char *rfind(const char *list, strsize_t offset = npos) const;
+    const char *rfind(const char *list, size_t offset = npos) const;
 
     /**
      * Split the string by a pointer position.  Everything after the pointer
@@ -661,7 +616,9 @@ public:
      * is removed.
      * @param offset to split position in string.
      */
-    void split(strsize_t offset);
+    void split(size_t offset);
+
+    void fill(size_t size, char fill);
 
     /**
      * Split the string by a pointer position.  Everything before the pointer
@@ -675,7 +632,7 @@ public:
      * is removed.
      * @param offset to split position in string.
      */
-    void rsplit(strsize_t offset);
+    void rsplit(size_t offset);
 
     /**
      * Find pointer in string where specified character appears.
@@ -696,27 +653,23 @@ public:
      * Get length of string.
      * @return length of string.
      */
-    strsize_t len(void) const;
-
-    /**
-     * Get filler character used for field array strings.
-     * @return filler character or 0 if none.
-     */
-    char fill(void);
+    size_t len(void) const;
 
     /**
      * Casting reference to raw text string.
      * @return null terminated text of string.
      */
-    inline operator const char *() const
-        {return c_str();}
+    inline operator const char *() const {
+        return c_str();
+    }
 
     /**
      * Reference raw text buffer by pointer operator.
      * @return null terminated text of string.
      */
-    inline const char *operator*() const
-        {return c_str();}
+    inline const char *operator*() const {
+        return c_str();
+    }
 
     /**
      * Test if the string's allocated space is all used up.
@@ -730,23 +683,25 @@ public:
      * @param size of substring or 0 if to end.
      * @return string object holding substring.
      */
-    String operator()(int offset, strsize_t size) const;
+    String operator()(int offset, size_t size) const;
 
     /**
      * Convenience method for left of string.
      * @param size of substring to gather.
      * @return string object holding substring.
      */
-    inline String left(strsize_t size) const
-        {return operator()(0, size);}
+    inline String left(size_t size) const {
+        return operator()(0, size);
+    }
 
     /**
      * Convenience method for right of string.
      * @param offset of substring from right.
      * @return string object holding substring.
      */
-    inline String right(strsize_t offset) const
-        {return operator()(-((int)offset), 0);}
+    inline String right(size_t offset) const {
+        return operator()(-((int)offset), 0);
+    }
 
     /**
      * Convenience method for substring extraction.
@@ -754,8 +709,9 @@ public:
      * @param size of string to return.
      * @return string object holding substring.
      */
-    inline String copy(strsize_t offset, strsize_t size) const
-        {return operator()((int)offset, size);}
+    inline String copy(size_t offset, size_t size) const {
+        return operator()((int)offset, size);
+    }
 
     /**
      * Reference a string in the object by relative offset.  Positive
@@ -818,7 +774,7 @@ public:
      * Concatenate null terminated text to our object.
      * @param text to concatenate.
      */
-    String operator+(const char *text);
+    const String operator+(const char *text) const;
 
     /**
      * Concatenate null terminated text to our object.  This creates a new
@@ -863,7 +819,7 @@ public:
      * Delete a specified number of characters from start of string.
      * @param number of characters to delete.
      */
-    String& operator+=(strsize_t number);
+    String& operator+=(size_t number);
 
     /**
      * Delete last character from string.
@@ -874,13 +830,13 @@ public:
      * Delete a specified number of characters from end of string.
      * @param number of characters to delete.
      */
-    String& operator-=(strsize_t number);
+    String& operator-=(size_t number);
 
     /**
      * Delete a specified number of characters from start of string.
      * @param number of characters to delete.
      */
-    String& operator*=(strsize_t number);
+    String& operator*=(size_t number);
 
     /**
      * Compare our object with null terminated text.
@@ -924,11 +880,13 @@ public:
      */
     bool operator>=(const char *text) const;
 
-    inline String& operator<<(const char *text)
-        {add(text); return *this;}
+    inline String& operator<<(const char *text) {
+        add(text); return *this;
+    }
 
-    inline String& operator<<(char code)
-        {add(code); return *this;}
+    inline String& operator<<(char code) {
+        add(code); return *this;
+    }
 
     /**
      * Parse short integer value from a string.
@@ -938,7 +896,7 @@ public:
     String &operator%(short& value);
 
     /**
-     * Parse long integer value from a string.
+     * Parse short integer value from a string.
      * @param value to store.
      * @return object in expression.
      */
@@ -1138,8 +1096,9 @@ public:
      */
     static int compare(const char *text1, const char *text2);
 
-    static inline int collate(const char *text1, const char *text2)
-        {return compare(text1, text2);}
+    inline static int collate(const char *text1, const char *text2) {
+        return compare(text1, text2);
+    }
 
     /**
      * Simple equal test for strings.
@@ -1167,14 +1126,6 @@ public:
      */
     static bool equal(const char *text1, const char *text2, size_t size);
 
-    /**
-     * Depreciated case insensitive string comparison function.
-     * @param text1 to compare.
-     * @param text2 to compare.
-     * @return 0 if equal, >0 if text1 > text2, <0 if text1 < text2.
-     */
-    static int case_compare(const char *text1, const char *text2);
-
     /**
      * Simple case insensitive equal test for strings.
      * @param text1 to test.
@@ -1182,16 +1133,7 @@ public:
      * @return true if equal.
      */
     static bool eq_case(const char *text1, const char *text2);
-
-    /**
-     * Depreciated case insensitive string comparison function.
-     * @param text1 to compare.
-     * @param text2 to compare.
-     * @param size limit of strings to compare.
-     * @return 0 if equal, >0 if text1 > text2, <0 if text1 < text2.
-     */
-    static int case_compare(const char *text1, const char *text2, size_t size);
-
+    
     /**
      * Simple case insensitive equal test for strings.
      * @param text1 to test.
@@ -1293,11 +1235,13 @@ public:
      */
     static const char *pos(const char *text, ssize_t offset);
 
-    inline static char *right(const char *text, size_t size)
-        {return dup(pos(text, -(ssize_t)size));}
+    inline static char *right(const char *text, size_t size) {
+        return dup(pos(text, -(ssize_t)size));
+    }
 
-    inline static char *copy(const char *text, size_t offset, size_t len)
-        {return left(pos(text, (ssize_t)offset), len);}
+    inline static char *copy(const char *text, size_t offset, size_t len) {
+        return left(pos(text, (ssize_t)offset), len);
+    }
 
     static void cut(char *text, size_t offset, size_t len);
 
@@ -1315,8 +1259,9 @@ public:
      * @param end of line marker characters or NULL if not used.
      * @return token extracted from string or NULL if no more tokens found.
      */
-    inline char *token(char **last, const char *list, const char *quote = NULL, const char *end = NULL)
-        {return token(c_mem(), last, list, quote, end);}
+    inline char *token(char **last, const char *list, const char *quote = NULL, const char *end = NULL) {
+        return token(data(), last, list, quote, end);
+    }
 
     /**
      * Convert string to a double value.
@@ -1324,8 +1269,9 @@ public:
      * @param pointer to update with end of parsed value.
      * @return double value of object.
      */
-    inline double tod(char **pointer = NULL)
-        {return strtod(c_mem(), pointer);}
+    inline double tod(char **pointer = NULL) {
+        return strtod(data(), pointer);
+    }
 
     /**
      * Convert string to a long value.
@@ -1333,8 +1279,9 @@ public:
      * @param pointer to update with end of parsed value.
      * @return long value of object.
      */
-    inline long tol(char **pointer = NULL)
-        {return strtol(c_mem(), pointer, 0);}
+    inline long tol(char **pointer = NULL) {
+        return strtol(data(), pointer, 0);
+    }
 
     /**
      * Convert text to a double value.
@@ -1342,8 +1289,9 @@ public:
      * @param pointer to update with end of parsed value.
      * @return double value of object.
      */
-    inline static double tod(const char *text, char **pointer = NULL)
-        {return strtod(text, pointer);}
+    inline static double tod(const char *text, char **pointer = NULL) {
+        return strtod(text, pointer);
+    }
 
     /**
      * Convert text to a long value.
@@ -1351,8 +1299,9 @@ public:
      * @param pointer to update with end of parsed value.
      * @return long value of object.
      */
-    inline static long tol(const char *text, char **pointer = NULL)
-        {return strtol(text, pointer, 0);}
+    inline static long tol(const char *text, char **pointer = NULL) {
+        return strtol(text, pointer, 0);
+    }
 
     /**
      * Standard radix 64 string encoding.
@@ -1362,6 +1311,8 @@ public:
      */
     static String b64(const uint8_t *binary, size_t size);
 
+    static size_t b64size(size_t size);
+
     /**
      * Standard radix 64 encoding.
      * @param string of encoded text save into.
@@ -1377,9 +1328,12 @@ public:
      * @param binary data to save.
      * @param string of encoded text.
      * @param size of destination buffer.
-     * @return number of bytes actually decoded.
+     * @param ws flag to skip whitespaces.
+     * @return number of characters actually decoded.
      */
-    static size_t b64decode(uint8_t *binary, const char *string, size_t size);
+    static size_t b64decode(uint8_t *binary, const char *string, size_t size, bool ws = false);
+
+    static size_t b64count(const char *str, bool ws = false);
 
     /**
      * 24 bit crc as used in openpgp.
@@ -1403,7 +1357,7 @@ public:
      * @param size of data.
      * @return string from data.
      */
-    static String hex(const unsigned char *binary, size_t size);
+    static String hex(const uint8_t *binary, size_t size);
 
     /**
      * Dump hex data to a string buffer.
@@ -1412,18 +1366,23 @@ public:
      * @param format string to convert with.
      * @return number of bytes processed.
      */
-    static unsigned hexdump(const unsigned char *binary, char *string, const char *format);
+    static size_t hexdump(const uint8_t *binary, char *string, const char *format);
 
     /**
      * Pack hex data from a string buffer.
      * @param binary memory to pack.
      * @param string to save into.
      * @param format string to convert with.
+     * @param whitespace flag to ignore.
      * @return number of bytes processed.
      */
-    static unsigned hexpack(unsigned char *binary, const char *string, const char *format);
+    static size_t hexpack(uint8_t *binary, const char *string, const char *format);
 
-    static unsigned hexsize(const char *format);
+    static size_t hex2bin(const char *string, uint8_t *binary, size_t maxsize, bool wsflag = false);
+
+    static size_t hexsize(const char *format);
+
+    static size_t hexcount(const char *str, bool ws = false);
 };
 
 /**
@@ -1436,42 +1395,39 @@ public:
 class __EXPORT memstring : public String
 {
 public:
-#if _MSC_VER > 1400        // windows broken dll linkage issue...
     const static size_t header = sizeof(String::cstring);
-#else
-    static const size_t header;
-#endif
 
 private:
-    bool resize(strsize_t size);
-    void cow(strsize_t adj = 0);
-    void release(void);
+    bool resize(size_t size) __FINAL;
+    void cow(size_t adj = 0) __FINAL;
+    void release(void) __FINAL;
 
 protected:
-    cstring *c_copy(void) const;
+    cstring *c_copy(void) const __OVERRIDE;
 
 public:
     /**
      * Assign the text of a string to our object.
      * @param object to copy text from.
      */
-    inline void operator=(String& object)
-        {set(object.c_str());}
+    inline void operator=(String& object) {
+        set(object.c_str());
+    }
 
     /**
      * Assign null terminated text to our object.
      * @param text to copy.
      */
-    inline void operator=(const char *text)
-        {set(text);}
+    inline void operator=(const char *text) {
+        set(text);
+    }
 
     /**
      * Create an instance of a memory string.
      * @param memory to use for cstring object.
      * @param size of string.  Total size must include space for overhead.
-     * @param fill character for fixed character fields.
      */
-    memstring(void *memory, strsize_t size, char fill = 0);
+    memstring(void *memory, size_t size);
 
     /**
      * Destroy memory string.
@@ -1481,17 +1437,15 @@ public:
     /**
      * Create a memory string with memory allocated from the heap.
      * @param size of string to allocate.  Automatically adds control size.
-     * @param fill character for fixed field strings.
      */
-    static memstring *create(strsize_t size, char fill = 0);
+    static memstring *create(size_t size);
 
     /**
      * Create a memory string with memory allocated from a pager.
      * @param pager to allocate memory from.
      * @param size of string to allocate.  Automatically adds control size.
-     * @param fill character for fixed field strings.
      */
-    static memstring *create(MemoryProtocol *pager, strsize_t size, char fill = 0);
+    static memstring *create(MemoryProtocol *pager, size_t size);
 };
 
 /**
@@ -1511,95 +1465,112 @@ public:
     /**
      * Create a new character buffer with an empty string.
      */
-    inline charbuf()
-        {buffer[0] = 0;}
+    inline charbuf() {
+        buffer[0] = 0;
+    }
 
     /**
      * Create a character buffer with assigned text.  If the text is
      * larger than the size of the object, it is truncated.
      * @param text to assign.
      */
-    inline charbuf(const char *text)
-        {String::set(buffer, S, text);}
+    inline charbuf(const char *text) {
+        String::set(buffer, S, text);
+    }
 
     /**
      * Copy constructor.
      */
-    inline charbuf(const charbuf& copy)
-        {String::set(buffer, S, copy.buffer);}
+    inline charbuf(const charbuf& copy) {
+        String::set(buffer, S, copy.buffer);
+    }
 
     /**
      * Assign null terminated text to the object.
      * @param text to assign.
      */
-    inline void operator=(const char *text)
-        {String::set(buffer, S, text);}
+    inline void operator=(const char *text) {
+        String::set(buffer, S, text);
+    }
 
     /**
      * Concatenate text into the object.  If the text is larger than the
      * size of the object, then it is truncated.
      * @param text to append.
      */
-    inline void operator+=(const char *text)
-        {String::add(buffer, S, text);}
+    inline void operator+=(const char *text) {
+        String::add(buffer, S, text);
+    }
 
     /**
      * Test if data is contained in the object.
      * @return true if there is text.
      */
-    inline operator bool() const
-        {return buffer[0];}
+    inline operator bool() const {
+        return buffer[0];
+    }
 
     /**
      * Test if the object is empty.
      * @return true if the object is empty.
      */
-    inline bool operator!() const
-        {return buffer[0] == 0;}
+    inline bool operator!() const {
+        return buffer[0] == 0;
+    }
 
     /**
      * Get text by casting reference.
      * @return pointer to text in object.
      */
-    inline operator char *()
-        {return buffer;}
+    inline operator char *() {
+        return buffer;
+    }
 
     /**
      * Get text by object pointer reference.
      * @return pointer to text in object.
      */
-    inline char *operator*()
-        {return buffer;}
+    inline char *operator*() {
+        return buffer;
+    }
 
     /**
      * Array operator to get a character from the object.
      * @param offset of character in string buffer.
      * @return character at offset.
      */
-    inline char& operator[](size_t offset) const
-        {return buffer[offset];}
+    inline char& operator[](size_t offset) const {
+        if(offset >= S)
+            __THROW_RANGE("charbuf offset"); 
+        return buffer[offset];
+    }
 
     /**
      * Get a pointer to an offset in the object by expression operator.
      * @param offset of character in string buffer.
      * @return pointer to offset in object.
      */
-    inline char *operator()(size_t offset)
-        {return buffer + offset;}
+    inline char *operator()(size_t offset) {
+        if(offset >= S)
+            __THROW_RANGE("charbuf range");
+        return buffer + offset;
+    }
 
     /**
      * Get allocated size of the object.
      * @return allocated size.
      */
-    inline size_t size(void) const
-        {return S;}
+    inline size_t size(void) const {
+        return S;
+    }
 
     /**
      * Get current length of string.
      * @return length of string.
      */
-    inline size_t len(void) const
-        {return strlen(buffer);}
+    inline size_t len(void) const {
+        return strlen(buffer);
+    }
 };
 
 /**
@@ -1619,7 +1590,7 @@ typedef String::regex stringex_t;
  * easily do.
  * @author David Sugar <dyfet@gnutelephony.org>
  */
-template<strsize_t S>
+template<size_t S>
 class stringbuf : public memstring
 {
 private:
@@ -1635,46 +1606,40 @@ public:
      * Create a string buffer from a null terminated string.
      * @param text to place in object.
      */
-    inline stringbuf(const char *text) : memstring(buffer, S) {set(text);}
+    inline stringbuf(const char *text) : memstring(buffer, S) {
+        set(text);
+    }
 
     /**
      * Assign a string buffer from a null terminated string.
      * @param text to assign to object.
      */
-    inline void operator=(const char *text)
-        {set(text);}
+    inline void operator=(const char *text) {
+        set(text);
+    }
 
     /**
      * Assign a string buffer from another string object.
      * @param object to assign from.
      */
-    inline void operator=(String& object)
-        {set(object.c_str());}
+    inline void operator=(String& object) {
+        set(object.c_str());
+    }
 };
 
-#if !defined(_MSWINDOWS_) && !defined(__QNX__)
-
-#ifndef stricmp
-#define stricmp(x,y) String::case_compare(x,y)
-#endif
-
-#ifndef strnicmp
-#define strnicmp(x,y,z) String::case_compare(x,y,z)
-#endif
-
-#endif
-
 /**
  * Compare two null terminated strings if equal.
  * @param s1 string to compare.
  * @param s2 string to compare.
  * @return true if equal.
  */
-inline bool eq(char const *s1, char const *s2)
-    {return String::equal(s1, s2);}
+inline bool eq(char const *s1, char const *s2) {
+    return String::equal(s1, s2);
+}
 
-inline bool ne(char const *s1, char const *s2)
-    {return !String::equal(s1, s2);}
+inline bool ne(char const *s1, char const *s2) {
+    return !String::equal(s1, s2);
+}
 
 /**
  * Compare two null terminated strings if equal up to specified size.
@@ -1683,11 +1648,13 @@ inline bool ne(char const *s1, char const *s2)
  * @param size of string to compare.
  * @return true if equal.
  */
-inline bool eq(char const *s1, char const *s2, size_t size)
-    {return String::equal(s1, s2, size);}
+inline bool eq(char const *s1, char const *s2, size_t size) {
+    return String::equal(s1, s2, size);
+}
 
-inline bool ne(char const *s1, char const *s2, size_t size)
-    {return !String::equal(s1, s2, size);}
+inline bool ne(char const *s1, char const *s2, size_t size) {
+    return !String::equal(s1, s2, size);
+}
 
 /**
  * Compare two string objects if equal.  The left string is an object,
@@ -1698,23 +1665,29 @@ inline bool ne(char const *s1, char const *s2, size_t size)
  * @param s2 string to compare.
  * @return true if equal.
  */
-inline bool eq(String &s1, const char *s2)
-    {return s1.compare(s2) == 0;}
+inline bool eq(String &s1, const char *s2) {
+    return s1.compare(s2) == 0;
+}
 
-inline bool ne(String &s1, String &s2)
-    {return s1.compare(s2) != 0;}
+inline bool ne(String &s1, String &s2) {
+    return s1.compare(s2) != 0;
+}
 
-inline bool lt(String &s1, const char *s2)
-    {return s1.compare(s2) < 0;}
+inline bool lt(String &s1, const char *s2) {
+    return s1.compare(s2) < 0;
+}
 
-inline bool gt(String &s1, const char *s2)
-    {return s1.compare(s2) > 0;}
+inline bool gt(String &s1, const char *s2) {
+    return s1.compare(s2) > 0;
+}
 
-inline bool le(String &s1, const char *s2)
-    {return s1.compare(s2) <= 0;}
+inline bool le(String &s1, const char *s2) {
+    return s1.compare(s2) <= 0;
+}
 
-inline bool ge(String &s1, const char *s2)
-    {return s1.compare(s2) >= 0;}
+inline bool ge(String &s1, const char *s2) {
+    return s1.compare(s2) >= 0;
+}
 
 /**
  * Compare two null terminated strings if equal ignoring case.  This is
@@ -1737,35 +1710,42 @@ inline bool ne_case(char const *s1, char const *s2)
  * @param size of string to compare.
  * @return true if equal.
  */
-inline bool eq_case(char const *s1, char const *s2, size_t size)
-    {return String::eq_case(s1, s2, size);}
+inline bool eq_case(char const *s1, char const *s2, size_t size) {
+    return String::eq_case(s1, s2, size);
+}
 
-inline String str(const char *string)
-    {return (String)string;}
+inline String str(const char *string) {
+    return (String)string;
+}
 
-inline String str(String& string)
-    {return (String)string;}
+inline String str(String& string) {
+    return (String)string;
+}
 
-inline String str(short value)
-    {String temp(16, "%hd", value); return temp;}
+inline String str(short value) {
+    String temp(16, "%hd", value); return temp;
+}
 
-inline String str(unsigned short value)
-    {String temp(16, "%hu", value); return temp;}
+inline String str(unsigned short value) {
+    String temp(16, "%hu", value); return temp;
+}
 
-inline String str(long value)
-    {String temp(32, "%ld", value); return temp;}
+inline String str(long value) {
+    String temp(32, "%ld", value); return temp;
+}
 
-inline String str(unsigned long value)
-    {String temp(32, "%lu", value); return temp;}
+inline String str(unsigned long value) {
+    String temp(32, "%lu", value); return temp;
+}
 
-inline String str(double value)
-    {String temp(40, "%f", value); return temp;}
-
-String str(CharacterProtocol& cp, strsize_t size);
+inline String str(double value) {
+    String temp(40, "%f", value); return temp;
+}
 
 template<>
-inline void swap<string_t>(string_t& s1, string_t& s2)
-    {String::swap(s1, s2);}
+inline void swap<string_t>(string_t& s1, string_t& s2) {
+    String::swap(s1, s2);
+}
 
 class __EXPORT strdup_t
 {
@@ -1773,38 +1753,53 @@ private:
     char *data;
 
 public:
-    inline strdup_t()
-        {data = NULL;}
+    inline strdup_t() {
+        data = NULL;
+    }
 
-    inline strdup_t(char *str)
-        {data = str;}
+    inline strdup_t(char *str) {
+        data = str;
+    }
 
-    inline ~strdup_t()
-        {if(data) ::free(data);}
+    inline ~strdup_t() {
+        if(data) 
+            ::free(data);
+    }
 
-    inline strdup_t& operator=(char *str)
-        {if(data) ::free(data); data = str; return *this;}
+    inline strdup_t& operator=(char *str) {
+        if(data) 
+            ::free(data); 
+        data = str; 
+        return *this;
+    }
 
-    inline operator bool() const
-        {return data != NULL;}
+    inline operator bool() const {
+        return data != nullptr;
+    }
 
-    inline bool operator!() const
-        {return data == NULL;}
+    inline bool operator!() const {
+        return data == nullptr;
+    }
 
-    inline operator char*() const
-        {return data;}
+    inline operator char*() const {
+        return data;
+    }
 
-    inline const char *c_str(void) const
-        {return data;}
+    inline const char *c_str(void) const {
+        return data;
+    }
 
-    inline const char *operator*() const
-        {return data;}
+    inline const char *operator*() const {
+        return data;
+    }
 
-    inline char& operator[](int size)
-        {return data[size];}
+    inline char& operator[](int size) {
+        return data[size];
+    }
 
-    inline char *operator+(size_t size)
-        {return data + size;}
+    inline char *operator+(size_t size) {
+        return data + size;
+    }
 };
 
 } // namespace ucommon
