@@ -18,12 +18,12 @@
 
 #include "local.h"
 
-static const unsigned char *_salt = NULL;
+static const uint8_t *_salt = NULL;
 static unsigned _rounds = 1;
 
 namespace ucommon {
 
-void Cipher::Key::assign(const char *text, size_t size, const unsigned char *salt, unsigned rounds)
+void Cipher::Key::assign(const char *text, size_t size, const uint8_t *salt, unsigned rounds)
 {
     keysize = 0;
 }
@@ -43,7 +43,7 @@ void Cipher::Key::assign(const char *text, size_t size)
     assign(text, size, _salt, _rounds);
 }
 
-void Cipher::Key::options(const unsigned char *salt, unsigned rounds)
+void Cipher::Key::options(const uint8_t *salt, unsigned rounds)
 {
     _salt = salt;
     _rounds = rounds;
@@ -54,7 +54,7 @@ bool Cipher::has(const char *id)
     return false;
 }
 
-void Cipher::push(unsigned char *address, size_t size)
+void Cipher::push(uint8_t *address, size_t size)
 {
 }
 
@@ -62,7 +62,7 @@ void Cipher::release(void)
 {
 }
 
-void Cipher::set(key_t key, mode_t mode, unsigned char *address, size_t size)
+void Cipher::set(const key_t key, mode_t mode, uint8_t *address, size_t size)
 {
     release();
 
@@ -73,7 +73,7 @@ void Cipher::set(key_t key, mode_t mode, unsigned char *address, size_t size)
     memcpy(&keys, key, sizeof(keys));
 }
 
-size_t Cipher::put(const unsigned char *data, size_t size)
+size_t Cipher::put(const uint8_t *data, size_t size)
 {
     size_t count = 0;
 
@@ -87,7 +87,7 @@ size_t Cipher::put(const unsigned char *data, size_t size)
     return 0;
 }
 
-size_t Cipher::pad(const unsigned char *data, size_t size)
+size_t Cipher::pad(const uint8_t *data, size_t size)
 {
     return 0;
 }
