@@ -20,6 +20,21 @@
 
 namespace ucommon {
 
+AutoClear::AutoClear(size_t max)
+{
+    pointer = ::malloc(max);
+    size = max;
+}
+
+AutoClear::~AutoClear()
+{
+    if(pointer) {
+        memset(pointer, 0, size);
+        ::free(pointer);
+        pointer = NULL;
+    }
+}
+
 Digest::Digest()
 {
     hashtype = NULL;
@@ -46,15 +61,29 @@ Digest::~Digest()
     memset(buffer, 0, sizeof(buffer));
 }
 
-const char *Digest::c_str(void)
+secure::string Digest::str(void)
 {
     if(!bufsize)
         get();
 
-    return textbuf;
+    if(!bufsize)
+        return secure::string();
+
+    return secure::string(textbuf);
 }
 
-void Digest::uuid(char *str, const char *name, const unsigned char *ns)
+secure::keybytes Digest::key(void)
+{
+    if(!bufsize)
+        get();
+
+    if(!bufsize)
+        return secure::keybytes();
+
+    return secure::keybytes(buffer, bufsize);
+}
+
+secure::string Digest::uuid(const char *name, const uint8_t *ns)
 {
     unsigned mask = 0x50;
     const char *type = "sha1";
@@ -67,21 +96,16 @@ void Digest::uuid(char *str, const char *name, const unsigned char *ns)
     if(ns)
         md.put(ns, 16);
     md.puts(name);
-    unsigned char *buf = (unsigned char *)md.get();
+    uint8_t *buf = (uint8_t *)md.get();
 
     buf[6] &= 0x0f;
     buf[6] |= mask;
     buf[8] &= 0x3f;
     buf[8] |= 0x80;
 
+    char str[40];
     String::hexdump(buf, str, "4-2-2-2-6");
-}
-
-secure::string Digest::uuid(const char *name, const unsigned char *ns)
-{
-    char buf[38];
-    uuid(buf, name, ns);
-    return secure::string(buf);
+    return secure::string(str);
 }
 
 secure::keybytes Digest::md5(const uint8_t *mem, size_t size)
@@ -92,7 +116,7 @@ secure::keybytes Digest::md5(const uint8_t *mem, size_t size)
     digest_t digest("md5");
     digest.put(mem, size);
     mem = digest.get();
-    return secure::keybytes(mem, digest.size() * 8, secure::KEY_DIGEST);
+    return secure::keybytes(mem, digest.size());
 }
 
 secure::keybytes Digest::sha1(const uint8_t *mem, size_t size)
@@ -103,7 +127,7 @@ secure::keybytes Digest::sha1(const uint8_t *mem, size_t size)
     digest_t digest("sha1");
     digest.put(mem, size);
     mem = digest.get();
-    return secure::keybytes(mem, digest.size() * 8, secure::KEY_DIGEST);
+    return secure::keybytes(mem, digest.size());
 }
 
 secure::keybytes Digest::sha256(const uint8_t *mem, size_t size)
@@ -114,7 +138,7 @@ secure::keybytes Digest::sha256(const uint8_t *mem, size_t size)
     digest_t digest("sha256");
     digest.put(mem, size);
     mem = digest.get();
-    return secure::keybytes(mem, digest.size() * 8, secure::KEY_DIGEST);
+    return secure::keybytes(mem, digest.size());
 }
 
 secure::keybytes Digest::sha384(const uint8_t *mem, size_t size)
@@ -125,7 +149,7 @@ secure::keybytes Digest::sha384(const uint8_t *mem, size_t size)
     digest_t digest("sha384");
     digest.put(mem, size);
     mem = digest.get();
-    return secure::keybytes(mem, digest.size() * 8, secure::KEY_DIGEST);
+    return secure::keybytes(mem, digest.size());
 }
 
 secure::string Digest::md5(const char *text)
@@ -135,7 +159,7 @@ secure::string Digest::md5(const char *text)
 
     digest_t digest("md5");
     digest.puts(text);
-    return secure::string(*digest, secure::MD5_DIGEST);
+    return secure::string(*digest);
 }
 
 secure::string Digest::sha1(const char *text)
@@ -145,7 +169,7 @@ secure::string Digest::sha1(const char *text)
 
     digest_t digest("sha1");
     digest.puts(text);
-    return secure::string(*digest, secure::SHA_DIGEST);
+    return secure::string(*digest);
 }
 
 secure::string Digest::sha256(const char *text)
@@ -155,7 +179,7 @@ secure::string Digest::sha256(const char *text)
 
     digest_t digest("sha256");
     digest.puts(text);
-    return secure::string(*digest, secure::SHA_DIGEST);
+    return secure::string(*digest);
 }
 
 secure::string Digest::sha384(const char *text)
@@ -165,7 +189,7 @@ secure::string Digest::sha384(const char *text)
 
     digest_t digest("sha384");
     digest.puts(text);
-    return secure::string(*digest, secure::SHA_DIGEST);
+    return secure::string(*digest);
 }
 
 secure::keybytes HMAC::sha256(secure::keybytes key, const uint8_t *mem, size_t size)
@@ -176,7 +200,7 @@ secure::keybytes HMAC::sha256(secure::keybytes key, const uint8_t *mem, size_t s
 	hmac_t hmac("sha256", key);
     hmac.put(mem, size);
     mem = hmac.get();
-    return secure::keybytes(mem, hmac.size() * 8, secure::KEY_DIGEST);
+    return secure::keybytes(mem, hmac.size());
 }
 
 secure::keybytes HMAC::sha384(secure::keybytes key, const uint8_t *mem, size_t size)
@@ -187,7 +211,7 @@ secure::keybytes HMAC::sha384(secure::keybytes key, const uint8_t *mem, size_t s
 	hmac_t hmac("sha384", key);
     hmac.put(mem, size);
     mem = hmac.get();
-    return secure::keybytes(mem, hmac.size() * 8, secure::KEY_DIGEST);
+    return secure::keybytes(mem, hmac.size());
 }
 
 #if defined(_MSWINDOWS_)
@@ -214,6 +238,17 @@ static void cexport(HCERTSTORE ca, FILE *fp)
     }
 }
 
+secure::string secure::pass(const char *prompt, size_t size)
+{
+    autoclear<char *> buffer(size);
+    secure::string out;
+
+    if(shell::getpass(prompt, *buffer, size))
+        out = *buffer;
+
+    return out;
+}
+
 const char *secure::oscerts(void)
 {
     const char *path = "c:/temp/ca-bundle.crt";
@@ -296,7 +331,7 @@ int secure::oscerts(const char *pathname)
 
 void secure::uuid(char *str)
 {
-    static unsigned char buf[16];
+    static uint8_t buf[16];
     static Timer::tick_t prior = 0l;
     static unsigned short seq;
     Timer::tick_t current = Timer::ticks();
@@ -310,18 +345,18 @@ void secure::uuid(char *str)
     if(current == prior)
         ++seq;
     else
-        Random::fill((unsigned char *)&seq, sizeof(seq));
+        Random::fill((uint8_t *)&seq, sizeof(seq));
 
-    buf[8] = (unsigned char)((seq >> 8) & 0xff);
-    buf[9] = (unsigned char)(seq & 0xff);
-    buf[3] = (unsigned char)(current & 0xff);
-    buf[2] = (unsigned char)((current >> 8) & 0xff);
-    buf[1] = (unsigned char)((current >> 16) & 0xff);
-    buf[0] = (unsigned char)((current >> 24) & 0xff);
-    buf[5] = (unsigned char)((current >> 32) & 0xff);
-    buf[4] = (unsigned char)((current >> 40) & 0xff);
-    buf[7] = (unsigned char)((current >> 48) & 0xff);
-    buf[6] = (unsigned char)((current >> 56) & 0xff);
+    buf[8] = (uint8_t)((seq >> 8) & 0xff);
+    buf[9] = (uint8_t)(seq & 0xff);
+    buf[3] = (uint8_t)(current & 0xff);
+    buf[2] = (uint8_t)((current >> 8) & 0xff);
+    buf[1] = (uint8_t)((current >> 16) & 0xff);
+    buf[0] = (uint8_t)((current >> 24) & 0xff);
+    buf[5] = (uint8_t)((current >> 32) & 0xff);
+    buf[4] = (uint8_t)((current >> 40) & 0xff);
+    buf[7] = (uint8_t)((current >> 48) & 0xff);
+    buf[6] = (uint8_t)((current >> 56) & 0xff);
 
     buf[6] &= 0x0f;
     buf[6] |= 0x10;
@@ -346,17 +381,6 @@ HMAC::HMAC()
     textbuf[0] = 0;
 }
 
-HMAC::HMAC(const char *digest, const char *key, size_t len)
-{
-    context = NULL;
-    bufsize = 0;
-    hmactype = NULL;
-    hmacid = 0;
-    textbuf[0] = 0;
-
-    set(digest, key, len);
-}
-
 HMAC::HMAC(const char *digest, const secure::keybytes& key)
 {
     context = NULL;
@@ -365,7 +389,7 @@ HMAC::HMAC(const char *digest, const secure::keybytes& key)
     hmacid = 0;
     textbuf[0] = 0;
 
-    set(digest, (const char *)*key, key.size() / 8);
+    set(digest, key);
 }
 
 HMAC::~HMAC()
@@ -374,14 +398,27 @@ HMAC::~HMAC()
     memset(buffer, 0, sizeof(buffer));
 }
 
-const char *HMAC::c_str(void)
+secure::string HMAC::str(void)
 {
     if(!bufsize)
         get();
 
-    return textbuf;
+    if(!bufsize)
+        return secure::string();
+
+    return secure::string(textbuf);
 }
 
+secure::keybytes HMAC::key(void)
+{
+    if(!bufsize)
+        get();
+
+    if(!bufsize)
+        return secure::keybytes();
+
+    return secure::keybytes(buffer, bufsize);
+}
 
 Cipher::Key::Key(const char *cipher)
 {
@@ -409,11 +446,7 @@ Cipher::Key::Key(const char *cipher, secure::keybytes& iv)
     hashid = algoid = 0;
 
     secure::init();
-
-    if(iv.type() == secure::IV_BUFFER)
-        set(cipher, *iv, iv.size() / 8);
-    else
-        set(cipher);
+    set(cipher, *iv, iv.size());
 }
 
 Cipher::Key::Key(const char *cipher, const char *digest)
@@ -425,7 +458,7 @@ Cipher::Key::Key(const char *cipher, const char *digest)
     set(cipher, digest);
 }
 
-Cipher::Key::Key(const char *cipher, const char *digest, const char *text, size_t size, const unsigned char *salt, unsigned rounds)
+Cipher::Key::Key(const char *cipher, const char *digest, const char *text, size_t size, const uint8_t *salt, unsigned rounds)
 {
     hashtype = algotype = NULL;
     hashid = algoid = 0;
@@ -447,13 +480,24 @@ Cipher::Key::~Key()
     clear();
 }
 
+bool Cipher::Key::operator==(const Key& other) const
+{
+    if(!keysize && !other.keysize)
+        return true;
+
+    if(keysize != other.keysize)
+        return false;
+
+    return !memcmp(keybuf, other.keybuf, keysize);
+}
+
 void Cipher::Key::b64(const char *key)
 {
     clear();
     String::b64decode(keybuf, key, sizeof(keybuf));
 }
 
-void Cipher::Key::set(const unsigned char *key, size_t size)
+void Cipher::Key::set(const uint8_t *key, size_t size)
 {
     if(!size || size >= sizeof(keybuf))
         return;
@@ -490,32 +534,25 @@ size_t Cipher::Key::get(uint8_t *keyout, uint8_t *ivout)
 bool Cipher::Key::set(const char *cipher, const secure::keybytes& iv)
 {
     const uint8_t *ivp = *iv;
-    size_t size = iv.size() / 8;
 
     if(!ivp)
         return false;
 
-    if(iv.type() != secure::IV_BUFFER)
+    if(iv.size() != blksize)
         return false;
 
-    if(size != blksize)
-        return false;
-
-    set(cipher, ivp, size);
+    set(cipher, ivp, iv.size());
     return true;
 }
 
 bool Cipher::Key::set(const secure::keybytes& key) 
 {
     const uint8_t *kvp = *key;
-    size_t size = key.size() / 8;
+    size_t size = key.size();
 
     if(!kvp)
         return false;
 
-    if(key.type() != secure::UNPAIRED_KEYTYPE)
-        return false;
-
     if(size != keysize)
         return false;
 
@@ -541,7 +578,7 @@ void Cipher::Key::clear(void)
     zerofill(ivbuf, sizeof(ivbuf));
 }
 
-Cipher::Cipher(key_t key, mode_t mode, unsigned char *address, size_t size)
+Cipher::Cipher(const key_t key, mode_t mode, uint8_t *address, size_t size)
 {
     bufaddr = NULL;
     bufsize = bufpos = 0;
@@ -583,17 +620,17 @@ size_t Cipher::puts(const char *text)
     size_t len = strlen(text) + 1;
     size_t pad = len % keys.iosize();
 
-    put((const unsigned char *)text, len - pad);
+    put((const uint8_t *)text, len - pad);
     if(pad) {
         memcpy(padbuf, text + len - pad, pad);
         memset(padbuf + pad, 0, keys.iosize() - pad);
-        put((const unsigned char *)padbuf, keys.iosize());
+        put((const uint8_t *)padbuf, keys.iosize());
         zerofill(padbuf, sizeof(padbuf));
     }
     return flush();
 }
 
-void Cipher::set(unsigned char *address, size_t size)
+void Cipher::set(uint8_t *address, size_t size)
 {
     flush();
     bufsize = size;
@@ -601,7 +638,7 @@ void Cipher::set(unsigned char *address, size_t size)
     bufpos = 0;
 }
 
-size_t Cipher::process(unsigned char *buf, size_t len, bool flag)
+size_t Cipher::process(uint8_t *buf, size_t len, bool flag)
 {
     set(buf);
     if(flag)
@@ -613,7 +650,7 @@ size_t Cipher::process(unsigned char *buf, size_t len, bool flag)
 int Random::get(void)
 {
     uint16_t v;;
-    fill((unsigned char *)&v, sizeof(v));
+    fill((uint8_t *)&v, sizeof(v));
     v /= 2;
     return (int)v;
 }
@@ -630,7 +667,7 @@ int Random::get(int min, int max)
     memset(&umax, 0xff, sizeof(umax));
 
     do {
-        fill((unsigned char *)&rand, sizeof(rand));
+        fill((uint8_t *)&rand, sizeof(rand));
     } while(rand > umax - (umax % range));
 
     return min + (rand % range);
@@ -642,7 +679,7 @@ double Random::real(void)
     unsigned rand;
 
     memset(&umax, 0xff, sizeof(umax));
-    fill((unsigned char *)&rand, sizeof(rand));
+    fill((uint8_t *)&rand, sizeof(rand));
 
     return ((double)rand) / ((double)umax);
 }
@@ -654,7 +691,7 @@ double Random::real(double min, double max)
 
 void Random::uuid(char *str)
 {
-    unsigned char buf[16];
+    uint8_t buf[16];
 
     fill(buf, sizeof(buf));
     buf[6] &= 0x0f;
