@@ -99,10 +99,10 @@ bool Digest::put(const void *address, size_t size)
 
     switch(*((char *)hashtype)) {
     case 'm':
-        MD5Update((MD5_CTX*)context, (const unsigned char *)address, size);
+        MD5Update((MD5_CTX*)context, (const uint8_t *)address, size);
         return true;
     case '1':
-        SHA1Update((SHA1_CTX*)context, (const unsigned char *)address, size);
+        SHA1Update((SHA1_CTX*)context, (const uint8_t *)address, size);
         return true;
     case '2':
         sha256_hash((const unsigned char *)address, size, (sha256_ctx *)context);
@@ -160,7 +160,7 @@ void Digest::recycle(bool bin)
         size = 16;
         MD5Init((MD5_CTX*)context);
         if(bin)
-            MD5Update((MD5_CTX*)context, (const unsigned char *)buffer, size);
+            MD5Update((MD5_CTX*)context, (const uint8_t *)buffer, size);
         else {
             unsigned count = 0;
             while(count < size) {
@@ -168,8 +168,7 @@ void Digest::recycle(bool bin)
 "%2.2x", buffer[count]);
                 ++count;
             }
-            MD5Update((MD5_CTX*)context, (const unsigned
-char *)textbuf, size * 2);
+            MD5Update((MD5_CTX*)context, (const uint8_t *)textbuf, size * 2);
         }
         break;
     case '1':
@@ -178,7 +177,7 @@ char *)textbuf, size * 2);
         size = 20;
         SHA1Init((SHA1_CTX*)context);
         if(bin)
-            SHA1Update((SHA1_CTX*)context, (const unsigned char *)buffer, size);
+            SHA1Update((SHA1_CTX*)context, (const uint8_t *)buffer, size);
         else {
             unsigned count = 0;
             while(count < size) {
@@ -228,7 +227,7 @@ char *)textbuf, size * 2);
     bufsize = 0;
 }
 
-const unsigned char *Digest::get(void)
+const uint8_t *Digest::get(void)
 {
     unsigned count = 0;
 
