@@ -49,10 +49,13 @@ typedef struct {
     int v;
 } maptest;
 
+static uint8_t memdata[7] = {0x20, 0x55, 0x77, 0x78, 0x33, 0x66, 0x55};
+
 extern "C" int main()
 {
     stringlist_t mylist;
     stringlistitem_t *item;
+    Atomic::aligned<int> al;
 
     mylist.add("100");
     mylist.add("050");
@@ -86,6 +89,15 @@ extern "C" int main()
     assert((int)iptr == 3);
     assert(iptr.copies() == 2);
 
+    static typeref_guard gptr;
+    typeref<int> lptr;
+    lptr = (int)3;
+    gptr = lptr;
+    assert(lptr.copies() == 2);
+    typeref<int> nextptr;
+    nextptr = gptr;
+    assert(nextptr.copies() == 3);
+
     iptr = 17;
     assert((int)iptr == 17);
     assert(iptr.copies() == 1);
@@ -101,8 +113,8 @@ extern "C" int main()
     xptr.clear();
     assert(dval == 1);
 
-    stringref sref = "this is a test";
-    stringref xref = sref;
+    stringref<auto_release> sref = "this is a test";
+    stringref<auto_release> xref = sref;
     assert(*sref == *xref);
     assert(!strcmp((const char *)sref, "this is a test"));
     assert(sref.size() == 14);
@@ -228,5 +240,44 @@ extern "C" int main()
     assert(out == nullptr);
     pmap.release(out);
 
+    int a = 1, b = 2;
+    swap(a, b);
+    assert(b == 1);
+    assert(a == 2);
+    copy(b, a);
+    assert(a == 1);
+    b = 2; 
+    move(b, a);
+    assert(a == 2);
+    assert(b == 0);
+    clear(a);
+    assert(a == 0);
+
+    stringref<auto_release> s1, s2;
+    s1.hex(memdata, sizeof(memdata));
+    s2.b64(memdata, 4);
+
+    assert(eq(s1, "20557778336655"));
+    assert(eq(s2, "IFV3eA=="));
+
+    byteref<auto_release> bin;
+    bin.hex(*s1);
+    s2 = bin.hex();
+    assert(eq(s2, "20557778336655"));
+
+    bin.b64("IFV3eA");
+    s1 = bin.hex();
+    assert(eq(s1, "20557778"));
+
+    al((int)3);
+    assert(*al == 3);
+
+    stringref<secure_release> s4 = "abc";
+
+    stringref<release_later> s6 = "abc";
+    stringref<release_later> s7 = "123";
+    s6 = "";
+    s7 = "";
+    assert(release_later.purge() == 2);
     return 0;
 }
