@@ -44,7 +44,7 @@ extern "C" int main()
     assert(localhost.get(AF_INET) != NULL);
     Socket::query(localhost.get(AF_INET), addrbuf, sizeof(addrbuf));
     assert(0 == strcmp(addrbuf, "127.0.0.1"));
-    Socket::via((struct sockaddr *)&addr, localhost.get(AF_INET));
+    Socket::via((struct sockaddr *)&addr, localhost.get(AF_INET), sizeof(addr));
     Socket::query((struct sockaddr *)&addr, addrbuf, sizeof(addrbuf));
     assert(0 == strcmp(addrbuf, "127.0.0.1"));
     assert(eq((struct sockaddr *)&addr, localhost.get(AF_INET)));
@@ -67,7 +67,7 @@ extern "C" int main()
     assert(testing.getPort() == 4444);
     assert(testing == localhost);
     localhost.setAny();
-    assert(localhost);
+    assert(localhost.is_valid());
     assert(localhost.isAny());
     assert(localhost.family() == AF_INET);
 
@@ -90,7 +90,7 @@ extern "C" int main()
     len = localhost6.print(addrbuf, sizeof(addrbuf), false, true);
     assert(len == 5);
     assert(0 == strcmp(addrbuf, "[::1]"));
-    if(!Socket::via((struct sockaddr *)&addr, localhost6.get(AF_INET6))) {
+    if(!Socket::via((struct sockaddr *)&addr, localhost6.get(AF_INET6), sizeof(addr))) {
         Socket::query((struct sockaddr *)&addr, addrbuf, sizeof(addrbuf));
         assert(0 == strcmp(addrbuf, "::1"));
         assert(Socket::equal((struct sockaddr *)&addr, localhost6.get(AF_INET6)));
