@@ -50,7 +50,7 @@ extern "C" int main()
     assert(eq_case("second test", *testing));
     assert(eq_case(" Is a test", mystr(-10)));
     mystr = "  abc 123 \n  ";
-    assert(eq_case("abc 123", String::strip(mystr.c_mem(), " \n")));
+    assert(eq_case("abc 123", String::strip(mystr.data(), " \n")));
     String::set(buff, sizeof(buff), "this is \"a test\"");
     while(NULL != (tp = String::token(buff, &tokens, " ", "\"\"")) && count < 4)
         array[count++] = tp;
@@ -58,13 +58,13 @@ extern "C" int main()
     assert(eq_case(array[1], "is"));
     assert(eq_case(array[2], "a test"));
 
-    unsigned char core[4] = {0x01, 0x10, 0x2f, 0x45};
+    uint8_t core[4] = {0x01, 0x10, 0x2f, 0x45};
     char hexbuf[12];
 
     assert(String::hexdump(core, hexbuf, "3-1") == 9);
     assert(eq(hexbuf, "01102f-45"));
 
-    unsigned char hcore[4];
+    uint8_t hcore[4];
 
     String::hexpack(hcore, hexbuf, "3-1");
     assert(String::hexdump(hcore, hexbuf, "3-1") == 9);
@@ -95,7 +95,7 @@ extern "C" int main()
     assert(String::check("xxx", 3));
     assert(!String::check("xxxx", 3));
 
-    unsigned char hbuf[2];
+    uint8_t hbuf[2];
     hbuf[0] = 0x23;
     hbuf[1] = 0xa9;
     string_t hex = String::hex(hbuf, 2);
@@ -105,14 +105,14 @@ extern "C" int main()
     strfree(cdup);
 
     stringref_t cvs;
-    charvalues_t cv1 = stringref::create(64);
+    charvalues_t cv1 = stringref<auto_release>::create(64);
     charvalues_t cv2 = cv1;
 
-    snprintf(cv1->get(), cv1->max(), "test %d\n", 1);
-    stringref::expand(&cv1, 64);
+    snprintf(*cv1, cv1->max(), "test %d\n", 1);
+    stringref<auto_release>::expand(&cv1, 64);
     assert(cv1->max() == 128);
     assert(cv2 != cv1);
-    assert(eq(cv1->get(), "test 1\n"));
+    assert(eq(*cv1, "test 1\n"));
     cvs.assign(cv1);
     assert(cvs.copies() == 1);
     assert(eq(cvs, "test 1\n"));
