@@ -36,11 +36,11 @@ static char suffix[80] = {0, 0};
 static void output(bool middle, const char *arg)
 {
     if(is(lines))
-        file::cout << prefix << arg << suffix << "\n";
+        std::cout << prefix << arg << suffix << "\n";
     else if(middle)
-        file::cout << *delim << prefix << arg << suffix;
+        std::cout << *delim << prefix << arg << suffix;
     else
-        file::cout << prefix << arg << suffix;
+        std::cout << prefix << arg << suffix;
 }
 
 static void dirpath(bool middle, String path, bool top = true)
@@ -136,7 +136,7 @@ int main(int argc, char **argv)
     }
 
     if(!lines)
-        file::cout << "\n";
+        std::cout << "\n";
 
     return 0;
 }
