@@ -81,7 +81,8 @@ static void result(const char *path, int code)
     if(!code) {
         if(!path)
             path="-";
-        shell::printf("%s %s\n", *md, path);
+        secure::string sum = *md;
+        shell::printf("%s %s\n", *sum, path);
         return;
     }
 
@@ -97,7 +98,7 @@ static void digest(const char *path = NULL)
 {
     fsys_t fs;
     fsys::fileinfo_t ino;
-    unsigned char buffer[1024];
+    uint8_t buffer[1024];
 
     if(path) {
         int err = fsys::info(path, &ino);
